#ifndef GNCACCOUNTNODE_H
#define GNCACCOUNTNODE_H
///////////////////////////////////////////////////////////////////////
//
// This file is part of the finlink conduit for PalmCoins 
// Copyright (C) 2001 Andrew S. Townley 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA,
// 02111-1307  USA
//
// Name:	GNCAccountNode.h
// Author:	Andrew S. Townley <adz1092 at netscape dot net>
// Created:	Sat May 26 11:42:41 CDT 2001
//
///////////////////////////////////////////////////////////////////////

#include "finlink.h"
#include "gnucash.h"

/**
 * This class implements the behavior of the AccountNode in terms of
 * the data that is required for GNUCash.
 */

class GNCAccountNode : public AccountNode
{
public:
	typedef std::list<AccountNode*>		ChildList;
	typedef std::list<TransactionNode*>	TransList;

	GNCAccountNode();
	GNCAccountNode(Account *a);
	virtual ~GNCAccountNode();

	virtual AccountGroup* getChildAccountGroup();
	virtual std::string getName();
	virtual void setName(const std::string& s);
	virtual std::string getDescription();
	virtual void setDescription(const std::string& s);
	virtual double getBalance();
	virtual AccountType getType();
	virtual void setType(AccountType t);
	virtual AccountNode* getParent();
	virtual void setParent(AccountNode *p);
	virtual void add(AccountNode *node);
	virtual void add(AccountNode *node, bool load);
	virtual void remove(AccountNode *node);
	virtual AccountNode::iterator* children();
	virtual AccountNode* newInstance() { return new GNCAccountNode(); }

	TransactionNode* newTransaction();
	virtual void deleteTransaction(TransactionNode *n);
	virtual void deleteAllTransactions();
	virtual TransactionNode::iterator* transactions();

	// splits
	virtual TransSplit* newSplit();
	virtual void addSplit(TransSplit *s);
	virtual void deleteSplit(TransSplit *s);
	virtual TransSplit::iterator* splits();

	virtual string toString();

protected:
	virtual void setBalance(double d);

protected:
	/** the GNUCash Account structure */
	Account		*acc;

	/** true if we're managing the account data */
	bool		_manage_account;

	/** our parent node */
	AccountNode	*_p;

	/** keep track of our child nodes */
	ChildList	_children;

	TransList	_transactions;

	ErrorTrace	trace;
};

#endif
