///////////////////////////////////////////////////////////////////////
//
// This file is part of the finlink conduit for PalmCoins 
// Copyright (C) 2001 Andrew S. Townley 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA,
// 02111-1307  USA
//
// Name:	GNCTransactionNode.cpp
// Author:	Andrew S. Townley <adz1092 at netscape dot net>
// Created:	Tue Jun  5 15:24:16 CDT 2001
//
///////////////////////////////////////////////////////////////////////

#include "GNCTransactionNode.h"
#include "GNCTransSplit.h"

#define MATURITY_LEVEL	0

template <class T>
class GNCSplitIterator : public Iterator<T>
{
public:
	GNCSplitIterator(Transaction *t)
		: _trans(t), _split(static_cast<Split*>(NULL)), 
		  psplit(static_cast<TransSplit*>(NULL)), _idx(0)
	{
//cout << "Transaction: " << xaccTransGetDescription(t) << endl;
//cout << "Number:      " << xaccTransGetNum(t) << endl;
//cout << "Split count: " << xaccTransCountSplits(t) << endl;
//cerr << "Account:  " << xaccAccountGetName(xaccSplitGetAccount(xaccTransGetSplit(t, 0))) << endl;
	}

	void help()
	{
		_split = xaccTransGetSplit(_trans, _idx);
//cout << "\tAccount: " << xaccAccountGetName(xaccSplitGetAccount(_split)) << endl;
//cout << "\tmemo:    " << xaccSplitGetMemo(_split) << endl;
//cout << "\tvalue:   " << xaccSplitGetValue(_split) << endl;
//cout << "\tbalance: " << xaccSplitGetBalance(_split) << endl;
		if(psplit != static_cast<TransSplit*>(NULL))
			{
			delete psplit;
			}
		psplit = new GNCTransSplit(_split);
	}

	void first()
	{
		_idx = 0;
		help();
	}

	void next()
	{
		_idx++;
		help();
	}

	bool moreElements() const
	{
		if(_idx < xaccTransCountSplits(_trans))
			return true;

		return false;
	}

	T current() const
	{
		return psplit;
	}

	~GNCSplitIterator()
	{
		if(psplit != static_cast<TransSplit*>(NULL))
			{
			delete psplit;
			}
	}

protected:
	Transaction 	*_trans;
	Split		*_split;
	TransSplit	*psplit;
	int		_idx;
};

time_t GNCTransactionNode::getDate()
{
	return xaccTransGetDate(_trans);
}

void GNCTransactionNode::setDate(time_t d)
{
	xaccTransBeginEdit(_trans, 0);
	xaccTransSetDateSecs(_trans, d);
	xaccTransCommitEdit(_trans);
}

std::string GNCTransactionNode::getNumber()
{
	string s(xaccTransGetNum(_trans));
	return s;
}

void GNCTransactionNode::setNumber(const string& s)
{
	xaccTransBeginEdit(_trans, 0);
	xaccTransSetNum(_trans, s.c_str());
	xaccTransCommitEdit(_trans);
}

std::string GNCTransactionNode::getDescription()
{
	string s(xaccTransGetDescription(_trans));
	return s;
}

void GNCTransactionNode::setDescription(const string& s)
{
	xaccTransBeginEdit(_trans, 0);
	xaccTransSetDescription(_trans, s.c_str());
	xaccTransCommitEdit(_trans);
}

TransSplit::iterator* GNCTransactionNode::splits()
{
	return new GNCSplitIterator<TransSplit*>(_trans);
}

static void output_splits(ErrorTrace& trace, Transaction *t)
{
	trace.tprintf(5, "Splits for transaction:  %s", xaccTransGetDescription(t));

	int c = xaccTransCountSplits(t);
	for(int i = 0; i < c; ++i)
		{
		Split *s = xaccTransGetSplit(t, i);
		trace.tprintf(5, "Split #%d", i);
		trace.tprintf(5, "\tGUID:      %d", xaccSplitGetGUID(s));
		trace.tprintf(5, "\tAccount:   %s", xaccAccountGetName(xaccSplitGetAccount(s)));
		trace.tprintf(5, "\tMemo:      %s", xaccSplitGetMemo(s));
		trace.tprintf(5, "\tValue:     %10.2lf", xaccSplitGetValue(s));
		trace.tprintf(5, "\tBalance:   %10.2lf", xaccSplitGetBalance(s));
		}
}

void GNCTransactionNode::addSplit(TransSplit *s)
{
	static const char *m = "addSplit";
	trace.methodStart(m);
	trace.methodArg("s", s);

	GNCTransSplit *gs;

	trace.tprintf(5, "Splits before add...");
	output_splits(trace, _trans);
	if(gs = (GNCTransSplit*)s)
		{
		xaccTransBeginEdit(_trans, 0);
		int sc = xaccTransCountSplits(_trans);
		Split *os = xaccTransGetSplit(_trans, sc - 1);
		if(sc == 1 && (xaccSplitGetValue(os) == 0.0))
			{
			xaccSplitDestroy(os);
			}
		xaccTransAppendSplit(_trans, gs->getGNCSplit());
		xaccTransCommitEdit(_trans);
		}
	trace.tprintf(5, "Splits after add...");
	output_splits(trace, _trans);

	trace.methodReturn(m);
}

void GNCTransactionNode::removeSplit(TransSplit *s)
{
	static const char *m = "removeSplit";
	trace.methodStart(m);
	trace.methodArg("s", s);
	
	GNCTransSplit *gs;

	if(gs = (GNCTransSplit*)s)
		{
		int	sc = xaccTransCountSplits(_trans);
		Split	*rs = gs->getGNCSplit();

		// find see if it exists
		for(int i = 0; i < sc; ++i)
			{
			if(rs == xaccTransGetSplit(_trans, i))
				{
				xaccTransBeginEdit(_trans, 0);
				xaccSplitDestroy(rs);
				xaccTransCommitEdit(_trans);
				break;
				}
			}
		}
	trace.methodReturn(m);
}

/**
 * If we are working with an account, then the values should
 * be for the split belonging to the account.
 */

double GNCTransactionNode::getValue()
{
	Split *s = static_cast<Split*>(NULL);
	if(_acc != static_cast<Account*>(NULL))
		{
		int count = xaccTransCountSplits(_trans);
		for(int i = 0; i < count; ++i)
			{
			s = xaccTransGetSplit(_trans, i);
			if(xaccSplitGetAccount(s) == _acc)
				break;
			}
		}
	else
		{
		s = xaccTransGetSplit(_trans, 0);
		}

	return xaccSplitGetValue(s);
}

double GNCTransactionNode::getBalance()
{
	Split *s = static_cast<Split*>(NULL);
	if(_acc != static_cast<Account*>(NULL))
		{
		int count = xaccTransCountSplits(_trans);
		for(int i = 0; i < count; ++i)
			{
			s = xaccTransGetSplit(_trans, i);
			if(xaccSplitGetAccount(s) == _acc)
				break;
			}
		}
	else
		{
		s = xaccTransGetSplit(_trans, 0);
		}

	return xaccSplitGetBalance(s);
}

GNCTransactionNode::GNCTransactionNode()
	: _trans(static_cast<Transaction*>(NULL)), 
	  _acc(static_cast<Account*>(NULL)), _manage(true)
{
	static const char *m = "GNCTransactionNode";
	trace.init(m, this, MATURITY_LEVEL);
	trace.methodStart(m);
	
	_trans = xaccMallocTransaction();
	
	trace.methodReturn(m);
}

GNCTransactionNode::GNCTransactionNode(Transaction* t)
	: _trans(t), 
	  _acc(static_cast<Account*>(NULL)), _manage(false)
{
	static const char *m = "GNCTransactionNode";
	trace.init(m, this, MATURITY_LEVEL);
	trace.methodStart(m);
	trace.methodArg("t", t);
	trace.methodReturn(m);
}

GNCTransactionNode::GNCTransactionNode(Transaction* t, Account *a)
	: _trans(t), _acc(a), _manage(false)
{
	static const char *m = "GNCTransactionNode";
	trace.init(m, this, MATURITY_LEVEL);
	trace.methodStart(m);
	trace.methodArg("t", t);
	trace.methodArg("a", a);
	trace.methodReturn(m);
}

GNCTransactionNode::~GNCTransactionNode()
{
	static const char *m = "~GNCTransactionNode";
	trace.methodStart(m);

	if(_trans != static_cast<Transaction*>(NULL) && _manage)
		{
//		xaccTransDestroy(_trans);
		}
	
	trace.methodReturn(m);
}

TransSplit* GNCTransactionNode::newSplit()
{
	return new GNCTransSplit();
}

TransactionNode* GNCTransactionNode::newInstance()
{
	return new GNCTransactionNode();
}

string GNCTransactionNode::toString()
{
	char buf[1024];
	strstream s(buf, 1024);
	memset(buf, 0, 1024);

	char b[255];
	time_t d = getDate();
	strftime(b, 255, "%d-%b-%y", localtime(&d));
	s << "[" << getNumber() << "]";
	s << "[" << b << "]";
	s << "[" << getDescription() << "]";
	s << "[" << getValue() << "]";
	s << "[" << getBalance() << "]";

	return s.str();
}
