#ifndef ACCOUNTMANAGER_H
#define ACCOUNTMANAGER_H
///////////////////////////////////////////////////////////////////////
//
// This file is part of the finlink conduit for PalmCoins 
// Copyright (C) 2001 Andrew S. Townley 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA,
// 02111-1307  USA
//
// Name:	AccountManager.h
// Author:	Andrew S. Townley <adz1092 at netscape dot net>
// Created:	Sat Jun  2 13:30:05 CDT 2001
//
///////////////////////////////////////////////////////////////////////

#include "AccountNode.h"
#include "FatalException.h"

class TransactionNode;

/**
 * This class encapsulates all the mechanics of reading and writing
 * account information behind a common facade.  Specific sub-classes
 * should be created to take the appropriate action, depending on how
 * the account information is really stored.
 */

class AccountManager
{
public:
	virtual ~AccountManager() {}

	virtual AccountNode::iterator* accounts() = 0;
	virtual void addAccount(AccountNode *a) = 0;
	virtual void deleteAccount(AccountNode *) = 0;
	virtual void deleteAllAccounts() = 0;
	virtual std::string getUsage() = 0;
	virtual AccountNode* newNode() = 0;
	virtual void readAccounts() throw(FatalException) = 0;
	virtual void writeAccounts() throw(FatalException) = 0;
	virtual AccountNode* getAccount(const std::string& path,
				const std::string& sep = "::");
	virtual void addTransaction(TransactionNode* n) = 0;
	virtual TransactionNode* getTransaction(TransactionNode *n) = 0;

protected:
	AccountManager() {}
	virtual AccountNode *getRoot() = 0;
	virtual AccountNode* findChildByName(AccountNode* start, 
					const std::string& name);
};

#endif
