#ifndef ERRORTRACE_H
#define ERRORTRACE_H
///////////////////////////////////////////////////////////////////////
//
// This file is part of the finlink conduit for PalmCoins 
// Copyright (C) 2001 Andrew S. Townley 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA,
// 02111-1307  USA
//
// Name:	ErrorTrace.h
// Author:	Andrew S. Townley <adz1092 at netscape dot net>
// Created:	Mon Jun 11 18:41:17 CDT 2001
//
///////////////////////////////////////////////////////////////////////

#include "stdcxx.h"

class Traceable
{
public:
	virtual string toString() = 0;
};

/**
 * This class implements generic trace facilities.
 */

class ErrorTrace
{
public:
	ErrorTrace();
	ErrorTrace(const string& className, int maturity = 0);
	virtual ~ErrorTrace();

	virtual void init(const string& className, void *ptr, int maturity = 0);
	virtual void methodStart(const char *name);
	virtual void methodArg(const char *name, const char *arg);
	virtual void methodArg(const char *name, const string& arg);
	virtual void methodArg(const char *name, int arg);
	virtual void methodArg(const char *name, void *arg);
	virtual void methodArg(const char *name, bool arg);
	virtual void methodArg(const char *name, double arg);
	virtual void methodArg(const char *name, Traceable& arg);
	virtual void methodArg(const char *name, Traceable *arg);
	virtual void methodReturn(const char *name);
	virtual void methodReturn(const char *name, const char *arg);
	virtual void methodReturn(const char *name, const string& arg);
//	virtual void methodReturn(const char *name, bool arg);
	virtual void methodReturn(const char *name, void *arg);
	virtual void methodReturn(const char *name, double arg);
	virtual void methodReturn(const char *name, Traceable& arg);
	virtual void methodReturn(const char *name, Traceable* arg);
	virtual void methodThrow(const char *name, const char *except, const char *msg = 0);

	virtual void tprintf(int threshold, const char *fmt, ...);
	virtual bool willTrace(int threshold);
	static void setTraceLevel(int l);
	static void setTraceFile(const string& filename);

	static const char*	TRACE_START_FMT;
	static const char*	TRACE_RETURN_FMT;
	static const char*	TRACE_RETURN_INT_FMT;
	static const char*	TRACE_RETURN_PTR_FMT;
	static const char*	TRACE_RETURN_TRC_FMT;
	static const char*	TRACE_RETURN_STR_FMT;
	static const char*	TRACE_RETURN_DBL_FMT;
	static const char*	TRACE_ARG_STR_FMT;
	static const char*	TRACE_ARG_INT_FMT;
	static const char*	TRACE_ARG_PTR_FMT;
	static const char*	TRACE_ARG_TRC_FMT;
	static const char*	TRACE_ARG_DBL_FMT;
	static const char*	TRACE_THROW_FMT;
	static const char*	TRACE_THROW_MSG_FMT;

protected:
	static string	_traceFileName;
	static FILE*	_traceFile;
	static int	_traceLevel;

	int		_maturity;
	string		_className;
};

#endif
