#ifndef MANAGERCREATOR_H
#define MANAGERCREATOR_H
///////////////////////////////////////////////////////////////////////
//
// This file is part of the finlink conduit for PalmCoins 
// Copyright (C) 2001 Andrew S. Townley 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA,
// 02111-1307  USA
//
// Name:	ManagerCreator.h
// Author:	Andrew S. Townley <adz1092 at netscape dot net>
// Created:	Sat Jun  2 20:10:26 CDT 2001
//
///////////////////////////////////////////////////////////////////////

#include "ManagerFactory.h"

/**
 * This class is required to provide the redirection necessary for
 * successfully using the ManagerFactory.  This class provides an
 * abstract interface for creating objects.  It will be used to create
 * the specific objects in the factory.
 */

class ManagerCreator
{
public:
	virtual AccountManager* createManager(const std::string& s) = 0;
};

/**
 * This class defines a standard creator that will be used for all
 * types of managers.
 */

template <class T>
class GenericManagerCreator : public ManagerCreator
{
public:
	GenericManagerCreator(const std::string& s);
	virtual AccountManager* createManager(const std::string& s);
};

template <class T>
GenericManagerCreator<T>::GenericManagerCreator(const std::string& s)
{
//	cout << "Creator constructor" << endl;
	ManagerFactory* f = ManagerFactory::getInstance();
	f->registerCreator(s, this);
}

template <class T>
AccountManager* GenericManagerCreator<T>::createManager(const std::string& s)
{
	return new T(s);
}

#endif
