#ifndef MODULELOADER_H
#define MODULELOADER_H
///////////////////////////////////////////////////////////////////////
//
// This file is part of the finlink conduit for PalmCoins 
// Copyright (C) 2001 Andrew S. Townley 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA,
// 02111-1307  USA
//
// Name:	ModuleLoader.h
// Author:	Andrew S. Townley <adz1092 at netscape dot net>
// Created:	Tue Jun  5 09:50:02 CDT 2001
//
///////////////////////////////////////////////////////////////////////

#include "stdcxx.h"

/**
 * This singleton class is responsible for handling all the module
 * loading.  We're putting this level of redirection in here for when
 * there are multiple modules and we need to do dynamic loading.
 *
 * NOTE:  right now, we're cheating because the loader explicitly
 * loads the modules for GNUCash and PalmCoins.
 */

class ModuleLoader
{
public:
	/**
	 * This static method insures that we only ever have a single
	 * instance of the module loader.
	 *
	 * @return the shared instance
	 */

	static ModuleLoader* getInstance();

	/**
	 * This method is used to load all the registered modules.
	 */

	virtual void loadModules();

protected:
	ModuleLoader() {}
	
	friend class auto_ptr<ModuleLoader>;
	virtual ~ModuleLoader();

private:
	static ModuleLoader*		_instance;
	static auto_ptr<ModuleLoader>	_del;
};

#endif
