#ifndef TRANSACTIONNODE_H
#define TRANSACTIONNODE_H
///////////////////////////////////////////////////////////////////////
//
// This file is part of the finlink conduit for PalmCoins 
// Copyright (C) 2001 Andrew S. Townley 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA,
// 02111-1307  USA
//
// Name:	TransactionNode.h
// Author:	Andrew S. Townley <adz1092 at netscape dot net>
// Created:	Tue Jun  5 14:55:50 CDT 2001
//
///////////////////////////////////////////////////////////////////////

#include "Iterator.h"
#include "ErrorTrace.h"

class TransactionNode;
class AccountManager;

/**
 * This class represents a transaction split.  The split stores the
 * information about the specific memo, the transfer account, and the
 * value.  Values will be either positive or negative depending on the
 * type of account.
 */

class TransSplit : public Traceable
{
public:	
	typedef Iterator<TransSplit*>		iterator;
	typedef IteratorPtr<TransSplit*>	iterator_ptr;

	virtual std::string getAccount() = 0;
	virtual std::string getMemo() = 0;
	virtual double getValue() = 0;
	virtual void setAccount(const std::string& s) = 0;
	virtual void setMemo(const std::string& s) = 0;
	virtual void setValue(double d) = 0;

	virtual TransSplit* newInstance() = 0;
	virtual void copy(TransSplit *src)
	{
//		setAccount(src->getAccount());
		setMemo(src->getMemo());
		setValue(src->getValue());
	}
	virtual ~TransSplit() {}

protected:
	TransSplit() {}
};

/**
 * This class represents a transaction for a given account.
 * Transactions are simply containers for a number of transaction
 * spilts (where the real data lives).  At the moment, we're going to
 * be using the standard time_t to represent time values (I know that
 * this won't work everywhere in the world, but....)
 */

class TransactionNode : public Traceable
{
public:
	typedef Iterator<TransactionNode*>	iterator;
	typedef IteratorPtr<TransactionNode*>	iterator_ptr;

	virtual time_t getDate() = 0;
	virtual void setDate(time_t) = 0;
	virtual	std::string getNumber() = 0;
	virtual void setNumber(const std::string& s) = 0;
	virtual std::string getDescription() = 0;
	virtual void setDescription(const std::string& s) = 0;
	virtual TransSplit::iterator* splits() = 0;
	virtual void addSplit(TransSplit *s) = 0;
	virtual void removeSplit(TransSplit *s) = 0;
	virtual double getValue() = 0;
	virtual double getBalance() = 0;
	virtual void copy(TransactionNode *src, AccountManager *m);
	virtual bool operator==(TransactionNode& rhs);
	virtual TransactionNode* newInstance() = 0;
	virtual TransSplit* newSplit() = 0;
	virtual ~TransactionNode() {}

protected:
	TransactionNode() {}
};

#endif
