///////////////////////////////////////////////////////////////////////
//
// This file is part of the finlink conduit for PalmCoins 
// Copyright (C) 2001 Andrew S. Townley 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA,
// 02111-1307  USA
//
// Name:	PCAccountManager.cpp
// Author:	Andrew S. Townley <adz1092 at netscape dot net>
// Created:	Sat May 26 10:03:27 CDT 2001
//
///////////////////////////////////////////////////////////////////////

#include "PCAccountManager.h"
#include "ManagerCreator.h"

using namespace std;

char *PCAccountManager::ACCOUNTS_DATABASE	= "PalmCoins-Accounts";

/**
 * This function is used to convert balances represented as double
 * values into the integer value used by the PalmCoins program.
 *
 * @param d the value of the balance as a double
 * @return the converted representation needed by PalmCoins
 */

int PCAccountManager::bal_cvt(double d)
{
	return (int)((d > 0.0 ? d + 0.005 : d - 0.005) * 100);
}

PCAccountManager::PCAccountManager(const string& s)
		throw(FatalException)
	: pilot(static_cast<DLP*>(NULL)), 
	  root(static_cast<PCAccountNode*>(NULL))
{
	string port;

	if(s == "netsync")
		{
		port = ".";
		}
	else if(s.length() > 0)
		{
		// need to strip the port from the name
		string::size_type	pos = s.find_first_of(":");
		string			t = s.substr(0, pos);
		string			port = s.substr(pos + 1);

		pilot = new DLP(port.c_str());

		// open the conduit
		pilot->openConduit();

		// set up the root of our account tree
		root = new PCAccountNode();
		}
}

PCAccountManager::~PCAccountManager()
{
	node_map_t::iterator p;
	for(p = node_map.begin(); p != node_map.end(); ++p)
		{
		if(p->second != static_cast<PCAccountNode*>(NULL))
			{
			delete (p->second);
			}

		node_map.erase(p);
		}

	if(pilot != static_cast<DLP*>(NULL))
		{
		delete pilot;
		}
	if(root != static_cast<PCAccountNode*>(NULL))
		{
		delete root;
		}
}

AccountNode::iterator* PCAccountManager::accounts()
{
	return(root->children());
}

string PCAccountManager::getUsage()
{
	return string("netsync | port:/dev/port");
}

void PCAccountManager::readAccounts() throw(FatalException)
{
	int err = pilot->openDB(0, dlpOpenReadWrite, 
			ACCOUNTS_DATABASE, &accounts_db);

	if(err < 0)
		throw FatalException("error:  received error code %d from pilot", err);

	// read the appinfo
	err = pilot->readAppBlock(accounts_db, 0, &app_info, sizeof(PalmCoinsAppInfo));
	if(err < 0)
		throw FatalException("error:  received error code %d from pilot", err);

	unsigned char buf[600];
	for(int i = 0; ; ++i)
		{
		recordid_t	idx;
		int		attr, cat;
		err = pilot->readRecordByIndex(accounts_db, i, buf, 
				&idx, 0, &attr, &cat);
		if(err < 0)
			{
			break;
			}

		PCAccountNode *n = new PCAccountNode(idx, buf);
		// load the map
		node_map[idx] = n;
		}
	
	// now, we have to build the parents
	node_map_t::iterator p;
	for(p = node_map.begin(); p != node_map.end(); ++p)
		{
		cout << "current account:  " << p->second->getName() << endl;
		unsigned long pid = p->second->getParentId();
		if(pid != 0)
			{
		cout << "current parent:  " << pid << endl;
			node_map[pid]->add(p->second);
			}
		else
			{
		cout << "added to root" << endl;
			root->add(p->second);
			}
		}
	
	pilot->endOfSync(dlpEndCodeNormal);
}

void PCAccountManager::writeAccounts() throw(FatalException)
{
}
