///////////////////////////////////////////////////////////////////////
//
// This file is part of the finlink conduit for PalmCoins 
// Copyright (C) 2001 Andrew S. Townley 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA,
// 02111-1307  USA
//
// Name:	PCAccountNode.cpp
// Author:	Andrew S. Townley <adz1092 at netscape dot net>
// Created:	Sat May 26 10:03:27 CDT 2001
//
///////////////////////////////////////////////////////////////////////

#include "PCAccountNode.h"
#include "PCAccountManager.h"

using namespace std;

/**
 * Default constructor.  Initializes all PalmCoins specific data to
 * reasonable default values.
 */

PCAccountNode::PCAccountNode()
	: _id(0), _balance(0), _exp_balance(0), _credit_limit(0),
	  _parent(0), _kids_balance(0), _kids_exp_balance(0),
	  _depth(0), _child_count(0), _collapsed(0), _credit_account(0),
	  _desc(""), _name(""), _p(static_cast<PCAccountNode*>(NULL))
{
}

/**
 * Reads the account data from the supplied byte stream.
 */

PCAccountNode::PCAccountNode(recordid_t id, const unsigned char *bytes)
	: _p(static_cast<AccountNode*>(NULL))
{
	_id = id;

	unsigned char	*ptr = (unsigned char*)bytes;

	// unpack stuff

	// 1. balance
	_balance = get_slong(ptr);
	ptr += sizeof(long);

	// 2. expected_balance
	_exp_balance = get_slong(ptr);
	ptr += sizeof(long);

	// 3. credit limit
	_credit_limit = get_slong(ptr);
	ptr += sizeof(long);

	// 4. parent id
	_parent = get_long(ptr);
	ptr += sizeof(unsigned long);

	// 5. balance of children
	_kids_balance = get_slong(ptr);
	ptr += sizeof(long);

	// 6. expected balance of children
	_kids_exp_balance = get_slong(ptr);
	ptr += sizeof(long);

	// 7. depth of this node
	_depth = get_byte(ptr);
	ptr += sizeof(unsigned char);

	// 8. number of children
	_child_count = get_byte(ptr);
	ptr += sizeof(unsigned char);

	// 9. collapsed state
	_collapsed = get_byte(ptr);
	ptr += sizeof(unsigned char);

	// 10. credit account
	_credit_account = get_byte(ptr);
	ptr += sizeof(unsigned char);

	// 11. description
	_desc = (char*)ptr;
	ptr += _desc.length() + 1;

	// 12. name
	_name = (char*)ptr;

cout << "loaded account:  " << _name << endl;
cout << "\tid: " << _id << endl;
cout << "\tparent: " << _parent << endl;
}

/**
 * At the moment, the destructor doesn't have to do much...
 */

PCAccountNode::~PCAccountNode()
{
}

string PCAccountNode::getName()
{
	return _name;
}

void PCAccountNode::setName(const string& s)
{
	_name = s;
}

string PCAccountNode::getDescription()
{
	return _desc;
}

void PCAccountNode::setDescription(const string& s)
{
	_desc = s;
}

double PCAccountNode::getBalance()
{
	return _balance / 100;
}

AccountNode::AccountType PCAccountNode::getType()
{
	if(_credit_account)
		return EXPENSE;
	
	return INCOME;
}
	
void PCAccountNode::setType(AccountNode::AccountType t)
{
	switch(t)
		{
		case EXPENSE:
			_credit_account = 1;
			break;
		case INCOME:
			_credit_account = 0;
			break;
		}
}

AccountNode* PCAccountNode::getParent()
{
	return _p;
}

void PCAccountNode::setParent(AccountNode *p)
{
	_p = p;
}

void PCAccountNode::setBalance(double d)
{
	_balance = PCAccountManager::bal_cvt(d);
}

void PCAccountNode::add(AccountNode *node)
{
	_children.push_back(node);
}

void PCAccountNode::remove(AccountNode *node)
{
	ChildList::iterator p = find(_children.begin(), 
			_children.end(), node);
	if(p != _children.end())
		{
		_children.erase(p);
		}
}

AccountNode::iterator* PCAccountNode::children()
{
	return new ListIterator<AccountNode*>(_children);
}

/**
 * This method really does the reverse of the constructor.  First, we
 * need to figure out how big the puppy is and then create the byte
 * stream.
 */

unsigned char* PCAccountNode::getBytes()
{
	int		sz = 0;
	unsigned char	*bs = static_cast<unsigned char*>(NULL);

	sz += sizeof(_balance);
	sz += sizeof(_exp_balance);
	sz += sizeof(_credit_limit);
	sz += sizeof(_parent);
	sz += sizeof(_kids_balance);
	sz += sizeof(_kids_exp_balance);
	sz += sizeof(_depth);
	sz += sizeof(_child_count);
	sz += sizeof(_collapsed);
	sz += sizeof(_credit_account);
	sz += _desc.length() + 1;
	sz += _name.length() + 1;

	bs = new unsigned char[sz];
	
	return bs;
}

