#ifndef PCACCOUNTNODE_H
#define PCACCOUNTNODE_H
///////////////////////////////////////////////////////////////////////
//
// This file is part of the finlink conduit for PalmCoins 
// Copyright (C) 2001 Andrew S. Townley 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA,
// 02111-1307  USA
//
// Name:	PCAccountNode.h
// Author:	Andrew S. Townley <adz1092 at netscape dot net>
// Created:	Sat May 26 09:43:02 CDT 2001
//
///////////////////////////////////////////////////////////////////////

#include "finlink.h"
#include "palmcoins.h"

/**
 * This class implements the behavior of the AccountNode in terms of
 * the data that is required for PalmCoins.
 */

class PCAccountNode : public AccountNode
{
public:
	typedef std::list<AccountNode*> ChildList;

	/**
	 * Default constructor.
	 */

	PCAccountNode();

	/**
	 * This constructor can create an initialized account node
	 * from the specified bytestream.
	 *
	 * @param id the pilot record ID of this node
	 * @param bytes the encoded account data
	 */

	PCAccountNode(recordid_t id, const unsigned char *bytes);
	virtual ~PCAccountNode();

	virtual unsigned long getId() { return _id; }
	virtual std::string getName();
	virtual void setName(const std::string& s);
	virtual std::string getDescription();
	virtual void setDescription(const std::string& s);
	virtual double getBalance();
	virtual AccountType getType();
	virtual void setType(AccountType t);
	virtual AccountNode* getParent();
	virtual unsigned long getParentId() { return _parent; }
	virtual void setParent(AccountNode *p);
	virtual void add(AccountNode *node);
	virtual void remove(AccountNode *node);
	virtual iterator* children();
	virtual AccountNode* newInstance() { return new PCAccountNode(); }

	/**
	 * This method is used to "flatten" the account node to the
	 * device-dependent representation.  The caller is responsible
	 * for freeing the byte stream returned.
	 */

	unsigned char* getBytes();

protected:
	virtual void setBalance(double d);

protected:
	/** our record id */
	unsigned long	_id;

	/** the balance of the account */
	long		_balance;

	/** the expected balance for a given date */
	long		_exp_balance;

	/** the credit limit/overdraft value */
	long		_credit_limit;

	/** reference to the parent's uid */
	unsigned int	_parent;

	/** the total balance of all child accounts */
	long		_kids_balance;

	/** the total expected balance of all child accounts */
	long		_kids_exp_balance;

	/** the depth in the tree of this account */
	char		_depth;

	/** the number of children of this account */
	char		_child_count;

	/** indicates the expand/collapse state of the account */
	char		_collapsed;

	/** indicates an income or expense account */
	char		_credit_account;

	/** the account description */
	std::string	_desc;

	/** the account name */
	std::string	_name;
	
	/** our parent node */
	AccountNode	*_p;

	/** keep track of our child nodes */
	ChildList	_children;
};

#endif

