///////////////////////////////////////////////////////////////////////
//
// This file is part of the finlink conduit for PalmCoins 
// Copyright (C) 2001 Andrew S. Townley 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA,
// 02111-1307  USA
//
// Name:	AccountManager.cpp
// Author:	Andrew S. Townley <adz1092 at netscape dot net>
// Created:	Wed Jun  6 14:17:09 CDT 2001
//
///////////////////////////////////////////////////////////////////////

#include "AccountManager.h"
#include "TransactionNode.h"

/**
 * This method is used to find the specific account given the
 * path and the path separator.
 *
 * @param path the path to the desired account (must start
 * 	with path sep)
 * @param sep the path separator 
 * @return a pointer to the desired account or NULL if the
 * 	account doesn't exist
 */

AccountNode* AccountManager::getAccount(const std::string& path,
			const std::string& sep)
{
//cout << "Looking for node: " << path << endl;
	AccountNode *node = static_cast<AccountNode*>(NULL);
	std::string 		pc = path.substr(sep.length());
	std::string::size_type 	pos = pc.find_first_of(sep);
	std::string::size_type	spos = sep.length();
	pc = pc.substr(0, pos);
//cout << "pos = " << pos << endl;
//cout << "pc =  " << pc << endl;
	node = findChildByName(getRoot(), pc);

	while(pos != std::string::npos)
		{
		pos += spos;
		spos = pos + sep.length();
		pc = path.substr(spos);
//cout << "pc =  " << pc << endl;
		pos = pc.find_first_of(sep);
		pc = pc.substr(0, pos);
//cout << "pos = " << pos << endl;
//cout << "pc =  " << pc << endl;
		node = findChildByName(node, pc);
		}
//if(node != 0)
//	cout << "Found node:  " << node->getPath(sep) << endl;

	return node;
}

AccountNode* AccountManager::findChildByName(AccountNode* start, 
					const std::string& name)
{
//cout << "findChildByName:  start->name: " << start->getName() << endl;
//cout << "findChildByName:  name: " << name << endl;
	AccountNode* node = static_cast<AccountNode*>(NULL);
	AccountNode::iterator_ptr ip(start->children());
	for(ip->first(); ip->moreElements(); ip->next())
		{
		AccountNode *c = ip->current();
		if(c->getName() == name)
			{
			node = c;
			break;
			}
		}

	return node;
}
