///////////////////////////////////////////////////////////////////////
//
// This file is part of the finlink conduit for PalmCoins 
// Copyright (C) 2001 Andrew S. Townley 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA,
// 02111-1307  USA
//
// Name:	ManagerFactory.cpp
// Author:	Andrew S. Townley <adz1092 at netscape dot net>
// Created:	Sat Jun  2 21:01:28 CDT 2001
//
///////////////////////////////////////////////////////////////////////

#include "ManagerCreator.h"

using namespace std;

ManagerFactory* ManagerFactory::_instance = static_cast<ManagerFactory*>(NULL);
auto_ptr<ManagerFactory> ManagerFactory::_del;
ManagerFactory::string_list ManagerFactory::_list;
ManagerFactory::creator_map ManagerFactory::_map;

ManagerFactory* ManagerFactory::getInstance()
{
	if(_instance == static_cast<ManagerFactory*>(NULL))
		{
//cout << "created new instance of manager factory" << endl;
		_instance = new ManagerFactory();
		_del.reset(_instance);
		}

	return _instance;
}

AccountManager* ManagerFactory::createManager(const string& uri)
		throw(FatalException)
{
	string::size_type	pos = uri.find_first_of(":");
	string			p = uri.substr(0, pos);
	string			q;

	if(pos != string::npos)
		{
		q = uri.substr(pos + 1);
		}
	else
		{
		q = "";
		}

	ManagerCreator* c = _map[p];
	if(c == static_cast<ManagerCreator*>(NULL))
		{
		return static_cast<AccountManager*>(NULL);
		}

	return c->createManager(q);
}

ManagerFactory::iterator* ManagerFactory::elements()
{
	return new ListIterator<string>(_list);
}

void ManagerFactory::registerCreator(const string& s,
					ManagerCreator *c)
{
	_list.push_back(s);
	_map[s] = c;
}

