/**	
	$Revision: 1.9 $
	Last updated: $Date: 2001/07/24 09:02:43 $

	Copyright (C) 2000-2001Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
	$Log: AccountsDB.cpp,v $
	Revision 1.9  2001/07/24 09:02:43  dizzy
	Renamed some #defines
	
	Revision 1.8  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.7  2001/07/23 10:48:33  chaese
	Pack bools into bitfields
	
	Revision 1.6  2001/06/29 21:32:44  dizzy
	Eliminated bug discovered with os4 (releasing record which was not busy)
	
	Revision 1.5  2001/06/29 16:17:26  dizzy
	Expanded Carsten's #define to handle strings and transformed it into a template fct. This should help tidying up packing operations a bit
	(provided it works as it should)
	
	Revision 1.4  2001/06/29 00:24:57  chaese
	Rewrote how total balances are tracked. This should eliminate all problems
	that were associated with the old way (i.e. balances not being recalculated
	when they should have been etc.)
	
	Revision 1.3  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.2  2001/06/27 03:15:12  chaese
	Resurrected category selector in main form as type group selector
	
	Revision 1.1  2001/06/25 10:42:35  dizzy
	Change file extensions
	
	Revision 1.41  2001/06/21 11:04:23  chaese
	* initial support for account types
	* fixed some obscure bugs that materialized on a PalmOS 3.5 debug rom
	(see related message on palmcoins-misc for details)
	
	Revision 1.40  2001/06/17 05:23:32  chaese
	*** CAUTION: DATABASE STRUCTURE HAS CHANGED YET AGAIN ***
	I generally don't like 'cross-logging', but too many changes have accumulated
	to keep them separated.
	1. completely eliminated account categories
	2. introduced currently unused type field in the account record structure
	(This will become what the user sees as 'category selector'. Coming soon!)
	3. completely eliminated protected accounts (mostly because #1 made it so much
	harder to locate the "Misc" account in the Expense category:) Those should be
	replaced with Pref. settings for default Income and Expense accounts anyway.
	4. changes to user interface; some as mere consequences of 1. (e.g.
	Account chooser), some for usability reasons (Transaction form).
	5. bugfixes and minor improvements.
	
	Revision 1.39  2001/06/14 04:19:49  chaese
	corrected/optimized preferences handling
	
	Revision 1.38  2001/06/14 00:52:05  chaese
	show account names with as much of their full path name as possible
	
	Revision 1.37  2001/06/11 10:57:53  chaese
	bugfix in deleteRecord
	
	Revision 1.36  2001/06/10 04:05:41  chaese
	basic infrastructure for split transactions (somewhat tested, but not
	guaranteed to be bug free)
	CAUTION! Database format has changed! Expect nastiness when you run the
	new version on an old Database.
*/

#include "AccountsDB.h"

#include <DateTime.h>
#include <FeatureMgr.h>

#include "TransactionsDB.h"
#include "SplitsDB.h"
#include "AccountsTable.h"
#include "CoinsPref.h"

#include "FreeCoinsRsc.h"

AccountsDB* accounts_db;

Int32	AccountsDB::getBalance(UInt32 uid)
{
	accountStructure	as;	
	
	getUnpackedRecord(uid, &as);
	return as.balance;
}

Int32	AccountsDB::getExpectedBalance(UInt32 uid)
{
	accountStructure		 	as;	
	
	getUnpackedRecord(uid, &as);
	return as.expected_balance;
}

void AccountsDB::unpackRecord (accountStructure* account, packedAccountStructure* packedAccount)
{
	Char*	s = packedAccount->description;
	
	account->balance = packedAccount->balance;
	account->expected_balance = packedAccount->expected_balance;
	account->credit_limit = packedAccount->credit_limit;
	account->parent_uid = packedAccount->parent_uid;
	account->children_total = packedAccount->children_total;
	account->children_expected_total = packedAccount->children_expected_total;
	account->acc_type = (AccountType)packedAccount->acc_type;
	account->depth = packedAccount->depth;
	account->children_no = packedAccount->children_no;
	account->collapsed = packedAccount->flags.collapsed;	
	account->credit_account = packedAccount->flags.credit_account;
			
	account->description = s;
	s += StrLen(s)+1;
	account->acname = s;
}

void AccountsDB::packRecord (accountStructure* account, MemHandle accountDBEntry)
{
	// figure out necessary size
	UInt32	length = 0;
	MemPtr	s;
	UInt32	offset = 0;
	AccountFlags flags;
	
	//total length is the size of the packed struct + size of the strings
	length = sizeof(packedAccountStructure)
			+ StrLen(account->description) + StrLen(account->acname) + 1;

	flags.collapsed = account->collapsed;
	flags.credit_account = account->credit_account;
	flags.reserved = 0;

	if (MemHandleResize(accountDBEntry, length) == 0) {
		// copy the fields
		s = MemHandleLock(accountDBEntry);
		writeAndAdvance(s,offset,&account->balance);
		writeAndAdvance(s,offset,&account->expected_balance);
		writeAndAdvance(s,offset,&account->credit_limit);
		writeAndAdvance(s,offset,&account->parent_uid);
		writeAndAdvance(s,offset,&account->children_total);
		writeAndAdvance(s,offset,&account->children_expected_total);
		writeAndAdvance(s,offset,&flags);
		writeAndAdvance(s,offset,&account->acc_type);
		writeAndAdvance(s,offset,&account->depth);
		writeAndAdvance(s,offset,&account->children_no);
		writeAndAdvance(s,offset,account->description);
		writeAndAdvance(s,offset,account->acname);
	
		MemHandleUnlock(accountDBEntry);
	}
	
}

AccountsDB::AccountsDB() : Database <accountStructure, packedAccountStructure> (ACCOUNTSDB_TYPE, FREE_COINS_CREATOR, ACCOUNTSDB_NAME, FREE_COINS_CATEGORIES)
{
	totals.total=totals.expected_total=0;
	current_cat=CategoryFind(ref,"Visible");
}

AccountsDB::AccountsDB(DmOpenRef db_open_ref) : Database <accountStructure, packedAccountStructure> (db_open_ref)
{
	totals.total=totals.expected_total=0;
	current_cat=CategoryFind(ref,"Visible");
}

UInt32 AccountsDB::createAccount(char *name, UInt32 parent_uid,
	 AccountType acc_type)
{
	accountStructure as;
	UInt16 index;
	UInt32 uid;

	MemSet(&as, sizeof(accountStructure), 0);
	as.parent_uid = parent_uid;
	if (parent_uid) as.depth = 1;
	as.acname = name;
	as.description = "";
	as.acc_type = acc_type;
	index = newRecord(&as);
	uid = getRecordUid(index);
	return uid;
}
	
void	AccountsDB::initializeAccounts()
{
	UInt16 visible_cat;
	UInt32 uid, uid2;

	WinDrawChars("Creating sample accounts...",27,0,0);
	
	current_cat=visible_cat=CategoryFind(ref,"Visible");

    // bank accounts
	uid = createAccount("Bank",0, BANK);
	createAccount("Checking", uid, BANK);
	createAccount("Saving", uid, BANK);

	// credit card 'category' empty, don't want to endorse anybody :)
	uid = createAccount("Credit Cards",0, CREDIT);
		
	// income accounts
	uid = createAccount("Income",0, INCOME);
	createAccount("Misc", uid, INCOME);
	createAccount("Interest", uid, INCOME);
	createAccount("Salary", uid, INCOME);
	createAccount("Gift", uid, INCOME);
	createAccount("Allowance", uid, INCOME);

	// expense accounts
	uid = createAccount("Expense",0, EXPENSE);
	createAccount("Misc", uid, EXPENSE);
	uid2 = createAccount("Utilities", uid, EXPENSE);
	createAccount("Gas", uid2, EXPENSE);
	createAccount("TV", uid2, EXPENSE);
	createAccount("Phone", uid2, EXPENSE);
	createAccount("Electricity", uid2, EXPENSE);
	createAccount("Rent", uid2, EXPENSE);

	uid2 = createAccount("Travel", uid, EXPENSE);
	createAccount("Land", uid2, EXPENSE);
	createAccount("Sea", uid2, EXPENSE);
	createAccount("Air", uid2, EXPENSE);
	createAccount("Meals", uid2, EXPENSE);

	uid2 = createAccount("Food", uid, EXPENSE);
	createAccount("Dining", uid2, EXPENSE);

	createAccount("Clothing", uid, EXPENSE);
	createAccount("Gifts", uid, EXPENSE);
	createAccount("Computer", uid, EXPENSE);
	createAccount("Books", uid, EXPENSE);
	createAccount("Entertainment", uid, EXPENSE);

	WinEraseChars("Creating sample accounts...",27,0,0);	
}

static UInt32 type_masks[7] = {15, 3, 12, 1, 2,  4, 8};

void AccountsDB::incrementBalance(UInt32 uid, Int32 clr_amt, Int32 exp_amt,
	Boolean is_parent /*= false*/)
{
	accountStructure	a;
	MemHandle			h;
	UInt32 type_mask;

	h = getUnpackedRecord(uid,&a);

	// if the account is visible, update grand total
	if (preferences.get()->visible_acc_types >= 7) {
		type_mask = 0xffff;
	}
	else {
		type_mask = type_masks[preferences.get()->visible_acc_types];
	}
	if ((1L<<((UInt8)a.acc_type)) & type_mask) {
		totals.total += clr_amt;
		totals.expected_total += exp_amt;
	}

	if (is_parent) {
		a.children_total += clr_amt;
		a.children_expected_total += exp_amt;
	}
	else {
		a.balance += clr_amt;
		a.expected_balance += exp_amt;
	}
	packRecord(&a,h);

	while (a.depth!=0) {
		uid = a.parent_uid;
		h=getUnpackedRecord(uid,&a);
		a.children_total += clr_amt;
		a.children_expected_total += exp_amt;
		packRecord(&a,h);
	}
}

void AccountsDB::updateBalance(UInt32 uid,Int32 newbal, char* payee /*= NULL*/)
{
	accountStructure		a;
	transactionStructure	t;
	splitStructure			s;
	UInt16 index;

	getUnpackedRecord(uid,&a);

	if (a.balance == newbal)
		return;

	if (payee) t.payee = payee;
	else t.payee="Balance update";

	DateSecondsToDate(TimGetSeconds(),& t.date);
	t.method=pm_transfer;
	t.num="";	
	t.note="";	
	t.num_splits = 1;
	index = transactions_db->newRecord(&t,true);	//cleared by default

	s.trans_uid = transactions_db->getRecordUid(index);
	s.acc_uid = uid;
	s.amount = newbal-a.balance;
	s.line_no = 1;
	splits_db->newRecord(&s, true);
}	


UInt32	AccountsDB::findAccountByName(char* n, UInt16 categ)
{
	UInt16				recordNo;
	UInt32				uniqueID;
	accountStructure	as;

  	for(recordNo=0;;recordNo++) {
		if(!DmQueryNextInCategory(ref,&recordNo,categ)) return 0;		
		DmRecordInfo(ref,recordNo,NULL,&uniqueID,NULL);

		getUnpackedRecord(uniqueID,&as);

		if(!StrCaselessCompare(n,as.acname)) return uniqueID;
	}	
}

void AccountsDB::getFullNameTruncLeftToFit(UInt32 acc, Int16 width,
	Char *dest)
{
	UInt32 uids[10];
	accountStructure a;
	Char* s;
	Int16 l, i;
	Char ellipsis;

    l = 0;
	i = 0;
	do {
		getUnpackedRecord(acc, &a);
		l += StrLen(a.acname);
		uids[i] = acc;
		acc = a.parent_uid;
		i++;
	} while (a.depth>0 && i<10);
	l += i;
	i -= 2;
	s = (Char*)MemPtrNew(l);
	StrCopy(s, a.acname);
	while(i>=0) {
		getUnpackedRecord(uids[i], &a);
		StrCat(s,":");
		StrCat(s, a.acname); 
		i--;
	}

	for (i=l-1; i>=0; i--) {
		width -= FntCharWidth(s[i]);
		if (width<0) break;
	}
	if (width<0) {
		ChrHorizEllipsis(&ellipsis);
		width -= FntCharWidth(ellipsis);
		while (width<0) {
			width += FntCharWidth(s[i]);
			i++;
		}
		s[i] = ellipsis;
		MemMove(s, s+i, l-i);
	}
	StrCopy(dest, s);
	MemPtrFree(s);
}

void	AccountsDB::getAccountName(UInt32 acc, Char* buf)
{
	accountStructure as;
	
	getUnpackedRecord(acc,&as);
	StrCopy(buf,as.acname);
}

Boolean	AccountsDB::isDisplayable(UInt16 acc)
{
	accountStructure as;
	
	getUnpackedRecord(acc,&as);
	if(as.depth == 0) return true; //all root accounts are visible
	for(UInt32 parent = as.parent_uid; as.depth!=0; parent = as.parent_uid)
	{
		getUnpackedRecord(parent,&as);
		if(as.collapsed) {
			return false;
		}
	}
	return true;
}


UInt16	AccountsDB::newRecord(accountStructure* s)
{
	accountStructure	parent;
	MemHandle			h,p,parent_h;
	UInt16						index;
	Int32							bal;
	
	//clear the balance since it is maintained via the transaction
	bal=s->balance;
	s->balance=s->expected_balance=0;
	s->collapsed=true;
	if(s->depth!=0) { //update parent information
		parent_h=getUnpackedRecord(s->parent_uid,&parent);
		parent.children_no++;
		//overwrite the depth
		s->depth=parent.depth+1;
		
		packRecord(&parent,parent_h);
	}

	//set size to 1 - will be resized by packrecord
	h=DmNewHandle(ref,1);
	
	ErrFatalDisplayIf(!h,"Creating new account failed. No space left on the device?");

	packRecord(s,h);

	p = (MemHandle) MemHandleLock(h);
	index=DmFindSortPosition(ref,p, 0, (DmComparF *) compareRecords, 0);
	MemPtrUnlock(p);
	
	DmAttachRecord(ref,&index,h,0); //automatically sets the dirty bit

	if(current_cat!=dmAllCategories) setRecordCategory(index,current_cat);

	if (bal!=0) {
		UInt32 acc_uid;
		DmRecordInfo(ref,index,NULL,&acc_uid,NULL);
		updateBalance(acc_uid, bal, "Account opened");
	}
	
	return index;	
}

void	AccountsDB::deleteRecord(UInt16	id)
{
	UInt16					recordNo;
	UInt32					trans_uid,acc_uid;
	UInt16					index, count;
	transactionStructure	ts;
    accountStructure		a;
	splitStructure			s;
    MemHandle				h;
 	
	//sanity check: if deleting the account set as default, unset the default account
	if(getRecordUid(id) == preferences.get()->def_acc) {
		preferences.get()->def_acc = 0;
	}
	
	acc_uid = getRecordUid(id);

	// if necessary, notify parent that we're gone
	getUnpackedRecord(acc_uid, &a);
	if (a.depth!=0) {
		UInt32 p_uid = a.parent_uid;
		h = getUnpackedRecord(p_uid,&a);
		a.children_no--;
		packRecord(&a,h);
	}

	// in all transactions delete all splits that mention this account
	for(recordNo=0;;recordNo++) {
		if(!DmQueryNextInCategory(transactions_db->getRef(),&recordNo,dmAllCategories)) break;				
		DmRecordInfo(transactions_db->getRef(),recordNo,NULL,&trans_uid,NULL);
		h = transactions_db->getUnpackedRecord(trans_uid,&ts);

		count = ts.num_splits;
		for (index=splits_db->locateFirstSplit(trans_uid);count>0;
			index++,count--){
			splits_db->getUnpackedRecord(index, &s);
			if (s.acc_uid==acc_uid) {
				splits_db->deleteRecord(index);
				index--;
				ts.num_splits--;
			}
		}
		transactions_db->packRecord(&ts,h);
		if (ts.num_splits==0) {
			transactions_db->deleteRecord(recordNo);
			recordNo--;
		}
	}

	Database <accountStructure, packedAccountStructure>::deleteRecord(id);
}


Int16	AccountsDB::compareRecords(packedAccountStructure* rec1,packedAccountStructure* rec2,Int16 other,SortRecordInfoPtr sr1,SortRecordInfoPtr sr2,MemHandle app_info)
{
	accountStructure ac1,ac2;
	accounts_db->unpackRecord(&ac1,rec1);
	accounts_db->unpackRecord(&ac2,rec2);
	Int8 result = 0, tie_breaker = 0;

	// if the procedure below results in a tie, then the two accounts were
	// on the same branch. In that case, sort the higher one first, the deeper
    // one second.
	tie_breaker = ac1.depth - ac2.depth;

	// trace the deeper node back to the same level as the higher node
	while (ac1.depth != ac2.depth) {
		if (ac1.depth > ac2.depth) {
			accounts_db->getUnpackedRecord(ac1.parent_uid,&ac1);
		}
		else {
			accounts_db->getUnpackedRecord(ac2.parent_uid,&ac2);
		}
	}

	// trace both back until they have the same parent
    while (ac1.depth!=0 && ac1.parent_uid != ac2.parent_uid) {
		accounts_db->getUnpackedRecord(ac1.parent_uid,&ac1);
		accounts_db->getUnpackedRecord(ac2.parent_uid,&ac2);
	}

	// now they're comparable.
	result = StrCaselessCompare(ac1.acname, ac2.acname);

	if (result==0)
		result = tie_breaker;
	return result;
}

void AccountsDB::updateVisibility()
{
	UInt16 visible = CategoryFind(ref, "Visible");
	UInt16 invisible = CategoryFind(ref, "Invisible");
	UInt32 type_mask = 0;
	accountStructure as;
	UInt16				recordNo;

	if (preferences.get()->visible_acc_types >= 7) {
		type_mask = 0xffff;
	}
	else {
		type_mask = type_masks[preferences.get()->visible_acc_types];
	}

	totals.total = totals.expected_total = 0;
  	for(recordNo=0;;recordNo++) {
		if(!DmQueryNextInCategory(ref,&recordNo,dmAllCategories))
			break;
		getUnpackedRecord(recordNo,&as);
		if ((1L<<((UInt8)as.acc_type)) & type_mask) {
			setRecordCategory(recordNo, visible);
			totals.total += as.balance;
			totals.expected_total += as.expected_balance;
		}
		else {
			setRecordCategory(recordNo, invisible);
		}
	}
}
