/**	
	$Revision: 1.10 $
	Last updated: $Date: 2001/07/24 01:00:04 $

	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
	$Log: AccountsForm.cpp,v $
	Revision 1.10  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.9  2001/07/15 11:38:12  dizzy
	Added interface for purging transactions
	
	Revision 1.8  2001/07/15 05:17:07  chaese
	* display only one balance (expected or cleared) in account table, popup
	  trigger lets user choose.
	* New function PopupListSetSelection to eliminate code duplication
	
	Revision 1.7  2001/07/09 02:29:39  chaese
	implement splits GUI & other small improvements along the way. The bulk of
	the splits GUI is complete, small data integrity checks are coming soon
	(need to prevent multiple splits for the same account within one transaction
	and stuff like that...)
	
	Revision 1.6  2001/06/29 00:24:57  chaese
	Rewrote how total balances are tracked. This should eliminate all problems
	that were associated with the old way (i.e. balances not being recalculated
	when they should have been etc.)
	
	Revision 1.5  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.4  2001/06/27 03:15:12  chaese
	Resurrected category selector in main form as type group selector
	
	Revision 1.3  2001/06/26 23:30:37  dizzy
	Forms are now classes rather then namespaces. This should enable future code optimisations
	
	Revision 1.2  2001/06/26 01:53:22  chaese
	improved redraw after collapse/expand: redraw table only
	
	Revision 1.1  2001/06/25 10:42:35  dizzy
	Change file extensions
	
	Revision 1.38  2001/06/23 04:51:48  chaese
	fixed bug #435294 and minor cleanups
	
	Revision 1.37  2001/06/21 11:04:23  chaese
	* initial support for account types
	* fixed some obscure bugs that materialized on a PalmOS 3.5 debug rom
	(see related message on palmcoins-misc for details)
	
	Revision 1.36  2001/06/17 05:23:32  chaese
	*** CAUTION: DATABASE STRUCTURE HAS CHANGED YET AGAIN ***
	I generally don't like 'cross-logging', but too many changes have accumulated
	to keep them separated.
	1. completely eliminated account categories
	2. introduced currently unused type field in the account record structure
	(This will become what the user sees as 'category selector'. Coming soon!)
	3. completely eliminated protected accounts (mostly because #1 made it so much
	harder to locate the "Misc" account in the Expense category:) Those should be
	replaced with Pref. settings for default Income and Expense accounts anyway.
	4. changes to user interface; some as mere consequences of 1. (e.g.
	Account chooser), some for usability reasons (Transaction form).
	5. bugfixes and minor improvements.
*/

#include "AccountsForm.h"
#include "AccountsDB.h"
#include "CoinsPref.h"

#include "FreeCoinsRsc.h"

void	AccountsForm::displayTotals()
{
	Char* 			s;
	RectangleType	rp;
	Int32 amount;

	if (preferences.get()->disp_expected_bal)
        amount = accounts_db->totals.expected_total;
	else
		amount = accounts_db->totals.total;

	s=Pence2String(amount);
	RctSetRectangle(&rp, 111, 129, 50, 11);
	WinEraseRectangle(&rp,0);
	DrawCharsRightAlign(s,&rp);
	MemPtrFree(s);
	
	//add some lines at the bottom
	WinDrawLine(0,128,160,128);
	WinDrawLine(0,141,160,141);	
}

void AccountsForm::updatePopupTrigger(void)
{
	PopupListSetSelection(ACC_TYPE_LIST, ACC_TYPE_TRIGGER,
		preferences.get()->visible_acc_types);
	PopupListSetSelection(ACC_TOTAL_LIST, ACC_TOTAL_TRIGGER,
		preferences.get()->disp_expected_bal?0:1);
}

Boolean AccountsForm::handleEvent( EventPtr event )
{
	FormPtr		f;//about form
	Boolean 	handled = true;
	ControlPtr	ptr;
	
	// Get the form pointer
	frm = FrmGetActiveForm();

	// Key Events
	switch(event->eType) {

	case penDownEvent:
	case penUpEvent:
		// it seems that tblSelect doesn't set the pen coordinates in the
		// event structure, therefore I cache the coordinates from the pen
		// events to see where exactly the name column was tapped.
		last_screenX = event->screenX;
		last_screenY = event->screenY;
		handled = false;
		break;

	// Form Open
	case frmOpenEvent:
		//first initialise the table
		ac_tbl.initTable();
	
		if(!osVerGe(palmos_3_5)) FrmEraseForm(frm);
		//draw form
		FrmDrawForm(frm);
		updatePopupTrigger();
	
		ac_tbl.showScrollArrows();
	
		//display the running totals
		displayTotals();
		break;

	case keyDownEvent:
		switch( event->data.keyDown.chr ) {
			case pageUpChr:			// Scroll up key presed?
				//link the hard button functionality to the soft up/down controls
				ptr = (ControlPtr) GetObjectPtr(ACC_PGUP);
				CtlHitControl(ptr);
				break;								
			case pageDownChr:		// Scroll down key presed?						
				ptr = (ControlPtr) GetObjectPtr(ACC_PGDN);
				CtlHitControl(ptr);
				break;
			default:
  				handled = false;
		};
		break;
	
	// Controls
	case ctlSelectEvent:
		switch( event->data.ctlSelect.controlID ) {
			case ACC_NEW_TRANS:
				handled = true;
				if(preferences.get()->def_acc == 0)
					FrmCustomAlert(GENERIC_ERROR,
						"You need to set a default account in the application "
						"preferences in order to use this feature", NULL,NULL);
				else {
					accounts_db->selectRecord(preferences.get()->def_acc);
					edit_trans_form->setRecordIsNew(true);
					edit_trans_form->setTransIsSched(false);
					edit_trans_form->setCallerFormId(ACCOUNTS_MAIN_FORM);
					FrmPopupForm(EDIT_TRANS_FORM);			
				}
				break;
			case ACC_PGUP:
				if(ac_tbl.pageUp()) FrmUpdateForm(ACCOUNTS_MAIN_FORM,1);
				break;								
			case ACC_PGDN:		// Scroll down control presed?	
				if(ac_tbl.pageDown()) FrmUpdateForm(ACCOUNTS_MAIN_FORM,1);
				break;
				
			default:
				handled = false;
		}
		break;
	
	// a item in the table was selected
	// should probably be moved in the table files
	case tblSelectEvent:		
		//save the record on the current row into DetailRecord variable
		ac_tbl.unhighlight();
		if(event->data.tblSelect.column==AccountsTable::name_col) {
			// tap in the name column: check if it's in the indented area
			accountStructure a;
			MemHandle handle = accounts_db->getUnpackedRecord(TblGetRowID(
				(TablePtr) GetObjectPtr(ACC_TABLE),
				event->data.tblSelect.row), &a);
			if (last_screenX <= ACC_TBL_INDENT*(a.depth+2)) {
				// yes: check for children
				if (a.children_no > 0) {
	                // yes: toggle collapsed state
		  			a.collapsed = !a.collapsed;
		  			accounts_db->packRecord(&a, handle);
					FrmUpdateForm(ACCOUNTS_MAIN_FORM,1);
					break;
		  		}
			}
		}
		
		accounts_db->selectRecord( TblGetRowID((TablePtr) GetObjectPtr(ACC_TABLE),event->data.tblSelect.row));

		FrmGotoForm(TRANSACTIONS_FORM);
		break;
			
	// popup list
	case popSelectEvent:
		switch(event->data.popSelect.controlID) {
		case ACC_TYPE_TRIGGER:
			if (event->data.popSelect.selection !=
				event->data.popSelect.priorSelection) {
				preferences.get()->visible_acc_types =
					event->data.popSelect.selection;
				updatePopupTrigger();
				accounts_db->updateVisibility();
				ac_tbl.top_visible_item = 0;
				FrmUpdateForm(ACCOUNTS_MAIN_FORM,3);
			}
			break;
		case ACC_TOTAL_TRIGGER:
			preferences.get()->disp_expected_bal = 
				(event->data.popSelect.selection==0);
			FrmUpdateForm(ACCOUNTS_MAIN_FORM,3);
			break;
		default:
			handled = false;
		}
		break;

	// Menus
	case menuEvent:
		switch (event->data.menu.itemID) {
			case ACC_MENU_NEW:
				edit_account_form->setRecordIsNew(true);
				if(!osVerGe(palmos_3_5)) FrmEraseForm(frm);
				FrmPopupForm(EDIT_ACC_FORM);
				break;
			case ACC_MENU_PREF:
				FrmPopupForm(PREF_FORM);
				break;
			case ACC_MENU_PURGE:
				FrmPopupForm(PURGE_FORM);
				break;
			case ACC_MENU_ABOUT:
			    f = FrmInitForm(ABOUT_FORM);
    
				FrmSetActiveForm(f);
    			FrmDoDialog(f);
        
			    if (frm) {
			        FrmSetActiveForm(frm);
				    FrmDeleteForm(f);
				}
				break;
			case ACC_MENU_HELP:
				FrmHelp(ACCOUNTS_HELP);
				break;
				
			default:
				handled = false;
		}
		break;
	
	// Form Close
	case frmCloseEvent:
		ac_tbl.hideScrollArrows();
		handled = false;
		break;
	
	// Form Update
	case frmUpdateEvent:
		ac_tbl.loadData();

		if(event->data.frmUpdate.updateCode == 0 ||
			event->data.frmUpdate.updateCode == frmRedrawUpdateCode) {
			if(!osVerGe(palmos_3_5)) FrmEraseForm(frm);
		
			//draw form
			FrmDrawForm(frm);
			updatePopupTrigger();
			//display the running totals
			displayTotals();
		}
	
		if(event->data.frmUpdate.updateCode & 1) {
			TblMarkTableInvalid((TablePtr)GetObjectPtr(ACC_TABLE));
			TblRedrawTable((TablePtr)GetObjectPtr(ACC_TABLE));
		}

		if(event->data.frmUpdate.updateCode & 2) {
			updatePopupTrigger();
			displayTotals();
		}

		ac_tbl.showScrollArrows();	
		break;
	
	// Everything else...
	default:
		handled = false;
	}	
	return handled;
}
