/**	
	$Revision: 1.15 $
	Last updated: $Date: 2001/06/27 22:58:11 $

	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
	$Log: AccountsForm.h,v $
	Revision 1.15  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.14  2001/06/27 03:15:12  chaese
	Resurrected category selector in main form as type group selector
	
	Revision 1.13  2001/06/26 23:30:37  dizzy
	Forms are now classes rather then namespaces. This should enable future code optimisations
	
	Revision 1.12  2001/06/17 05:23:32  chaese
	*** CAUTION: DATABASE STRUCTURE HAS CHANGED YET AGAIN ***
	I generally don't like 'cross-logging', but too many changes have accumulated
	to keep them separated.
	1. completely eliminated account categories
	2. introduced currently unused type field in the account record structure
	(This will become what the user sees as 'category selector'. Coming soon!)
	3. completely eliminated protected accounts (mostly because #1 made it so much
	harder to locate the "Misc" account in the Expense category:) Those should be
	replaced with Pref. settings for default Income and Expense accounts anyway.
	4. changes to user interface; some as mere consequences of 1. (e.g.
	Account chooser), some for usability reasons (Transaction form).
	5. bugfixes and minor improvements.
	
	Revision 1.11  2001/01/31 18:07:24  dizzy
	More updates to the scheduling code (still not working though :( )
	Moved all Form code in a separate code section
	
	Revision 1.10  2000/12/11 16:27:24  dizzy
	C++ version of the code.
	
	Revision 1.9  2000/10/12 17:17:24  dizzy
	Email address update; lots of bugfixes
	
	Revision 1.8  2000/09/15 10:10:48  dizzy
	First useful version
	
	Revision 1.7  2000/09/13 11:50:27  dizzy
	The transaction module is fully functional
	
	Revision 1.6  2000/09/08 10:41:10  dizzy
	Transactiondatabas initial support and column display preferences support
	
	Revision 1.5  2000/09/04 14:23:07  dizzy
	General code cleanup	
*/

#ifndef __ACCOUNTS_FORM__
#define __ACCOUNTS_FORM__

#include <PalmOS.h>

#include "Form.h"
#include "AccountsTable.h"
#include "TransactionsForm.h"
#include "EditAccountForm.h"

/**
	contains function to manipulate the main form of the program
	the variables are defined in the .c file (due to visibility issues ;)
*/
class AccountsForm : public Form 
{
	private:
	EditAccountForm*		edit_account_form;
	EditTransactionForm*	edit_trans_form;
	
	/// name of the current category
	Char cat_name [dmCategoryLength];

	///the normal way to do this would be to declare a pointer to the object here and allocate it when the form is opened
	///unfortunatelly memory management is not that straight forward for the palm.
	AccountsTable ac_tbl;
	
	/// store last known pen X position
	Int16 last_screenX;
	/// store last known pen Y position
	Int16 last_screenY;
		
	public:
	AccountsForm(EditAccountForm* a, EditTransactionForm* t) : edit_account_form(a), edit_trans_form(t) {}
	virtual ~AccountsForm() {};
	/**
		calculates the totals (if necessary) and draws them on the screen
	*/
	void	displayTotals() FORM_SECTION;

	/// reflect choice of visible accounts on popup trigger
	void updatePopupTrigger(void) FORM_SECTION;

	/**
		Generic handler for the application's static form(s)
		@return true if the event was handled, false otherwise
	*/
	Boolean handleEvent(EventPtr event) FORM_SECTION;
};

#endif
