/**	
	$Revision: 1.8 $
	Last updated: $Date: 2001/07/24 01:00:04 $
	
	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: AccountsTable.cpp,v $
	Revision 1.8  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.7  2001/07/23 10:51:49  chaese
	code cleanup in loadData method
	
	Revision 1.6  2001/07/15 05:17:07  chaese
	* display only one balance (expected or cleared) in account table, popup
	  trigger lets user choose.
	* New function PopupListSetSelection to eliminate code duplication
	
	Revision 1.5  2001/06/30 01:17:05  chaese
	credit accounts are handled correctly now
	
	Revision 1.4  2001/06/29 00:24:57  chaese
	Rewrote how total balances are tracked. This should eliminate all problems
	that were associated with the old way (i.e. balances not being recalculated
	when they should have been etc.)
	
	Revision 1.3  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.2  2001/06/27 03:15:12  chaese
	Resurrected category selector in main form as type group selector
	
	Revision 1.1  2001/06/25 10:42:35  dizzy
	Change file extensions
	
	Revision 1.26  2001/06/21 11:04:23  chaese
	* initial support for account types
	* fixed some obscure bugs that materialized on a PalmOS 3.5 debug rom
	(see related message on palmcoins-misc for details)
	
	Revision 1.25  2001/06/11 11:03:21  chaese
	Increased display speed in account and transaction tables:
	* Associate table rows with the record index rather than UID. (Minor gain)
	* Perform preference lookup at program startup rather than every time a
	date or number is drawn. (Major gain! That was really stupid of me to put
	the lookups into the drawing function in the first place!)
	
	Revision 1.24  2001/06/02 14:32:10  chaese
	Make use of bitmaps in Account tree. Merged 'arrow' column and 'name' column
	to draw arrow directly in front of the name instead of at the left edge.
	
	Revision 1.23  2001/05/16 17:37:20  dizzy
	Fixed bug 424845 (deleting the default account)
	Fixed bug which prevented displaying last account under certain conditions
	Groovy new icons :) (large, small, b&w and colour)
	
	Revision 1.22  2001/05/14 17:15:55  dizzy
	Parent accounts display total in children accounts when collapsed
	The program displays a message while creating the sample accounts on startup
	Quick transaction button (together with default account selection)
	Can choose default cleared state for transactions
	Fixed bug 407185 (adding scheduled transactions)
	
	Revision 1.21  2001/04/25 21:10:30  dizzy
	Account names are clipped if they don't fit in the column
	Collapsable account tree
	
	Revision 1.20  2001/04/23 18:17:57  dizzy
	New account browser form
	Transactions use the new account browser
	Can create sub-accounts
	Replaced the fake indentation with real one - based on depth
	
	Revision 1.19  2000/12/13 18:06:54  dizzy
	Fixed a crash occuring when trying to delete an account
	Fixed a bug which caused the preferences to be inactive
	
	Revision 1.18  2000/12/12 00:17:28  dizzy
	Fixed a bug relate to displaying scroll arrows
	
	Revision 1.17  2000/12/11 16:27:24  dizzy
	C++ version of the code.	
*/

#include "AccountsTable.h"

#include "CoinsPref.h" 
#include "AccountsDB.h"
#include "MiscFunc.h"

#include "FreeCoinsRsc.h"

void AccountsTable::drawCurrency(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	Char*				s;
	accountStructure 	as;
	Int32 amount = 0;

	accounts_db->getUnpackedRecord(TblGetRowID((TablePtr) table,row),&as);
	switch ( (as.collapsed?2:0)
			 + (preferences.get()->disp_expected_bal?1:0) ) {
		case 0: amount = as.balance; break;
		case 1: amount = as.expected_balance; break;
		case 2: amount = as.children_total+as.balance; break;
		case 3: amount = as.children_expected_total+as.expected_balance; break;
	}

	if (as.credit_account) amount *= -1;
	s = Pence2String(amount);
	DrawCharsRightAlign(s, bounds);

	//free memory allocated by Pence2String
	MemPtrFree(s);
}

void AccountsTable::drawText(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	accountStructure 	as;
    UInt8 offset, fudge;

	accounts_db->getUnpackedRecord(TblGetRowID((TablePtr) table,row),&as);

  	offset = ACC_TBL_INDENT * as.depth;
	if (as.children_no!=0) {
		fudge = (TblGetRowHeight((TablePtr) table,row)-ACC_TBL_INDENT)/2;
		MemHandle resH = DmGetResource('Tbmp',
			as.collapsed ? BMP_EXPAND : BMP_COLLAPSE);
		BitmapType *bitmap = (BitmapType *)MemHandleLock(resH);
		WinDrawBitmap(bitmap, bounds->topLeft.x+offset,bounds->topLeft.y+fudge);
		MemHandleUnlock(resH);
		DmReleaseResource(resH);
	}
   	offset += ACC_TBL_INDENT + 1;
	WinDrawTruncChars(as.acname,StrLen(as.acname),
		bounds->topLeft.x+offset,bounds->topLeft.y,bounds->extent.x-offset);
}

Boolean AccountsTable::recordIsValid(UInt16 id)
{
	return (accounts_db->isDisplayable(id));
}

void AccountsTable::loadData()
{
	UInt16		recordNo,row,i;
	
	row = 0;
	i = 0;
	
	initColumns();
	
	for (recordNo=0;
		DmQueryNextInCategory(accounts_db->getRef(),
		&recordNo,accounts_db->current_cat); recordNo++)
	{
		if (!recordIsValid(recordNo)) continue;
		if (i++ < top_visible_item) continue;
		if (row>=rows) continue;
		TblSetRowID(table, row, recordNo);
		TblSetRowUsable(table, row, true);
		row++;
	}
	setRowsUnusable(row);
	total_items = i;
}	


void AccountsTable::initColumns()
{	
	//the numbers are drawn with our custom procedure
	TblSetCustomDrawProcedure(table, name_col, drawText);
	TblSetCustomDrawProcedure(table, balance_col, drawCurrency);

	//set style for all cells
	for (UInt16 row = 0; row < rows; row++) {
		TblSetItemStyle(table, row, name_col, customTableItem);
		TblSetItemStyle(table, row, balance_col, customTableItem);
	}
	
	//set all columns as usable
	TblSetColumnUsable(table, name_col, true);
	TblSetColumnUsable(table, balance_col, true);
}		

AccountsTable::AccountsTable() : Table(ACC_TABLE, ACC_PGUP, ACC_PGDN)
{
}
