/**	
	$Revision: 1.18 $
	Last updated: $Date: 2001/07/15 05:17:07 $

	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: AccountsTable.h,v $
	Revision 1.18  2001/07/15 05:17:07  chaese
	* display only one balance (expected or cleared) in account table, popup
	  trigger lets user choose.
	* New function PopupListSetSelection to eliminate code duplication
	
	Revision 1.17  2001/06/29 00:24:57  chaese
	Rewrote how total balances are tracked. This should eliminate all problems
	that were associated with the old way (i.e. balances not being recalculated
	when they should have been etc.)
	
	Revision 1.16  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.15  2001/06/02 14:32:10  chaese
	Make use of bitmaps in Account tree. Merged 'arrow' column and 'name' column
	to draw arrow directly in front of the name instead of at the left edge.
	
	Revision 1.14  2001/04/25 21:10:30  dizzy
	Account names are clipped if they don't fit in the column
	Collapsable account tree
	
	Revision 1.13  2000/12/11 16:27:24  dizzy
	C++ version of the code.	
*/

#ifndef __ACCOUNTS_TABLE_H__
#define __ACCOUNTS_TABLE_H__

#include <PalmOS.h>

#include "Table.h"

/// number of pixels per indentation depth in the account tree
#define ACC_TBL_INDENT 5

/**
	the table displays a list of accounts
*/
class AccountsTable : public Table
{
  public:
	static const Int16		name_col=0;
	static const Int16		balance_col=1;	

	/// initialises the table
	AccountsTable();

	///loads data into the accounts table
	void loadData();
	
	///inits the columns in the table
	void initColumns();
	
	///overloads the method in Table
	Boolean recordIsValid(UInt16 id);

	///draws an arrow which indicates the state of the branch
	static void drawArrow (void* table,Int16 row, Int16 column, RectangleType* bounds);

	///draws a number into the specified pos of a table
	static void drawCurrency (void* table,Int16 row, Int16 column, RectangleType* bounds);

	///draws text into the specified pos of a table
	static void drawText(void* table, Int16 row, Int16 column, RectangleType* bounds);
};

#endif
