/**
    $Revision: 1.5 $
    Last updated: $Date: 2001/07/16 02:15:57 $

    Copyright (C) 2001 Vlad Mereuta <dizzy@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    $Log: ChooseAccountForm.cpp,v $
    Revision 1.5  2001/07/16 02:15:57  chaese
    ChooseAccountForm now shows the whole list of accounts (rather than having
    the user navigate through the hierarchy levels) and preselects the current
    setting as specified by the caller.

    Revision 1.4  2001/07/09 02:29:39  chaese
    implement splits GUI & other small improvements along the way. The bulk of
    the splits GUI is complete, small data integrity checks are coming soon
    (need to prevent multiple splits for the same account within one transaction
    and stuff like that...)

    Revision 1.3  2001/07/01 03:31:42  chaese
    eliminated artifacts from old way of categorizing accounts

    Revision 1.2  2001/06/26 23:30:37  dizzy
    Forms are now classes rather then namespaces. This should enable future code optimisations

    Revision 1.1  2001/06/25 10:42:35  dizzy
    Change file extensions

    Revision 1.3  2001/06/17 05:23:32  chaese
    *** CAUTION: DATABASE STRUCTURE HAS CHANGED YET AGAIN ***
    I generally don't like 'cross-logging', but too many changes have accumulated
    to keep them separated.
    1. completely eliminated account categories
    2. introduced currently unused type field in the account record structure
    (This will become what the user sees as 'category selector'. Coming soon!)
    3. completely eliminated protected accounts (mostly because #1 made it so much
    harder to locate the "Misc" account in the Expense category:) Those should be
    replaced with Pref. settings for default Income and Expense accounts anyway.
    4. changes to user interface; some as mere consequences of 1. (e.g.
    Account chooser), some for usability reasons (Transaction form).
    5. bugfixes and minor improvements.

    Revision 1.2  2001/05/14 17:15:55  dizzy
    Parent accounts display total in children accounts when collapsed
    The program displays a message while creating the sample accounts on startup
    Quick transaction button (together with default account selection)
    Can choose default cleared state for transactions
    Fixed bug 407185 (adding scheduled transactions)

    Revision 1.1  2001/04/25 21:12:36  dizzy
    initial cvs commit
*/

#include "ChooseAccountForm.h"
#include "AccountsDB.h"

ChooseAccountForm* ChooseAccountForm::theForm = NULL;

void ChooseAccountForm::drawAccountName(Int16 itemNum,
	RectangleType *bounds, Char **txt)
{
	Char acc_name[80];
	UInt32 acc_uid;

	acc_uid = accounts_db->getRecordUid(itemNum);
	accounts_db->getFullNameTruncLeftToFit(acc_uid, bounds->extent.x, acc_name);
	WinDrawChars(acc_name, StrLen(acc_name),
		bounds->topLeft.x, bounds->topLeft.y);
}

void ChooseAccountForm::init()
{
	ListPtr list= (ListPtr) GetObjectPtr(CHOOSE_ACC_LIST);
	Int16 sel;
	chosen = false;

	LstSetDrawFunction(list, drawAccountName);
	LstSetListChoices(list, NULL, DmNumRecordsInCategory(accounts_db->getRef(),
		dmAllCategories));
	sel = noListSelection;
	if (chosen_account) {
		if(DmFindRecordByID(accounts_db->getRef(),chosen_account,(UInt16*)&sel))
			sel = noListSelection;
	}
	LstSetSelection(list, sel);
	if (sel!=noListSelection)
		LstMakeItemVisible(list, sel);
	if (allow_no_selection)
		FrmShowObject(frm, FrmGetObjectIndex(frm, CHOOSE_ACC_NONE));
}

Boolean ChooseAccountForm::handleEvent(EventPtr event)
{
	UInt32 acc_uid;
	Boolean 	handled = true;
	ListPtr list= (ListPtr) GetObjectPtr(CHOOSE_ACC_LIST);

	// Get the form pointer
	frm = FrmGetActiveForm();

	switch(event->eType) {
	// Form Open
	case frmOpenEvent:		
		//draw form
		init();
		FrmDrawForm(frm);
		break;
	
	//list selection
	case lstSelectEvent:
		handled=false;
		break;

	// Controls
	case ctlSelectEvent:
		switch( event->data.ctlSelect.controlID ) {
			case CHOOSE_ACC_OK: {
				handled = true;
				Int16 sel = LstGetSelection(list);
				if((sel == noListSelection) && !allow_no_selection) break;
				if((sel == noListSelection) && allow_no_selection) {
					//user made empty selection
					chosen_account = 0;
				} else {
					acc_uid = accounts_db->getRecordUid(sel);
					// first check that the chosen account is not the same
					// as the current one
					if(accounts_db->isRecordSelected() &&
						 (accounts_db->getSelectedUid() == acc_uid)) {
						FrmCustomAlert(GENERIC_ERROR,
					 "You must not choose the currently selected account.",
							NULL, NULL);
						break;
					}
					chosen_account = acc_uid;
				}
				chosen = true;
			
				FrmUpdateForm(caller_form_id,0);
								
				FrmReturnToForm(0);
				break;
			}
							
			case CHOOSE_ACC_CANCEL:
				//cancel button was pressed;
				//simply return to the main form
				handled = true;
				FrmReturnToForm(0);
				break;

			case CHOOSE_ACC_NONE:
				LstSetSelection(list, noListSelection);
				handled = true;
				break;
				
			default:
				handled = false;
		};
		break;	

	// Key Events
	case keyDownEvent:
		switch (event->data.keyDown.chr) {
			case pageUpChr:
				LstScrollList(list, winUp, 9);
				break;
			
			case pageDownChr:
				LstScrollList(list, winDown, 9);
				break;

			default:
				handled = false;
				break;
		}
		break;

	// Everything else...
	default:
		handled = false;
	}	
	return handled;
}
