/**	
	$Revision: 1.8 $
	Last updated: $Date: 2001/07/24 09:02:43 $
	
	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: CoinsPref.cpp,v $
	Revision 1.8  2001/07/24 09:02:43  dizzy
	Renamed some #defines
	
	Revision 1.7  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.6  2001/07/23 10:48:33  chaese
	Pack bools into bitfields
	
	Revision 1.5  2001/07/15 06:53:59  chaese
	Fix: #441393. To make things less convoluted, the preferences store the
	transaction category without offset, and the offset is used for displaying
	in the preferences form only.
	
	Revision 1.4  2001/07/15 05:17:07  chaese
	* display only one balance (expected or cleared) in account table, popup
	  trigger lets user choose.
	* New function PopupListSetSelection to eliminate code duplication
	
	Revision 1.3  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.2  2001/06/27 03:15:12  chaese
	Resurrected category selector in main form as type group selector
	
	Revision 1.1  2001/06/25 10:42:35  dizzy
	Change file extensions
	
	Revision 1.16  2001/06/14 04:19:49  chaese
	corrected/optimized preferences handling
	
	Revision 1.15  2001/05/14 17:15:55  dizzy
	Parent accounts display total in children accounts when collapsed
	The program displays a message while creating the sample accounts on startup
	Quick transaction button (together with default account selection)
	Can choose default cleared state for transactions
	Fixed bug 407185 (adding scheduled transactions)
	
	Revision 1.14  2001/04/20 20:58:43  dizzy
	fixed bug 408435
	Fixed preferences initialisation problem (causing a memory write to an invalid location) when the program is run for the first time
	
	Revision 1.13  2001/04/19 10:55:40  dizzy
	Moved to version 0.4.1
	
	Revision 1.12  2000/12/13 16:37:34  dizzy
	Ability to display a column which indicates/modifies the cleared state for each transaction
		(it is on by default; it can be turned off from the preferences dialog)
	The default view in the transaction list can be selected from the preferences dialog
	
	Revision 1.11  2000/12/11 16:27:24  dizzy
	C++ version of the code.
	
	Revision 1.10  2000/12/03 13:05:12  dizzy
	Compiles cleanily with the c++ compiler. Some bits&pieces translated to c++
	
	Revision 1.9  2000/11/15 18:58:42  dizzy
	New balance adjust option in transaction form menu.
	Help updated in Transaction form and Accounts form.
	Merged ShowScrollArrows functions.
	Updated a few variable and function names.
	
	Revision 1.8  2000/10/12 17:17:24  dizzy
	Email address update; lots of bugfixes
	
	Revision 1.7  2000/10/08 15:40:46  dizzy
	Fixed a load of bugs related to transaction support.
	Displaying i/e accounts in all view is optional.	
*/

#include "CoinsPref.h"

#include <Preferences.h>
#include <DateTime.h>

#include "AccountsDB.h" //to be able to manipulate the totals structure

CoinsPref::CoinsPref()
{
	id = 0;	//id for the pref structure above
	ver = 1;
	//load the application preferences
	read();	
}

CoinsPrefType* CoinsPref::get()
{
	return &coins_pref;
}

void CoinsPref::read()
{
	UInt16 t=sizeof(coins_pref);
	int ret=PrefGetAppPreferences(FREE_COINS_CREATOR,id,(void*) &coins_pref,&t,true);
	
	if (ret==noPreferenceFound) {
		setDefault();
		save();
	}
}

void CoinsPref::save()
{
	PrefSetAppPreferences(FREE_COINS_CREATOR,id,ver,(void*) &coins_pref,sizeof(coins_pref),true);
}

void CoinsPref::setDefault()
{
	//set the last update to yesterday
	DateType today;

	//get today's date
	DateSecondsToDate(TimGetSeconds(),& today);
	
	//store yesterday in today
	DateAdjust(&today,-1);
	coins_pref.last_update=today;
	coins_pref.visible_acc_types = 0;
	coins_pref.disp_expected_bal=false;
	coins_pref.disp_payee=false;
	coins_pref.disp_cleared_col=true;
	coins_pref.reserved = 0;
	coins_pref.remember_trans_view=false;
	coins_pref.trans_view=0;
	coins_pref.def_acc = 0;
	coins_pref.trans_cleared = false;
}

CoinsPref preferences;
