/**	
	$Revision: 1.19 $
	Last updated: $Date: 2001/07/24 01:00:04 $
	
	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: CoinsPref.h,v $
	Revision 1.19  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.18  2001/07/23 10:48:33  chaese
	Pack bools into bitfields
	
	Revision 1.17  2001/07/15 05:17:07  chaese
	* display only one balance (expected or cleared) in account table, popup
	  trigger lets user choose.
	* New function PopupListSetSelection to eliminate code duplication
	
	Revision 1.16  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.15  2001/06/27 03:15:12  chaese
	Resurrected category selector in main form as type group selector
	
	Revision 1.14  2001/06/14 04:19:49  chaese
	corrected/optimized preferences handling
	
	Revision 1.13  2001/05/14 17:15:55  dizzy
	Parent accounts display total in children accounts when collapsed
	The program displays a message while creating the sample accounts on startup
	Quick transaction button (together with default account selection)
	Can choose default cleared state for transactions
	Fixed bug 407185 (adding scheduled transactions)
	
	Revision 1.12  2000/12/13 16:37:34  dizzy
	Ability to display a column which indicates/modifies the cleared state for each transaction
		(it is on by default; it can be turned off from the preferences dialog)
	The default view in the transaction list can be selected from the preferences dialog
	
	Revision 1.11  2000/12/11 16:27:24  dizzy
	C++ version of the code.
	
	Revision 1.10  2000/12/03 13:05:12  dizzy
	Compiles cleanily with the c++ compiler. Some bits&pieces translated to c++
	
	Revision 1.9  2000/11/15 18:58:42  dizzy
	New balance adjust option in transaction form menu.
	Help updated in Transaction form and Accounts form.
	Merged ShowScrollArrows functions.
	Updated a few variable and function names.
	
	Revision 1.8  2000/10/12 17:17:24  dizzy
	Email address update; lots of bugfixes
	
	Revision 1.7  2000/10/08 15:40:46  dizzy
	Fixed a load of bugs related to transaction support.
	Displaying i/e accounts in all view is optional.	
*/

#ifndef __COINS_PREFERENCES__
#define __COINS_PREFERENCES__

#include <PalmOS.h>

///structure containing the program preferences
typedef struct {
	///when the scheduled transactions were last updated
	DateType	last_update;	
	///account types the user wants to see
	UInt16			visible_acc_types;
	///default view in the transaction list
	UInt16			trans_view;
	///default account for quick transactions (0 if not set)
	UInt32			def_acc;
	///toogle for displaying the expected balance in the totals column
	unsigned		disp_expected_bal	:1;
	///true if the payee is displayed in transaction list, false otherwise
	unsigned		disp_payee			:1;
	///true if we want to display a cleared column in the transactions table, false otherwise
	unsigned		disp_cleared_col	:1;
	///if true the last view will allways be remembered
	unsigned		remember_trans_view	:1;
	///default cleared state for transactions
	unsigned		trans_cleared		:1;
	///reserved for future extensions
	unsigned		reserved 			:11;
} CoinsPrefType;


///manipulates the program preferences
class CoinsPref 
{
 private:
	///the structure holds the app's preferences
	CoinsPrefType 	coins_pref;	
	
	///id for the pref structure above
	UInt16	id;	
	/// version
	Int16 	ver;
	
 public:
	/// the constructor sets the values for the id and version
	CoinsPref();
		 
	///@return pointer to the current preferences:
	CoinsPrefType* get();
	
	///retrieve the preferences into coins_pref
	void read();

	///save the preferences from coins_pref
	void save();


	///assigns default values to coins_pref
	void setDefault();
};

extern CoinsPref preferences;

#endif
