/**
    $Revision: 1.14 $
    Last updated: $Date: 2001/07/24 23:35:10 $

    Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    $Log: EditAccountForm.cpp,v $
    Revision 1.14  2001/07/24 23:35:10  chaese
    FIX: infinite loop check mustn't be performed while creating a new account

    Revision 1.13  2001/07/24 01:00:04  chaese
    eliminated references to PalmCoins except in situations that require
    renaming CVS controlled files (or other instances that I've overlooked :)

    Revision 1.12  2001/07/16 02:15:57  chaese
    ChooseAccountForm now shows the whole list of accounts (rather than having
    the user navigate through the hierarchy levels) and preselects the current
    setting as specified by the caller.

    Revision 1.11  2001/07/15 05:17:07  chaese
    * display only one balance (expected or cleared) in account table, popup
      trigger lets user choose.
    * New function PopupListSetSelection to eliminate code duplication

    Revision 1.10  2001/07/09 02:29:39  chaese
    implement splits GUI & other small improvements along the way. The bulk of
    the splits GUI is complete, small data integrity checks are coming soon
    (need to prevent multiple splits for the same account within one transaction
    and stuff like that...)

    Revision 1.9  2001/07/01 16:00:42  chaese
    Prevent placing an account inside itself (fixes bug #437699)

    Revision 1.8  2001/07/01 03:33:50  chaese
    fixed bugs #437662 and #437663
    Also fixed the bug that a newly created account would be visible in
    in the main form regardless of the selected subset.

    Revision 1.7  2001/06/30 01:36:00  chaese
    prevent accounts with children from being deleted

    Revision 1.6  2001/06/30 01:17:05  chaese
    credit accounts are handled correctly now

    Revision 1.5  2001/06/29 00:24:57  chaese
    Rewrote how total balances are tracked. This should eliminate all problems
    that were associated with the old way (i.e. balances not being recalculated
    when they should have been etc.)

    Revision 1.4  2001/06/27 22:58:11  dizzy
    Restructured #include directives

    Revision 1.3  2001/06/26 23:30:37  dizzy
    Forms are now classes rather then namespaces. This should enable future code optimisations

    Revision 1.2  2001/06/26 01:29:19  chaese
    Selecting a parent account inherits the parent's type and credit flag

    Revision 1.1  2001/06/25 10:42:35  dizzy
    Change file extensions

    Revision 1.25  2001/06/21 11:04:23  chaese
    * initial support for account types
    * fixed some obscure bugs that materialized on a PalmOS 3.5 debug rom
    (see related message on palmcoins-misc for details)
*/

#include "EditAccountForm.h"

#include "AccountsDB.h"
#include "MiscFunc.h"
#include "FreeCoinsRsc.h"

Boolean EditAccountForm::save()
{	
	Int32						newbal;
	UInt16						index;
	Int8	depth_increment = 0;
	MemHandle           h;   

    //if the balance is invalid it will be set to 0 by default
   	newbal=String2Pence((Char*) GetTextPtr(EDIT_ACC_BALANCE));

	//get the credit limit
   	account.credit_limit=String2Pence((Char*) GetTextPtr(EDIT_ACC_CREDIT));

	//get the description
   	account.description=(Char*) GetTextPtr(EDIT_ACC_DESCRIPTION);
	if(!account.description) account.description = "";
	//same for account name; save is forbidden if no name is set
   	account.acname=(Char*) GetTextPtr(EDIT_ACC_NAME);
   	if(!account.acname) return false;

	ListType *type_list =
	(ListType*)FrmGetObjectPtr(frm,
		FrmGetObjectIndex(frm,EDIT_ACC_TYPE_LIST));
	account.acc_type = (AccountType)LstGetSelection(type_list);
	account.credit_account=FrmGetControlValue(frm,FrmGetObjectIndex(frm,EDIT_ACC_CREDIT_CBX));
	
	if (account.credit_account) {
		newbal *= -1;
	}

	if (record_is_new) {
    	account.balance=newbal;
    	account.expected_balance=newbal;
        account.children_no = 0;
    	index=accounts_db->newRecord(&account);
	}
	else {
		//write the data

		//check if the account has been reparented. If so, update the old and
		//new parent's children_no and children total, and the account's depth
		if (account.parent_uid!=prev_parent_uid || account.depth!=prev_depth)
		{
			accountStructure    parent_acct;

			// decrease old parent's children count
			if (prev_depth!=0) {
				h = accounts_db->getUnpackedRecord(prev_parent_uid,
					&parent_acct);
				parent_acct.children_no--;
				accounts_db->packRecord(&parent_acct, h);

				// subtract the balance from the old parent's children totals
				accounts_db->incrementBalance(prev_parent_uid,
					-account.balance-account.children_total,
					-account.expected_balance-account.children_expected_total,
					true);
			}

			// increase new parent's children count and set new depth
			if (account.depth!=0) {
				h = accounts_db->getUnpackedRecord(account.parent_uid,
					&parent_acct);
				parent_acct.children_no++;
				accounts_db->packRecord(&parent_acct, h);

				// add the balance to the new parent's children totals
				accounts_db->incrementBalance(account.parent_uid,
					account.balance+account.children_total,
					account.expected_balance+account.children_expected_total,
					true);
			}
		}
		depth_increment = account.depth - prev_depth;
		account.depth = prev_depth;

		accounts_db->packRecord(&account,rech);

		// apply depth change to the account and all descendants
		if (depth_increment) {
			accountStructure a;
			UInt16 rec_id = accounts_db->getSelectedRecord();

			while (DmQueryNextInCategory(accounts_db->getRef(),
						&rec_id, dmAllCategories)) {
				h = accounts_db->getUnpackedRecord(rec_id, &a);
				if (a.depth<=prev_depth &&
					rec_id!=accounts_db->getSelectedRecord()) {
					break;
				}
				a.depth += depth_increment;
				accounts_db->packRecord(&a,h);
				rec_id++;
			}
		}

		//better late than never, this sets the busy bit
		DmGetRecord(accounts_db->getRef(), accounts_db->getSelectedRecord());

		//release record clears the busy bit and set the dirty bit
		DmReleaseRecord(accounts_db->getRef(), accounts_db->getSelectedRecord(), true); 

		if (newbal!=account.balance) { 
			accounts_db->updateBalance(accounts_db->getSelectedUid(),newbal);
		}


		//sort the records in the db - just in case the name changed
		UInt32 uid = accounts_db->getSelectedUid();
		DmInsertionSort(accounts_db->getRef(),
			(DmComparF *) accounts_db->compareRecords,0);
		// Reselect the record in case sorting changed its position.
		// The database should probably track the selected record by its uid,
		// but until that happens, this will do.
		accounts_db->selectRecord(uid);
	}
	accounts_db->updateVisibility();
	return true;
}

void EditAccountForm::init()
{
	Char*	s;
	
	selected_cat=accounts_db->findCategory("Visible");

	if (record_is_new) {
		FrmCopyTitle(frm, "Create New Account");

		account.credit_account = false;
		setParentAccount(0); //by default no parent
		account.acc_type = BANK;
	} else {
		FrmCopyTitle(frm, "Account Details");
		FrmShowObject(frm, FrmGetObjectIndex(frm, EDIT_ACC_DELETE));
		rech = accounts_db->getUnpackedRecord(&account);				

		//set the credit checkbox
		FrmSetControlValue(frm,FrmGetObjectIndex(frm,EDIT_ACC_CREDIT_CBX),account.credit_account);
				
		SetFieldTextFromStr(EDIT_ACC_DESCRIPTION,account.description);
		SetFieldTextFromStr(EDIT_ACC_NAME,account.acname);

		s=Pence2String((account.credit_account?-1:1)*account.balance);
		SetFieldTextFromStr(EDIT_ACC_BALANCE,s);
		MemPtrFree(s);
		s=Pence2String(account.credit_limit);
		SetFieldTextFromStr(EDIT_ACC_CREDIT,s);
		//free memory allocated by Pence2String
		MemPtrFree(s);
		
		if(account.depth != 0)
			setParentAccount(account.parent_uid); //parent is an account
		else
			setParentAccount(0); //no parent

		prev_parent_uid = account.parent_uid;
		prev_depth = account.depth;
	}
    updateTypeSelection();
}

void EditAccountForm::updateTypeSelection(void)
{
	PopupListSetSelection(EDIT_ACC_TYPE_LIST, EDIT_ACC_TYPE_TRIGGER,
		(UInt8)account.acc_type);
}

void	EditAccountForm::setParentAccount(UInt32 id)
{
	if(id) { //if the user selected an account
		accountStructure as;
		account.parent_uid = id;
		accounts_db->getUnpackedRecord(id, &as);
		account.depth = as.depth + 1;
		accounts_db->getFullNameTruncLeftToFit(account.parent_uid, 110,
			parent_name);
	} else {
		StrCopy(parent_name, "Top level");
		account.depth = account.parent_uid = 0;
	}
	CtlSetLabel((ControlType*) GetObjectPtrByForm(FrmGetActiveForm(),EDIT_ACC_PARENT),parent_name);
}

Boolean EditAccountForm::handleEvent(EventPtr event)
{
	Boolean 	handled = true;
	accountStructure acc;

	// Get the form pointer
	frm = FrmGetActiveForm();
	
	// Key Events
	switch(event->eType) {

	case frmUpdateEvent:
		if(choose_account_form->chosen) {
			UInt32 this_uid, parent_uid;
			Boolean bad_choice = false;
			choose_account_form->chosen = false;
			if (!record_is_new) {
				this_uid = accounts_db->getRecordUid(
					accounts_db->getSelectedRecord());	
				parent_uid = choose_account_form->chosen_account;
				while (parent_uid && parent_uid != this_uid) {
					accounts_db->getUnpackedRecord(parent_uid, &acc);
					if (acc.depth==0)
						break;
					parent_uid = acc.parent_uid;
				}
				if (parent_uid == this_uid)
					bad_choice = true;
			}
			if (bad_choice) {
				FrmCustomAlert(GENERIC_ERROR, "This choice would place the account inside itself.", NULL, NULL);
				choose_account_form->chosen_account = 0;
			}
			else {
				setParentAccount(choose_account_form->chosen_account);
			}
			// inherit type and credit account flag from parent
			if (choose_account_form->chosen_account) {
				accounts_db->getUnpackedRecord(
					choose_account_form->chosen_account, &acc);
				account.acc_type = acc.acc_type;
				if (account.credit_account != acc.credit_account) {
					Int32 bal;
					Char *s;
					bal = -String2Pence((Char*) GetTextPtr(EDIT_ACC_BALANCE));
					s = Pence2String(bal);
					SetFieldTextFromStr(EDIT_ACC_BALANCE,s);
					MemPtrFree(s);	
					account.credit_account = acc.credit_account;
				}
				FrmSetControlValue(frm,
					FrmGetObjectIndex(frm,EDIT_ACC_CREDIT_CBX),
					 account.credit_account);
				updateTypeSelection();
			}
		}
		if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
		FrmDrawForm(frm);
		break;
	
	// Form Open
	case frmOpenEvent:		
		//draw form
		FrmDrawForm(frm);
		init();
		break;
	
	// Controls
	case ctlSelectEvent:
		switch( event->data.ctlSelect.controlID ) {
			case EDIT_ACC_CREDIT_CBX:
				{
					Int32 bal;
					Char *s;
					bal = -String2Pence((Char*) GetTextPtr(EDIT_ACC_BALANCE));
					s = Pence2String(bal);
					SetFieldTextFromStr(EDIT_ACC_BALANCE,s);
					MemPtrFree(s);	
					handled = true;
				}
				break;

			case EDIT_ACC_OK:
				handled = true;
				//if something went wrong do nothing
				if(! save()) break;
				
				//if the record is new, we were caller from the accounts from, otherwise from the transactions form
				if(record_is_new) FrmUpdateForm(ACCOUNTS_MAIN_FORM,0);
				else FrmUpdateForm(TRANSACTIONS_FORM,0);				
				FrmReturnToForm(0);
				break;

			case EDIT_ACC_PARENT:
				handled = true;
				choose_account_form->allow_no_selection = true;	
				choose_account_form->setCallerFormId(EDIT_ACC_FORM);				
				choose_account_form->chosen_account = account.parent_uid;
				FrmPopupForm(CHOOSE_ACCOUNT_FORM);							
				break;
			
			case EDIT_ACC_CANCEL:
				//cancel button was pressed;
				//simply return to the main form
				handled = true;
				if(record_is_new) FrmUpdateForm(ACCOUNTS_MAIN_FORM,0);
				else FrmUpdateForm(TRANSACTIONS_FORM,0);				
				FrmReturnToForm(0);
				break;

			case EDIT_ACC_DELETE:
				//delete button
				if (record_is_new) {
					// we shouldn't be here, the button is invisible.
					// Just to be safe, though...
					handled = false;
				} else {
					handled = true;
					{
						// quick and dirty anti-crash precaution: don't allow
						// accounts with children to be deleted
						accountStructure a;
						accounts_db->getUnpackedRecord(&a);
						if (a.children_no) {
							FrmCustomAlert(GENERIC_ERROR, "Deleting an account with children is not allowed.", NULL, NULL);
							break;
						}
					}
					if (FrmCustomAlert(DELETE_ALERT,"account",NULL,NULL)==1)
						break;
					accounts_db->deleteRecord(accounts_db->getSelectedRecord());
	
					FrmUpdateForm(ACCOUNTS_MAIN_FORM,0);		
					//delete the transaction form
					FrmEraseForm(FrmGetFormPtr(TRANSACTIONS_FORM));				
					FrmDeleteForm(FrmGetFormPtr(TRANSACTIONS_FORM));				
					//return to the accounts form
					FrmGotoForm(ACCOUNTS_MAIN_FORM);
				}
				break;
				
			default:
				handled = false;
		};
		break;	
	// Everything else...
	default:
		handled = false;
	}	
	return handled;
}
