/**
    $Revision: 1.10 $
    Last updated: $Date: 2001/07/16 02:15:57 $

    Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    $Log: EditAccountForm.h,v $
    Revision 1.10  2001/07/16 02:15:57  chaese
    ChooseAccountForm now shows the whole list of accounts (rather than having
    the user navigate through the hierarchy levels) and preselects the current
    setting as specified by the caller.

    Revision 1.9  2001/06/27 22:58:11  dizzy
    Restructured #include directives

    Revision 1.8  2001/06/26 23:30:37  dizzy
    Forms are now classes rather then namespaces. This should enable future code optimisations

    Revision 1.7  2001/06/26 01:29:19  chaese
    Selecting a parent account inherits the parent's type and credit flag

    Revision 1.6  2001/06/17 05:23:32  chaese
    *** CAUTION: DATABASE STRUCTURE HAS CHANGED YET AGAIN ***
    I generally don't like 'cross-logging', but too many changes have accumulated
    to keep them separated.
    1. completely eliminated account categories
    2. introduced currently unused type field in the account record structure
    (This will become what the user sees as 'category selector'. Coming soon!)
    3. completely eliminated protected accounts (mostly because #1 made it so much
    harder to locate the "Misc" account in the Expense category:) Those should be
    replaced with Pref. settings for default Income and Expense accounts anyway.
    4. changes to user interface; some as mere consequences of 1. (e.g.
    Account chooser), some for usability reasons (Transaction form).
    5. bugfixes and minor improvements.

    Revision 1.5  2001/04/23 18:17:57  dizzy
    New account browser form
    Transactions use the new account browser
    Can create sub-accounts
    Replaced the fake indentation with real one - based on depth

    Revision 1.4  2001/01/31 18:07:24  dizzy
    More updates to the scheduling code (still not working though :( )
    Moved all Form code in a separate code section

    Revision 1.3  2001/01/29 18:49:30  dizzy
    Put code for scheduled transactions in a separate code section to enable the linking process to succeed.
    Scheduled trans info can be add/deleted/updated
    Added cleared field to trans structure

    Revision 1.2  2000/12/11 16:27:25  dizzy
    C++ version of the code.

    Revision 1.1  2000/11/10 01:25:55  chaese
    merged New Account and Account Details forms
*/

#ifndef __EDIT_ACCOUNT_FORM__
#define __EDIT_ACCOUNT_FORM__

#include <PalmOS.h>

#include "Form.h"
#include "ChooseAccountForm.h"
#include "AccountsDB.h"

///the form is used to create a new transaction or to modify an existing one, depending on the record_is_new member
class EditAccountForm : public Form
{	
	public:
	
	/// the selected category
	UInt16	selected_cat;
	
	///buffer to store the parent name
	Char	parent_name[55];

	///holds the data being manipulated
	accountStructure 			account;
	MemHandle	rech;	
	UInt32 prev_parent_uid;
	UInt8 prev_depth;
			
	/// this is set to true if we want to create a new account and to false if we want the 'details' functionality
	Boolean record_is_new;

	///contains the id of the last saved transaction
	UInt16 trans_id;
	
	ChooseAccountForm* choose_account_form;

	///constructor
	EditAccountForm (ChooseAccountForm* c) : choose_account_form(c) {}	
	///saves the changes to the database
	Boolean save() FORM_SECTION;

	///set the values of various form fields from the database
	void	init() FORM_SECTION;

	/// reflect change in account type selection on popup trigger and list
	void 	updateTypeSelection() FORM_SECTION;

	/**
		saves the data provided by ChooseAccountForm and updates the screen
		@param is_account if true, the function will display and select a category  otherwise, an account
		@param id of the account/category to be displayed
	*/
	void	setParentAccount(UInt32 id) FORM_SECTION;
	
	///event handler for the form
	Boolean handleEvent(EventPtr event) FORM_SECTION;
	
	void setRecordIsNew(Boolean b) { record_is_new = b; }
};


#endif
