/**	
	$Revision: 1.7 $
	Last updated: $Date: 2001/07/24 01:00:04 $
	
	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: EditScheduledTransForm.cpp,v $
	Revision 1.7  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.6  2001/07/18 23:06:29  dizzy
	A few scheduling bugfixes. Plenty of issues left to resolve :-(
	
	Revision 1.5  2001/07/02 14:03:53  dizzy
	Horde of bugfixes to the transaction editing routines...
	
	Revision 1.4  2001/07/01 12:21:55  dizzy
	Merged ScheduledDB into TransactionsDB
	
	Revision 1.3  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.2  2001/06/26 23:30:37  dizzy
	Forms are now classes rather then namespaces. This should enable future code optimisations
*/

#include "EditScheduledTransForm.h"

#include "MiscFunc.h"
#include "FreeCoinsRsc.h"


void EditScheduledTransForm::setRecordIsNew(Boolean b)
{ 
   record_is_new = b; 
}

void EditScheduledTransForm::newSession(scheduledStructure* s, DateType* trans_date)
{ 
   st = s;
   setSaved(false);
   setRecordIsNew(true);
   this->trans_date = trans_date;
}

Boolean EditScheduledTransForm::save()
{	
   trans_date->year=start_date.year;
   trans_date->month=start_date.month;
   trans_date->day=start_date.day;

   st->frequency=StrAToI((Char*) GetTextPtr(SCHED_FREQ));
   if (!st->frequency) {
      FrmCustomAlert(ZERO_FIELD,"frequency",NULL,NULL);
      return false; //frequency has to be != 0
   }
	
	if(!st->end_by_date && st->repeat_times!=-1)
      st->repeat_times=StrAToI((Char*) GetTextPtr(SCHED_REPEAT));
   
   setSaved(true);
   setRecordIsNew(false);
   		
	return true;
}

void EditScheduledTransForm::setInterval(repeatInterval interval)
{
	//memorise the new interval first
	st->repeat_interval = interval;
	//now display changes
	switch (interval) {
		case rep_day:
			FrmSetControlGroupSelection(frm,1,SCHED_OPT_DAY);
			FrmCopyLabel(frm,SCHED_INTERVAL_LABEL,"day(s)  ");
			break;
		case rep_week:
			FrmSetControlGroupSelection(frm,1,SCHED_OPT_WEEK);
			FrmCopyLabel(frm,SCHED_INTERVAL_LABEL,"week(s) ");
			break;
		case rep_month:
			FrmSetControlGroupSelection(frm,1,SCHED_OPT_MONTH);
			FrmCopyLabel(frm,SCHED_INTERVAL_LABEL,"month(s)");
			break;
		case rep_year:
			FrmSetControlGroupSelection(frm,1,SCHED_OPT_YEAR);
			FrmCopyLabel(frm,SCHED_INTERVAL_LABEL,"year(s) ");
			break;
		}			
}

void EditScheduledTransForm::setDefaults()
{
	if(record_is_new) {
		//show the start date fields
		FrmCopyTitle(frm, "New scheduling info");
		//show the start date fields
        FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_FIRST_TRANS_LABEL));			
        FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_START_DATE));
      
		setInterval(rep_month); //default interval is month
		
		SetFieldTextFromStr(SCHED_FREQ,"1"); //default frequency is 1
		st->frequency=1;
		
		FrmSetControlGroupSelection(frm,2,SCHED_OPT_NEVER); //default end: repeat forever
		st->repeat_times=-1;

	    DateSecondsToDate(TimGetSeconds(),&start_date); //first set the default start date to today
	    DateSecondsToDate(TimGetSeconds(),&st->end_date); //first set the default end date to today
		updateDateField(SCHED_START_DATE,start_date,start_date_txt);						
	} else {
        FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_NEXT_TRANS_LABEL));			
        FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_NEXT_DATE));
        FrmCopyTitle(frm, "Scheduling details");
      
		setInterval(st->repeat_interval); //set the interval from the record
		SetFieldTextFromInt(SCHED_FREQ,st->frequency); //default frequency is 1
		updateDateField(SCHED_NEXT_DATE, *trans_date,start_date_txt);						
		
		//now set the end conditions:
		if( st->repeat_times==-1 ) 
			FrmSetControlGroupSelection(frm,2,SCHED_OPT_NEVER); //default end: repeat forever
		else	
         if (st->end_by_date) {
				FrmSetControlGroupSelection(frm,2,SCHED_OPT_ON); //stop on a date
	        	FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_END_DATE));	
				updateDateField(SCHED_END_DATE,st->end_date,end_date_txt);
			} else { //set number of repeats
				FrmSetControlGroupSelection(frm,2,SCHED_OPT_AFTER); //stop after a no of times
	         	FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT));			
	        	FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT_LABEL));			
				SetFieldTextFromInt(SCHED_REPEAT,st->repeat_times); 			
			}         
	}
}


Boolean EditScheduledTransForm::handleEvent(EventPtr event)
{
	Boolean handled = true;

	// Get the form pointer
	frm = FrmGetActiveForm();
		
	// Key Events
	switch(event->eType) {

	// Form Open
	case frmOpenEvent:		
		//draw form
		FrmDrawForm(frm);
		setDefaults();
		break;
	
	// Controls
	case ctlSelectEvent:
		switch( event->data.ctlSelect.controlID ) {
			case SCHED_TRANS_OK:
				//if something went wrong do nothing
				if(! save()) break;
				//now return to the previous form
				FrmUpdateForm(TRANSACTIONS_FORM,0);				
				FrmReturnToForm(0);
				break; 
			case SCHED_START_DATE:
				selectDate(start_date,SCHED_START_DATE,start_date_txt);
				break;
			case SCHED_END_DATE:
				selectDate(st->end_date,SCHED_END_DATE,end_date_txt);
				break;
			case SCHED_OPT_DAY: 
				setInterval(rep_day);
				break;
			case SCHED_OPT_WEEK: 
				setInterval(rep_week);
				break;
			case SCHED_OPT_MONTH: 
				setInterval(rep_month);
				break;
			case SCHED_OPT_YEAR: 
				setInterval(rep_year);
				break;							
			case SCHED_OPT_NEVER:
				FrmSetControlGroupSelection(frm,2,SCHED_OPT_NEVER); 
				st->repeat_times=-1;
				FrmHideObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT));			
				FrmHideObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT_LABEL));			
				FrmHideObject(frm, FrmGetObjectIndex(frm, SCHED_END_DATE));			
				st->end_by_date=false;
				break;
			case SCHED_OPT_AFTER:
				FrmSetControlGroupSelection(frm,2,SCHED_OPT_AFTER); 
				if(record_is_new || st->repeat_times<1) {
					st->repeat_times=3;
					SetFieldTextFromStr(SCHED_REPEAT,"3");
				} else {
					SetFieldTextFromInt(SCHED_REPEAT,st->repeat_times);
				}
				st->end_by_date=false;
				FrmHideObject(frm, FrmGetObjectIndex(frm, SCHED_END_DATE));			
				FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT));			
				FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT_LABEL));			
				break;
			case SCHED_OPT_ON:
				FrmSetControlGroupSelection(frm,2,SCHED_OPT_ON);
				if(st->repeat_times<0) st->repeat_times=0; 
				if(record_is_new)
				    DateSecondsToDate(TimGetSeconds(),&st->end_date);
				FrmHideObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT));			
				FrmHideObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT_LABEL));							
				FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_END_DATE));
				st->end_by_date=true;
				updateDateField(SCHED_END_DATE,st->end_date,end_date_txt);								
				break;
			
			case SCHED_TRANS_CANCEL:
				//cancel button was pressed;
				//simply return to the previous form (i.e. 0)
				handled = true;
				FrmUpdateForm(EDIT_TRANS_FORM,0);
				FrmReturnToForm(0);
				break;
				
			default:
				handled = false;
		};
		break;	
	// Everything else...
	default:
		handled = false;
	}	
	return handled;
}
