/**	
	$Revision: 1.7 $
	Last updated: $Date: 2001/07/24 01:00:04 $

	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
	$Log: EditSplitForm.cpp,v $
	Revision 1.7  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.6  2001/07/16 02:15:57  chaese
	ChooseAccountForm now shows the whole list of accounts (rather than having
	the user navigate through the hierarchy levels) and preselects the current
	setting as specified by the caller.
	
	Revision 1.5  2001/07/15 02:47:55  chaese
	Special handling for the split that refers to the selected account:
	You can't delete it, and assigning it to a different account makes that
	account the selected account.
	
	Revision 1.4  2001/07/14 04:18:02  chaese
	Fix: #435302 and similar memory leaks due to improper frmClose handling
	
	Revision 1.3  2001/07/13 03:12:14  chaese
	prevent same account from being entered on more than one split
	
	Revision 1.2  2001/07/12 03:55:00  chaese
	Don't allow amount to be 0
	
	Revision 1.1  2001/07/09 02:29:39  chaese
	implement splits GUI & other small improvements along the way. The bulk of
	the splits GUI is complete, small data integrity checks are coming soon
	(need to prevent multiple splits for the same account within one transaction
	and stuff like that...)
	
	
*/

#include "EditSplitForm.h"
#include "SplitsForm.h"
#include "ChooseAccountForm.h"
#include "FreeCoinsRsc.h"
#include "AccountsDB.h"

EditSplitForm * EditSplitForm::theForm = NULL;

Boolean EditSplitForm::checkData(void)
{
	Char *s;
	if (!acc_uid) {
		FrmCustomAlert(GENERIC_ERROR, "You must choose an account.",
			NULL, NULL);
		return false;
	}
	s = (Char*)GetTextPtr(EDIT_SPLIT_AMOUNT);
	amount = String2Pence(s)*(is_debit?1:-1);
	if (!validNumberStr(s, false) || amount==0) {
		FrmCustomAlert(GENERIC_ERROR,
			"The amount must be a positive decimal number.", NULL, NULL);
		return false;
	}
	return true;
}

void EditSplitForm::saveData(void)
{
	SplitsForm *spl_frm = SplitsForm::theForm;
	if (split_index==255) {
		split_index = spl_frm->getCount();
		spl_frm->setCount(split_index + 1);
	}
	// if the user changes the split for the currently selected account,
	// change the selected account accordingly. 
	if (spl_frm->getAccount(split_index)==accounts_db->getSelectedUid())
		accounts_db->selectRecord(acc_uid);
	spl_frm->setAccount(split_index, acc_uid);
	spl_frm->setAmount(split_index, amount);
}

Boolean EditSplitForm::eventHandler(EventPtr event)
{
	if (theForm) return theForm->handleEvent(event);
	else return false;
}

Boolean EditSplitForm::handleEvent( EventPtr event )
{
	Boolean           handled = true;
	SplitsForm *spl_frm = SplitsForm::theForm;
	ControlPtr ctl;
	Char *s;
	accountStructure  as;
	UInt8 acc_type;

	// Get the form pointer
	frm = FrmGetActiveForm();

	// Key Events
	switch(event->eType) {

	// Form Open
	case frmOpenEvent:
		if (split_index!=255) {
			acc_uid = SplitsForm::theForm->getAccount(split_index);
			amount = SplitsForm::theForm->getAmount(split_index);
			// don't allow the user to delete the entry for the current
			// register account
			if (acc_uid != accounts_db->getSelectedUid())
				FrmShowObject(frm, FrmGetObjectIndex(frm, EDIT_SPLIT_DELETE));
			FrmCopyTitle(frm, "Edit Split");
		}
		else {
			amount = 0;
			acc_uid = 0;
			FrmCopyTitle(frm, "New Split");
		}

	   	if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
		FrmDrawForm(frm);

		is_debit = (amount>0);
		s = Pence2String(is_debit?amount:-amount);
		SetFieldTextFromStr(EDIT_SPLIT_AMOUNT, s);
		MemPtrFree(s);

		FrmUpdateForm(EDIT_SPLIT_FORM, 0);
		break;

	case ctlSelectEvent:
		switch( event->data.ctlSelect.controlID ) {
			case EDIT_SPLIT_OK:
				if (checkData()) {
					saveData();
					FrmUpdateForm(SPLITS_FORM, 0);
					FrmReturnToForm(0);
				}
				break;
			case EDIT_SPLIT_CANCEL:
				FrmReturnToForm(0);
				break;
			case EDIT_SPLIT_DELETE:
				if (FrmCustomAlert(DELETE_ALERT, "split", NULL, NULL)==1)
					break;
				SplitsForm::theForm->deleteSplit(split_index);
				FrmUpdateForm(SPLITS_FORM, 0);
				FrmReturnToForm(0);
				break;	
			case EDIT_SPLIT_DEBIT:
				is_debit = true;
				break;
			case EDIT_SPLIT_CREDIT:
				is_debit = false;
				break;
			case EDIT_SPLIT_CHOOSE_ACC:
				ChooseAccountForm::theForm->allow_no_selection = false;
				ChooseAccountForm::theForm->chosen = false;
				ChooseAccountForm::theForm->setCallerFormId(EDIT_SPLIT_FORM);
				ChooseAccountForm::theForm->chosen_account = acc_uid;
				FrmPopupForm(CHOOSE_ACCOUNT_FORM);
				break;

			default:
				handled = false;
		}

		break;

	// Form Update
	case frmUpdateEvent:

		if (ChooseAccountForm::theForm->chosen) {
			ChooseAccountForm::theForm->chosen = false;
			if (acc_uid != ChooseAccountForm::theForm->chosen_account) {
				for (UInt8 i=0; i<spl_frm->getCount(); i++) {
					if (spl_frm->getAccount(i)
						==ChooseAccountForm::theForm->chosen_account) {
						ChooseAccountForm::theForm->chosen_account = 0;
						FrmCustomAlert(GENERIC_ERROR,
						"This account is already a part of the transaction.",
							NULL, NULL);
						break;
					}
				}
				if (ChooseAccountForm::theForm->chosen_account)
					acc_uid = ChooseAccountForm::theForm->chosen_account;
			}
		}

		//set account name and debit/credit button labels
		if (acc_uid) {
			accounts_db->getUnpackedRecord(acc_uid, &as);
			acc_type = as.acc_type;
			SysStringByIndex(ACC_DEBIT_DESC, (UInt8)acc_type, debit_label, 32);
			SysStringByIndex(ACC_CREDIT_DESC,(UInt8)acc_type,credit_label, 32);
			accounts_db->getFullNameTruncLeftToFit(acc_uid, 120, acc_name);
		}
		else {
			StrCopy(debit_label, "Debit");
			StrCopy(credit_label, "Credit");
			StrCopy(acc_name, "Not set");
		}
		ctl = (ControlPtr) GetObjectPtr(EDIT_SPLIT_DEBIT);
		CtlSetLabel(ctl, debit_label);
		ctl = (ControlPtr) GetObjectPtr(EDIT_SPLIT_CREDIT);
		CtlSetLabel(ctl, credit_label);
		ctl = (ControlPtr) GetObjectPtr(EDIT_SPLIT_CHOOSE_ACC);
		CtlSetLabel(ctl, acc_name);

		if(event->data.frmUpdate.updateCode == 0 ||
			event->data.frmUpdate.updateCode == frmRedrawUpdateCode) {

		   	if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
			FrmDrawForm(frm);
		}

		// set debit/credit button
		FrmSetControlGroupSelection(frm, 1,
			is_debit?EDIT_SPLIT_DEBIT:EDIT_SPLIT_CREDIT);

		break;
	
	// Everything else...
	default:
		handled = false;
	}	
	return handled;
}
