/**	
	$Revision: 1.17 $
	Last updated: $Date: 2001/07/24 01:00:04 $
	
	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: EditTransactionForm.cpp,v $
	Revision 1.17  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.16  2001/07/18 23:06:29  dizzy
	A few scheduling bugfixes. Plenty of issues left to resolve :-(
	
	Revision 1.15  2001/07/16 02:15:57  chaese
	ChooseAccountForm now shows the whole list of accounts (rather than having
	the user navigate through the hierarchy levels) and preselects the current
	setting as specified by the caller.
	
	Revision 1.14  2001/07/15 05:17:07  chaese
	* display only one balance (expected or cleared) in account table, popup
	  trigger lets user choose.
	* New function PopupListSetSelection to eliminate code duplication
	
	Revision 1.13  2001/07/15 02:47:55  chaese
	Special handling for the split that refers to the selected account:
	You can't delete it, and assigning it to a different account makes that
	account the selected account.
	
	Revision 1.12  2001/07/14 04:18:02  chaese
	Fix: #435302 and similar memory leaks due to improper frmClose handling
	
	Revision 1.11  2001/07/12 03:56:47  chaese
	Don't allow amount to be 0
	Ask for confirmation before recording an unbalanced transaction
	
	Revision 1.10  2001/07/12 01:15:44  chaese
	Fix: Cleared flag wasn't saved correctly since recent code change
	Fix: #437666
	
	Revision 1.9  2001/07/09 02:29:39  chaese
	implement splits GUI & other small improvements along the way. The bulk of
	the splits GUI is complete, small data integrity checks are coming soon
	(need to prevent multiple splits for the same account within one transaction
	and stuff like that...)
	
	Revision 1.8  2001/07/06 03:03:35  chaese
	First baby steps toward a GUI for split transactions. Viewing works,
	editing doesn't (yet).
	
	Revision 1.7  2001/07/02 15:18:24  chaese
	bugfix to vlad's bugfix :) (Negative transaction amounts need to be reversed
	before they are displayed, not after.)
	
	Revision 1.6  2001/07/02 14:03:53  dizzy
	Horde of bugfixes to the transaction editing routines...
	
	Revision 1.5  2001/07/01 12:21:55  dizzy
	Merged ScheduledDB into TransactionsDB
	
	Revision 1.4  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.3  2001/06/26 23:30:37  dizzy
	Forms are now classes rather then namespaces. This should enable future code optimisations
	
	Revision 1.2  2001/06/26 01:27:50  chaese
	fixed redraw problem
	
	Revision 1.1  2001/06/25 10:42:35  dizzy
	Change file extensions
	
	Revision 1.23  2001/06/23 04:51:48  chaese
	fixed bug #435294 and minor cleanups
	
	Revision 1.22  2001/06/21 11:04:23  chaese
	* initial support for account types
	* fixed some obscure bugs that materialized on a PalmOS 3.5 debug rom
	(see related message on palmcoins-misc for details)
	
	Revision 1.21  2001/06/17 05:23:32  chaese
	*** CAUTION: DATABASE STRUCTURE HAS CHANGED YET AGAIN ***
	I generally don't like 'cross-logging', but too many changes have accumulated
	to keep them separated.
	1. completely eliminated account categories
	2. introduced currently unused type field in the account record structure
	(This will become what the user sees as 'category selector'. Coming soon!)
	3. completely eliminated protected accounts (mostly because #1 made it so much
	harder to locate the "Misc" account in the Expense category:) Those should be
	replaced with Pref. settings for default Income and Expense accounts anyway.
	4. changes to user interface; some as mere consequences of 1. (e.g.
	Account chooser), some for usability reasons (Transaction form).
	5. bugfixes and minor improvements.
*/

#include "EditTransactionForm.h"

#include "SplitsDB.h"
#include "PreferencesForm.h"
#include "CoinsPref.h"
#include "SplitsForm.h"

#include "FreeCoinsRsc.h"
#include "AccountsDB.h"

Boolean EditTransactionForm::checkData()
{
	SplitsForm *spl_frm = SplitsForm::theForm;
	Char *s;
	UInt8 i;
	Int32 split_total = 0;

	saveToSplits();

	// check the amount
	s = (Char*)GetTextPtr(EDIT_TRANS_AMMOUNT);
	if (!s || !validNumberStr(s, false) ||
	spl_frm->getAmount(this_split)==0)
	{
		FrmCustomAlert(GENERIC_ERROR,
			"The amount must be a positive decimal number.", NULL, NULL); 	
		return false;
	}

	// check if the transaction is balanced
	for (i=0; i<spl_frm->getCount(); i++)
		split_total += spl_frm->getAmount(i);
	if (split_total!=0) {
		if (FrmCustomAlert(GENERIC_YES_NO,
			"This transaction is unbalanced. Do you really want to record it?",
			NULL, NULL)==1)
			return false;
	}

	return true;
}

void EditTransactionForm::save()
{	
	splitStructure spl;
	Boolean cleared;
	UInt32 trans_uid;
	UInt8 index;
	
	t.num_splits = SplitsForm::theForm->getCount();

	t.payee=(Char*) GetTextPtr(EDIT_TRANS_PAYEE);
	if(!t.payee) t.payee="n/a";

	t.num = (Char*) GetTextPtr(EDIT_TRANS_NUM);
	if(!t.num) t.num = "";
		
	// until we have note support, clear the note string. Fixes bug #437666
	t.note = "";

	t.method=(payment_method)LstGetSelection(
		(ListPtr)GetObjectPtr(EDIT_TRANS_TYPE_LIST));

	if (!record_is_new) {
		transactions_db->deleteRecord(transactions_db->getSelectedRecord());
	}

	//this sets the cleared state and creates the new record. for scheduled records the cleared field will
	//indicate whether the record will be cleared or not when introduced
	cleared = FrmGetControlValue(frm,
		FrmGetObjectIndex(frm,EDIT_TRANS_CLEARED_CBX));
	t.to_be_cleared = cleared;
	trans_id = transactions_db->newRecord(&t,t.to_be_cleared,trans_is_sched);
	trans_uid = transactions_db->getRecordUid(trans_id);	

	spl.trans_uid = trans_uid;
	for (index = 0; index<t.num_splits; index++) {
		spl.line_no = index + 1;
		spl.acc_uid = SplitsForm::theForm->getAccount(index);
		spl.amount = SplitsForm::theForm->getAmount(index);
		splits_db->newRecord(&spl, cleared);
	}

	saved=true;
}

void EditTransactionForm::setDefaults()
{
	ControlPtr  ctl;
	Int32         amount;
	UInt32      trans_uid = 0, acc_uid = 0;
	AccountType       acc_type;
	accountStructure  as;
	splitStructure spl;
	UInt16 count, index;
	UInt8 split_index;

	SplitsForm *spl_frm = SplitsForm::theForm;

	acc_uid = accounts_db->getSelectedUid();
	accounts_db->getUnpackedRecord(acc_uid, &as);
	acc_type = as.acc_type;

   //set debit/credit button labels
   SysStringByIndex(ACC_DEBIT_DESC, (UInt8)acc_type, debit_label, 32);
	ctl = (ControlPtr) GetObjectPtr(EDIT_TRANS_DEBIT);
	CtlSetLabel(ctl, debit_label);
	SysStringByIndex(ACC_CREDIT_DESC, (UInt8)acc_type, credit_label, 32);
	ctl = (ControlPtr) GetObjectPtr(EDIT_TRANS_CREDIT);
	CtlSetLabel(ctl, credit_label);
   
   //let the scheduled form know that we started a new session
   edit_scheduled_trans_form->newSession(&t.sched, &t.date);

	if (!record_is_new) { //we are editing an existing record
      transactions_db->getUnpackedRecord(&t);
      
      //get the uid and amount
  		trans_uid = transactions_db->getSelectedUid();
		amount = transactions_db->getAmount(trans_uid, acc_uid);
		is_debit = (amount>0);

		count = t.num_splits;	
		spl_frm->setCount(count);
		split_index = 0;
	    for (index=splits_db->locateFirstSplit(trans_uid);count>0;
			index++,split_index++,count--)
		{
			splits_db->getUnpackedRecord(index, &spl);
			spl_frm->setAccount(split_index, spl.acc_uid);
			spl_frm->setAmount(split_index, spl.amount);
		}

		//set cleared status
		if(transactions_db->isCleared(transactions_db->getSelectedRecord()))
			FrmSetControlValue(frm,FrmGetObjectIndex(frm,EDIT_TRANS_CLEARED_CBX),1);
		else 
			FrmSetControlValue(frm,FrmGetObjectIndex(frm,EDIT_TRANS_CLEARED_CBX),0);

		//date
		updateDateField(EDIT_TRANS_CHOOSE_DATE,t.date,date_txt);				

		//now we can fill in the text fields
		SetFieldTextFromStr(EDIT_TRANS_PAYEE,t.payee);
		SetFieldTextFromStr(EDIT_TRANS_NUM,t.num);
      
      //if our record is not new and was marked as scheduled we can edit its scheduled information
      if(transactions_db->isScheduled(transactions_db->getSelectedRecord())) {
         edit_scheduled_trans_form->setRecordIsNew(false);
         trans_is_sched = true; //let this form know that the transaction is scheduled
      }

      FrmShowObject(frm, FrmGetObjectIndex(frm, EDIT_TRANS_DELETE));
      FrmCopyTitle(frm, "Transaction Details");
      saved=true; //record was saved already in a prev session      
      
	} else {
     	amount = 0;
		is_debit = (acc_type==EXPENSE);

		spl_frm->setCount(1);
		spl_frm->setAccount(0, acc_uid);
		spl_frm->setAmount(0, 0);

		DateSecondsToDate(TimGetSeconds(),&t.date);
		t.to_be_cleared = preferences.get()->trans_cleared;
		t.payee = "";
		t.num = "";
		t.note = "";
		peer_acc = 0;
		t.num_splits = 1;
		t.method = pm_cheque;
		CtlSetLabel((ControlType*) GetObjectPtrByForm(FrmGetActiveForm(),
			EDIT_TRANS_CHOOSE_ACC),"Not set");
		FrmSetControlValue(frm,FrmGetObjectIndex(frm,EDIT_TRANS_CLEARED_CBX),t.to_be_cleared);		
		updateDateField(EDIT_TRANS_CHOOSE_DATE,t.date,date_txt);				
		FrmCopyTitle(frm, "Create New Transaction");
		saved=false; //only starting; didnt save anything yet
	}

	// update peer account and amount
	readFromSplits();

   //fill in and select the right element in the transaction type list   
	PopupListSetSelection(EDIT_TRANS_TYPE_LIST, EDIT_TRANS_TYPE_TRIGGER,
		(Int16)t.method);
}

void EditTransactionForm::readFromSplits(void)
{
	UInt8 index, count;
	UInt32 spl_acc, acc_uid;
	SplitsForm *spl_frm = SplitsForm::theForm;
	Int32 splits_total = 0;
	Int32 this_amount = 0;
	Char *s;

	acc_uid = accounts_db->getSelectedUid();
	count = spl_frm->getCount();
	t.num_splits = count;
	for (index=0; index<count; index++) {
		splits_total += spl_frm->getAmount(index);
		spl_acc = spl_frm->getAccount(index);
		if (spl_acc==acc_uid) {
			this_split = index;
			this_amount = spl_frm->getAmount(index);
		}
		else {
			peer_split = index;
			peer_acc = spl_acc;
		}
	}
	if (splits_total==0 && count==2)
		trans_class = 1;
	else if (count==1) trans_class = 0;
	else trans_class = 2;		

	switch (trans_class) {
		case 1:
			accounts_db->getFullNameTruncLeftToFit(peer_acc, 120, acc_name);	
			break;
		case 0:
			StrCopy(acc_name, "Not set");
			peer_split = peer_acc = 0;
			break;
		case 2:
			StrCopy(acc_name, "-- Split --");
			peer_split = peer_acc = 0;
			break;
	}
	CtlSetLabel((ControlType*) GetObjectPtrByForm(FrmGetActiveForm(),
		EDIT_TRANS_CHOOSE_ACC),acc_name);

	//select the right radio button
	if (this_amount>0)
		is_debit = true;
	if (this_amount<0)
		is_debit = false;
	if (is_debit)
		FrmSetControlGroupSelection(frm,1,EDIT_TRANS_DEBIT);
	else
		FrmSetControlGroupSelection(frm,1,EDIT_TRANS_CREDIT);

	if (this_amount != 0) {
		s = Pence2String(is_debit?this_amount:-this_amount);
		SetFieldTextFromStr(EDIT_TRANS_AMMOUNT,s);
		MemPtrFree(s); 
	}
}

void EditTransactionForm::saveToSplits(void)
{
	SplitsForm* spl_frm = SplitsForm::theForm;
	Int32 amount;

	amount = String2Pence((Char*) GetTextPtr(EDIT_TRANS_AMMOUNT));
	if (!is_debit) amount *= -1;

	spl_frm->setAmount(this_split, amount);
	if (peer_acc && peer_split) {
		spl_frm->setAccount(peer_split, peer_acc);
		spl_frm->setAmount(peer_split, -amount);
	}
}

Boolean EditTransactionForm::handleEvent(EventPtr event)
{
	Boolean 					handled = true;

	// Get the form pointer
	frm = FrmGetActiveForm();
	choose_account_form->allow_no_selection = false;
		
	// Key Events
	switch(event->eType) {

	// Form Update
	case frmUpdateEvent:
		if(choose_account_form->chosen) {
			choose_account_form->chosen = false;
			peer_acc = choose_account_form->chosen_account;
			if (trans_class==0) {
				SplitsForm::theForm->setCount(2);
				peer_split = 1;
				trans_class = 1;
			}
			saveToSplits();
			readFromSplits();
		}
      //update the date field (could have been changed by editscheduledtransform)
  		updateDateField(EDIT_TRANS_CHOOSE_DATE,t.date,date_txt);		
      if(edit_scheduled_trans_form->getSaved()) //if user saved the scheduled form, this trans is scheduled  
         trans_is_sched = true;		
		if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
		FrmDrawForm(frm);
		if (event->data.frmUpdate.updateCode & 1)
			readFromSplits();
        break;
  	
	case frmCloseEvent:
		SplitsForm::theForm->setCount(0);
		handled = false;
		break;

	// Form Open
	case frmOpenEvent:		
		//draw form
		FrmDrawForm(frm);
  		setDefaults(); //this will initialise the scheduled form
		// remember what account we were in when we started this form
     	prev_acc_uid = accounts_db->getSelectedUid();
 
      /**note that no transasction can be scheduled by default. A transaction is scheduled when:
            1. its saved and it's category is set to scheduled
            2. its scheduling information was edited and saved (no matter whether the tranasction itself is new or old)
      */      
		break;
	
	// Controls
	case ctlSelectEvent:
		switch( event->data.ctlSelect.controlID ) {
			case EDIT_TRANS_OK:
				//if something is wrong, an error message has already been
				//issued by checkData, so do nothing.
				if(!checkData())
					break;
				save();
                if(trans_is_sched) {
    				trans_is_sched=false; //set the scheduled flag to false for next call
                    transactions_db->schedule(true); //force the scheduling to happen now
                }
                
				// free the temporary splits array
				SplitsForm::theForm->setCount(0);
				//update the main form and return to the previous form
				FrmUpdateForm(TRANSACTIONS_FORM,0);				
				FrmReturnToForm(0);
				break;
			case EDIT_TRANS_DEBIT: //the radio buttons
				is_debit=true;
				break;
			case EDIT_TRANS_CREDIT:
				is_debit=false;
				break;				
			case EDIT_TRANS_CHOOSE_DATE:
				selectDate(t.date,EDIT_TRANS_CHOOSE_DATE,date_txt);
				break;
			case EDIT_TRANS_CHOOSE_ACC:
				if (trans_class==2) {
					saveToSplits();
					FrmPopupForm(SPLITS_FORM);
				}
				else {
					choose_account_form->setCallerFormId(EDIT_TRANS_FORM);
					choose_account_form->chosen = false;
					choose_account_form->chosen_account = peer_acc;
					FrmPopupForm(CHOOSE_ACCOUNT_FORM);		
				}
				break;
			case EDIT_TRANS_CANCEL:
				//cancel button was pressed;
				//simply return to the main form
				trans_is_sched=false;				
				SplitsForm::theForm->setCount(0);
				// playing with the splits may have changed the selected acct.
				// undo that.
				accounts_db->selectRecord(prev_acc_uid);
				FrmUpdateForm(caller_form_id,0);
				FrmReturnToForm(0);
				break;
			case EDIT_TRANS_DELETE:
				//delete button was pressed;
				//simply return to the main form
				// We shouldn't get here if the transactions is new,
				// but just to be safe...
				if (record_is_new) break;
				// ask for confirmation
				if (FrmCustomAlert(DELETE_ALERT, "transaction", NULL, NULL)==1)
					break;
				trans_is_sched=false;
				transactions_db->deleteRecord(transactions_db->getSelectedRecord());
				SplitsForm::theForm->setCount(0);
				FrmUpdateForm(caller_form_id,0);				
				FrmReturnToForm(0);
				break;
         case EDIT_TRANS_SCHED:
				FrmPopupForm(EDIT_SCHED_TRANS_FORM);		
            break;
            
		case EDIT_TRANS_RECORD_SPLITS:
			saveToSplits();
			FrmPopupForm(SPLITS_FORM);
            break;
				
		default:
			handled = false;
		};
		break;	
	// Everything else...
	default:
		handled = false;
	}	
	return handled;
}
