/**	
	$Revision: 1.13 $
	Last updated: $Date: 2001/07/15 02:47:55 $
	
	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: EditTransactionForm.h,v $
	Revision 1.13  2001/07/15 02:47:55  chaese
	Special handling for the split that refers to the selected account:
	You can't delete it, and assigning it to a different account makes that
	account the selected account.
	
	Revision 1.12  2001/07/12 03:56:47  chaese
	Don't allow amount to be 0
	Ask for confirmation before recording an unbalanced transaction
	
	Revision 1.11  2001/07/09 02:29:39  chaese
	implement splits GUI & other small improvements along the way. The bulk of
	the splits GUI is complete, small data integrity checks are coming soon
	(need to prevent multiple splits for the same account within one transaction
	and stuff like that...)
	
	Revision 1.10  2001/07/01 12:21:55  dizzy
	Merged ScheduledDB into TransactionsDB
	
	Revision 1.9  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.8  2001/06/26 23:30:37  dizzy
	Forms are now classes rather then namespaces. This should enable future code optimisations
*/

#ifndef __EDIT_TRANSACTION_FORM__
#define __EDIT_TRANSACTION_FORM__

#include <PalmOS.h>

#include "Form.h"
#include "ChooseAccountForm.h"
#include "EditScheduledTransForm.h"
#include "TransactionsDB.h"
#include "MiscFunc.h" //code section defs

///the form is used to create a new transaction or to modify an existing one, depending on the record_is_new member
class EditTransactionForm : public Form
{
 private:
	/// this is set to true if we want to create a new account and to false if we want the 'details' functionality
	Boolean record_is_new;
	/// set to true when creating/editing a scheduled transaction
	Boolean trans_is_sched;

	///contains the id of the last created record
	UInt16 trans_id;

   /// true if the current transaction is a debit
	Boolean	is_debit;	

   ///structure used to hold data entered by user
	transactionStructure 	t;
	MemHandle				rech;
   
	//the peer account for this transaction
	UInt32					peer_acc;
   ///name of the account chosen as peer
	Char					acc_name[60];
   ///date text
	Char					date_txt[dateStringLength];
	
   ///label for debit button
	Char debit_label[32];
   ///label for credit button
	Char credit_label[32];
	
   ///the choose account form can be accessed through this pointer
	ChooseAccountForm* choose_account_form;
   ///transaction scheduling data is accessed through this pointer
   EditScheduledTransForm* edit_scheduled_trans_form;

	/// classify current configuration 
	UInt8 trans_class;
	/// index of the split that affects the current account
	UInt8 this_split;
	/// index of the split that affects the peer account, if applicable
	UInt8 peer_split;

	/// remember which account was selected when form was opened
	UInt32 prev_acc_uid;

	public:
			
	EditTransactionForm(ChooseAccountForm* c, EditScheduledTransForm* s) : choose_account_form(c), edit_scheduled_trans_form(s) {}
	UInt16	getTransId() { return trans_id; }
	void 	setTransId(UInt16 i) { trans_id=i; }
	void setTransIsSched(Boolean b) { trans_is_sched = b; }
	void setRecordIsNew(Boolean b) { record_is_new = b; }

	/// pre-save data check, returns true if data is good, false otherwise.
	Boolean checkData(void) FORM_SECTION;
	
	///saves the transaction data and updates all the accounts involved
	void save() FORM_SECTION;

	///set the values for some of the fields in the form (from the transaction record)
	void setDefaults() FORM_SECTION;
	
	/// reflect peer account and transaction amount in underlying splits array
	void saveToSplits(void) FORM_SECTION;

	/// update amount and peer account according to the underlying splits array
	void readFromSplits(void) FORM_SECTION;

	///displays the name for the given account
	void	setPeerAccount(UInt32 account) FORM_SECTION;
	
	/// event handler for the form
	Boolean handleEvent(EventPtr event) FORM_SECTION;
};
#endif
