/**	
	$Revision: 1.2 $
	Last updated: $Date: 2001/07/24 10:49:17 $

	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
	$Log: FreeCoins.cpp,v $
	Revision 1.2  2001/07/24 10:49:17  chaese
	fix #include reference
	
	Revision 1.1  2001/07/24 08:50:23  dizzy
	renamed files to remove refernces to PalmCoins
	
	Revision 1.11  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.10  2001/07/15 11:38:12  dizzy
	Added interface for purging transactions
	
	Revision 1.9  2001/07/15 06:53:59  chaese
	Fix: #441393. To make things less convoluted, the preferences store the
	transaction category without offset, and the offset is used for displaying
	in the preferences form only.
	
	Revision 1.8  2001/07/09 02:29:39  chaese
	implement splits GUI & other small improvements along the way. The bulk of
	the splits GUI is complete, small data integrity checks are coming soon
	(need to prevent multiple splits for the same account within one transaction
	and stuff like that...)
	
	Revision 1.7  2001/07/06 03:03:35  chaese
	First baby steps toward a GUI for split transactions. Viewing works,
	editing doesn't (yet).
	
	Revision 1.6  2001/07/02 11:03:04  dizzy
	Cleaned up TransactionsForm.
	Removed details button (tap will take user to details)
	Unbalanced transactions are marked
	The total in the Scheduled view is the total of all scheduled transactions (as opposed to the account balance)
	
	Revision 1.5  2001/07/01 12:21:55  dizzy
	Merged ScheduledDB into TransactionsDB
	
	Revision 1.4  2001/06/29 00:24:57  chaese
	Rewrote how total balances are tracked. This should eliminate all problems
	that were associated with the old way (i.e. balances not being recalculated
	when they should have been etc.)
	
	Revision 1.3  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.2  2001/06/26 23:30:37  dizzy
	Forms are now classes rather then namespaces. This should enable future code optimisations
	
	Revision 1.1  2001/06/25 10:42:35  dizzy
	Change file extensions
*/

#include "FreeCoins.h"

#include "CoinsPref.h"

#include "AccountsDB.h" 
#include "TransactionsDB.h" 
#include "SplitsDB.h"

#include "AccountsForm.h" 
#include "EditAccountForm.h"
#include "TransactionsForm.h"
#include "EditTransactionForm.h"
#include "PreferencesForm.h"
#include "SplitsForm.h"
#include "EditSplitForm.h"
#include "PurgeTransForm.h"
#include "FreeCoinsRsc.h"

TransactionsDB trsdb;
AccountsDB accdb;
SplitsDB spldb;

/* all forms are global; this is because the event handler function used to load forms has to be present at the 
same location for all the duration of the run of the program. If the forms would be allocated only when needed, handleEvent
would be placed at different addresses each time, making the  operation of ApplicationHandleEvent extremly difficult
*/
ChooseAccountForm choose_form;
Boolean CAF_handleEvent(EventPtr e) { return choose_form.handleEvent(e); } 

EditAccountForm edit_acc_form(&choose_form);
Boolean EAF_handleEvent(EventPtr e) { return edit_acc_form.handleEvent(e); } 

EditScheduledTransForm edit_sched_trans_form;
Boolean ESTF_handleEvent(EventPtr e) { return edit_sched_trans_form.handleEvent(e); } 

EditTransactionForm edit_trans_form(&choose_form, &edit_sched_trans_form);
Boolean ETF_handleEvent(EventPtr e) { return edit_trans_form.handleEvent(e); } 

TransactionsForm trans_form(&edit_acc_form, &edit_trans_form);
Boolean TF_handleEvent(EventPtr e) { return trans_form.handleEvent(e); } 

AccountsForm accs_form(&edit_acc_form, &edit_trans_form);
Boolean AF_handleEvent(EventPtr e) { return accs_form.handleEvent(e); } 

PreferencesForm preferences_form(&choose_form);
Boolean PF_handleEvent(EventPtr e) { return preferences_form.handleEvent(e); } 

PurgeTransForm purge_form;
Boolean PTF_handleEvent(EventPtr e) { return purge_form.handleEvent(e); } 

SplitsForm      splits_form;
EditSplitForm  edit_split_form;

Boolean ApplicationHandleEvent( EventPtr event )
{
	Boolean	eventHandled = true;
	UInt16 formID;
	FormPtr frm;
	
	switch (event->eType) {
	case frmLoadEvent:
		// Load the form resource.		
		formID = event->data.frmLoad.formID;
		frm = FrmInitForm(formID);

		FrmSetActiveForm(frm);		

		// set the event handler for the form. the handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		//
		// for this app, all forms are handled by the same routine.
			
		switch( formID ){
			case ACCOUNTS_MAIN_FORM:
				FrmSetEventHandler(frm, AF_handleEvent);
				break;
			case EDIT_ACC_FORM:
				FrmSetEventHandler(frm,  EAF_handleEvent);
				break;
			case CHOOSE_ACCOUNT_FORM:
				FrmSetEventHandler(frm, CAF_handleEvent);
				break;
			case PREF_FORM:
				FrmSetEventHandler(frm, PF_handleEvent);
				break;
			case TRANSACTIONS_FORM:
				FrmSetEventHandler(frm, TF_handleEvent);
				break;
			case EDIT_TRANS_FORM:
				FrmSetEventHandler(frm, ETF_handleEvent);
				break;
			case EDIT_SCHED_TRANS_FORM:
				FrmSetEventHandler(frm, ESTF_handleEvent);
				break;					
			case PURGE_FORM:
				FrmSetEventHandler(frm, PTF_handleEvent);
				break;
			case SPLITS_FORM:
				FrmSetEventHandler(frm, SplitsForm::eventHandler);
				break;
			case EDIT_SPLIT_FORM:
				FrmSetEventHandler(frm, EditSplitForm::eventHandler);
				break;

			// if you add more forms to the app, put a case in for each new
			// form...
			default:
				eventHandled = false;
				break;				
		}
		break;

	case frmCloseEvent:
		frm = FrmGetFormPtr(event->data.frmClose.formID);
		FrmEraseForm(frm);
		FrmDeleteForm(frm);
		eventHandled = true;
        break;

	default:
		eventHandled = false;
	}
	
	return eventHandled;
}


void ApplicationEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);
		if(!SysHandleEvent(&event)) //not system event
			if(!MenuHandleEvent(NULL,&event,&error))
				if(!ApplicationHandleEvent(&event)) //our event handling function
					FrmDispatchEvent(&event);
	} while (event.eType != appStopEvent);
}

void StartApplication( void )
{
	UInt32 datePref;

	// init form pointers
	SplitsForm::theForm = &splits_form;
	EditSplitForm::theForm = &edit_split_form;
	ChooseAccountForm::theForm = &choose_form;

	//init the database pointers
	transactions_db = &trsdb;
	splits_db=&spldb;
	//set the db pointer to the right addresses
	accounts_db = &accdb;
	if(accounts_db->isNew()) accounts_db->initializeAccounts();

	LocGetNumberSeparators(
		(NumberFormatType) PrefGetPreference(prefNumberFormat),
		&MiscFunc::cThousandSep, &MiscFunc::cDecimalSep);	
	datePref = PrefGetPreference(prefDateFormat);
	switch(datePref) {
	case dfDMYWithSlashes: MiscFunc::bDayFirst = true; MiscFunc::cDateSep='/';
		break;
	case dfDMYWithDots: MiscFunc::bDayFirst = true; MiscFunc::cDateSep='.';
        break;
	case dfDMYWithDashes: MiscFunc::bDayFirst = true; MiscFunc::cDateSep='-';
        break;
	default:
	case dfYMDWithSlashes:
	case dfMDYWithSlashes: MiscFunc::bDayFirst = false; MiscFunc::cDateSep='/';
        break;
	case dfYMDWithDashes: MiscFunc::bDayFirst = false; MiscFunc::cDateSep='-';
        break;              
	case dfYMDWithDots: MiscFunc::bDayFirst = false; MiscFunc::cDateSep='.';
        break;              
	}

	//update scheduled transactions
	transactions_db->schedule();

	accounts_db->updateVisibility();
}


void StopApplication( void )
{
	// Close all open forms,  this will force any unsaved data to
	// be written to the database.
	FrmCloseAllForms();
	preferences.save();
}

/*
 * 	Application entry point
 */
UInt32	PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags )
{

	// If normal launch
	if ( cmd == sysAppLaunchCmdNormalLaunch )
	{
		//we can start the application
		 StartApplication();

		// first form.
		FrmGotoForm(ACCOUNTS_MAIN_FORM);
		
		// run the event loop. returns to exit.		
		ApplicationEventLoop();
		StopApplication();
	}
	return 0;
}	

