/**	
	$Revision: 1.3 $
	Last updated: $Date: 2001/07/15 05:17:07 $
	
	$Log: MiscFunc.cpp,v $
	Revision 1.3  2001/07/15 05:17:07  chaese
	* display only one balance (expected or cleared) in account table, popup
	  trigger lets user choose.
	* New function PopupListSetSelection to eliminate code duplication
	
	Revision 1.2  2001/07/09 02:29:39  chaese
	implement splits GUI & other small improvements along the way. The bulk of
	the splits GUI is complete, small data integrity checks are coming soon
	(need to prevent multiple splits for the same account within one transaction
	and stuff like that...)
	
	Revision 1.1  2001/06/25 10:42:35  dizzy
	Change file extensions
	
	Revision 1.28  2001/06/22 23:19:39  chaese
	rewrote Pence2String to fix bug #435295
	
	Revision 1.27  2001/06/21 11:04:23  chaese
	* initial support for account types
	* fixed some obscure bugs that materialized on a PalmOS 3.5 debug rom
	(see related message on palmcoins-misc for details)
	
	Revision 1.26  2001/06/13 01:15:27  chaese
	Pence2String produced leading space for positive quantities. Not no more!
	
	Revision 1.25  2001/06/12 03:28:35  chaese
	Shaved off code in date and currency formatting (amazing what you can do
	with StrPrintF! :)
	
	Revision 1.24  2001/06/11 11:03:21  chaese
	Increased display speed in account and transaction tables:
	* Associate table rows with the record index rather than UID. (Minor gain)
	* Perform preference lookup at program startup rather than every time a
	date or number is drawn. (Major gain! That was really stupid of me to put
	the lookups into the drawing function in the first place!)
	
	Revision 1.23  2001/05/14 17:15:55  dizzy
	Parent accounts display total in children accounts when collapsed
	The program displays a message while creating the sample accounts on startup
	Quick transaction button (together with default account selection)
	Can choose default cleared state for transactions
	Fixed bug 407185 (adding scheduled transactions)
	
	Revision 1.22  2001/04/23 18:17:57  dizzy
	New account browser form
	Transactions use the new account browser
	Can create sub-accounts
	Replaced the fake indentation with real one - based on depth
	
	Revision 1.21  2001/03/05 20:06:28  dizzy
	Updates to the scheduling code
	
	Revision 1.20  2001/01/31 18:07:24  dizzy
	More updates to the scheduling code (still not working though :( )
	Moved all Form code in a separate code section
	
	Revision 1.19  2001/01/29 18:49:30  dizzy
	Put code for scheduled transactions in a separate code section to enable the linking process to succeed.
	Scheduled trans info can be add/deleted/updated
	Added cleared field to trans structure
	
	Revision 1.18  2001/01/27 15:30:33  dizzy
	Added a function to update the text in a field from an int.
	
	Revision 1.17  2000/12/13 13:46:04  dizzy
	Fixed bug which caused a crash each time a transaction was updated
	PalmOS < 3.5 compatibility code (now the program seems to run on PalmOS 3.1)
	
	Revision 1.16  2000/12/11 16:27:25  dizzy
	C++ version of the code.
	
	Revision 1.15  2000/12/03 13:05:12  dizzy
	Compiles cleanily with the c++ compiler. Some bits&pieces translated to c++
	
	Revision 1.14  2000/11/29 18:19:10  dizzy
	Fixed Bug #123651 (the account list not initialised properly when creating a new transaction)
	Corrected spelling mistake ('amount' as opposed to 'ammount')
	
	Revision 1.13  2000/11/16 18:21:59  dizzy
	Added new fields to the Account structure.
	
	Revision 1.12  2000/11/15 18:58:42  dizzy
	New balance adjust option in transaction form menu.
	Help updated in Transaction form and Accounts form.
	Merged ShowScrollArrows functions.
	Updated a few variable and function names.
	
	Revision 1.11  2000/11/06 22:27:02  dizzy
	Numbers are now localised and alligned to the right; the dates are displayed using the default format (these changes were submitted by Carsten Haese
	
	Revision 1.10  2000/10/27 22:29:20  dizzy
	All records are sorted now; fake indentation in account view
	
	Revision 1.9  2000/10/09 21:30:19  dizzy
	Fixed a bug which prevented having the - sign displayed for amounts under 1
	

	Copyright (C) 2000 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	
*/

#include "MiscFunc.h"
namespace MiscFunc
{
	char cThousandSep;
	char cDecimalSep;
	Boolean bDayFirst;
	char cDateSep;
};

Boolean MiscFunc::osVerGe(UInt32 version_no)
{
	UInt32 dev_ver;
	FtrGet(sysFtrCreator,sysFtrNumROMVersion, &dev_ver);
	if(dev_ver<version_no) return false;
	return true;
}

Char*	MiscFunc::Pence2String(Int32 pence)
{
	Char* s = (Char*) MemPtrNew(30), *t;

	t = s;
	if (pence<0) {
		pence *= -1;
		*t++ = '-';
	}
	StrPrintF(t, "%ld%c%02ld", pence/100, cDecimalSep, pence%100);
	return s;
}

void MiscFunc::DrawCharsRightAlign(const Char* s, const RectangleType* bounds)
{
	Int16 offset;
	RectangleType oldcliprect, newcliprect;

	WinGetClip(&oldcliprect);
	RctGetIntersection(bounds, &oldcliprect, &newcliprect);
	WinSetClip(&newcliprect);
	offset = bounds->extent.x - FntCharsWidth(s, StrLen(s)) - 1;
	if (offset<0) offset = 0;
	WinDrawChars(s,StrLen(s),bounds->topLeft.x+offset, bounds->topLeft.y);
	WinSetClip(&oldcliprect);
}
 
Int32	MiscFunc::String2Pence(Char* sum)
{
	Char work[32];
	Char* dot;
	Char cThousandSep, cDecimalSep;
	Int32 i;

	//returns 0 if the pointer is 0
	if(!sum) return 0;

	LocGetNumberSeparators((NumberFormatType) PrefGetPreference(prefNumberFormat),&cThousandSep, &cDecimalSep);
	StrDelocalizeNumber(sum, cThousandSep, cDecimalSep);
	MemSet(work,32,0);
	StrCopy(work, sum);
	dot = StrChr(work, '.');
	if (!dot) StrCat(work, "00");
	else {
		for (i=0; i<2; i++) dot[i] = dot[i+1];
		if (dot[1]<'0'||dot[1]>'9') dot[1] = '0';
		if (dot[0]<'0'||dot[0]>'9') dot[0] = '0';
		dot[2] = 0;
	}
	return StrAToI(work);
}

void*	MiscFunc::GetObjectPtr(UInt16 objID)
{
	FormPtr frm;
	frm=FrmGetActiveForm();
	return (FrmGetObjectPtr (frm, FrmGetObjectIndex (frm,objID)));
}

void*	MiscFunc::GetObjectPtrByForm(FormPtr frm,UInt16 objID)
{
	return (FrmGetObjectPtr (frm, FrmGetObjectIndex (frm,objID)));
}

void*	MiscFunc::GetTextPtr(UInt16 objID)
{
	FieldPtr fp=(FieldPtr) GetObjectPtr(objID);
	
	return FldGetTextPtr(fp);
}

FieldPtr MiscFunc::SetFieldTextFromHandle(Int16 fieldID, MemHandle txtH)
{
   MemHandle	oldTxtH;
   FormPtr	frm = FrmGetActiveForm();
   FieldPtr	fldP;


   // get the field and the field's current text handle.
   fldP = (FieldPtr) GetObjectPtrByForm(frm, fieldID);
   ErrNonFatalDisplayIf(!fldP, "Cannot set text field: missing field");
   oldTxtH = FldGetTextHandle(fldP);
   
   // set the field's text to the new text.
   FldSetTextHandle(fldP, txtH);
   FldDrawField(fldP);

   // free the handle AFTER we call FldSetTextHandle().
   if (oldTxtH) 
      MemHandleFree(oldTxtH);
   
   return fldP;
} 

FieldPtr MiscFunc::SetFieldTextFromStr(Int16 fieldID, const Char* str)
{
   MemHandle      txtH;
   
   // get some space in which to stash the string.
   txtH  = MemHandleNew(StrLen(str) + 1);
   if (!txtH)
      return NULL;

   // copy the string to the locked handle.
   StrCopy((Char*) MemHandleLock(txtH), str);

   // unlock the string handle.
   MemHandleUnlock(txtH);
   
   // set the field to the handle
   return SetFieldTextFromHandle(fieldID, txtH);
} 

FieldPtr MiscFunc::SetFieldTextFromInt(Int16 fieldID, const Int32 i)
{
   MemHandle      txtH;
   
   // get some space in which to stash the string.
   txtH  = MemHandleNew(16);
   if (!txtH) 
      return NULL;

   // copy the string to the locked handle.
   StrIToA((Char*) MemHandleLock(txtH),i);

   // unlock the string handle.
   MemHandleUnlock(txtH);
   
   // set the field to the handle
   return SetFieldTextFromHandle(fieldID, txtH); 
} 

//clears the text from a field:
void  MiscFunc::ClearFieldText(Int16 fieldID)
{
   SetFieldTextFromHandle(fieldID, NULL);
}

void MiscFunc::updateDateField(UInt16 date_field_id, DateType new_date, Char* date_txt)
{
	DateFormatType	date_format;
	//stores the date text
		
	date_format = (DateFormatType) PrefGetPreference(prefDateFormat);
	DateToAscii(new_date.month, new_date.day, new_date.year+1904, date_format, date_txt);
	CtlSetLabel((ControlType*) GetObjectPtrByForm(FrmGetActiveForm(),date_field_id),date_txt);
}

 
void MiscFunc::selectDate(DateType& dest, UInt16 date_field_id, Char* date_txt)
{
	Boolean date_changed;
	Int16 	year,month,day;
	
	year=dest.year+1904;
	month=dest.month;
	day=dest.day;
	
	date_changed=SelectDay(selectDayByDay,&month,&day,&year,"Choose date");
	if(date_changed) {
		dest.year=year-1904;
		dest.month=month;
		dest.day=day;
		updateDateField(date_field_id,dest,date_txt);
	}
}

Int16 MiscFunc::compareDates(DateType date1, DateType date2)
{
	return DateToDays(date1) - DateToDays(date2);
}										

Boolean MiscFunc::validNumberStr(const Char *str, Boolean allow_minus)
{
	// ignore trailing blanks
	while (*str==' ') str++;

	// ignore minus sign if it's allowed
	if (allow_minus && *str=='-') str++;

	while (*str) {
		if ((*str<'0' || *str>'9') && *str != cDecimalSep)
			return false;
		str++;
	}
	return true;
}

void MiscFunc::PopupListSetSelection(
				UInt16 listId, UInt16 trigId, Int16 sel_index)
{
    ListType *list;
    ControlType *trig;
	FormType *frm = FrmGetActiveForm();
    list = (ListType*)FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,listId));
    trig = (ControlType*)FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,trigId));
    LstSetSelection(list, sel_index);
    CtlSetLabel(trig, LstGetSelectionText(list, LstGetSelection(list)));
}
