/**	
	$Revision: 1.28 $
	Last updated: $Date: 2001/07/24 09:02:43 $

	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
	$Log: MiscFunc.h,v $
	Revision 1.28  2001/07/24 09:02:43  dizzy
	Renamed some #defines
	
	Revision 1.27  2001/07/15 06:48:44  chaese
	All of a sudden I got really weird crashes. It seems that the program got
	too big. Assigning the functions in MiscFunc to separate segments seems
	to fix the problem.
	
	Revision 1.26  2001/07/15 05:17:07  chaese
	* display only one balance (expected or cleared) in account table, popup
	  trigger lets user choose.
	* New function PopupListSetSelection to eliminate code duplication
	
	Revision 1.25  2001/07/09 02:29:39  chaese
	implement splits GUI & other small improvements along the way. The bulk of
	the splits GUI is complete, small data integrity checks are coming soon
	(need to prevent multiple splits for the same account within one transaction
	and stuff like that...)
	
	Revision 1.24  2001/06/29 16:17:26  dizzy
	Expanded Carsten's #define to handle strings and transformed it into a template fct. This should help tidying up packing operations a bit
	(provided it works as it should)
	
	Revision 1.23  2001/06/26 23:30:37  dizzy
	Forms are now classes rather then namespaces. This should enable future code optimisations
	
	Revision 1.22  2001/06/11 11:03:21  chaese
	Increased display speed in account and transaction tables:
	* Associate table rows with the record index rather than UID. (Minor gain)
	* Perform preference lookup at program startup rather than every time a
	date or number is drawn. (Major gain! That was really stupid of me to put
	the lookups into the drawing function in the first place!)
*/

#ifndef __MISC_FUNC__
#define __MISC_FUNC__

#include <PalmOS.h>

/// application creator
#define FREE_COINS_CREATOR	'DyCo' 

///version string for os 3.5
#define palmos_3_5 0x03503000 
///version string for os 3.1
#define palmos_3_1 0x03103000 

/// section definition for form code
#define FORM_SECTION __attribute__ (( section ("formsect")))

/// section definition for accounts database code
#define ACDB_SECTION __attribute__ (( section ("db1")))

/// contains various functions used throughout the program
namespace MiscFunc {
	///Number format: Thousands separator (currently unused)
	extern char cThousandSep;
	///Number format: Decimal separator
	extern char cDecimalSep;
	///date format, print day first?
	extern Boolean bDayFirst;
	///date format, character between day and month
	extern char cDateSep;

	/**
		checks if the PalmOS version on the current device is greater or equal to the one specified.
		for example if(osVerGe(palmos_3_5)) DoSomething();
	
		@param version_no the version number - pass it one of the variables defined above (palmos_*****)
		@return true if the os has a verno greater or equal to the one specified, false otherwise
	*/
	Boolean osVerGe(UInt32 version_no);
	
	/**
		Get a 32bit int containg a sum of money in pence and return a pointer
		to a proper formatted string.

		the pointer has to be deallocated after use with MemPtrFree

	*/
	Char* Pence2String(Int32 pence) FORM_SECTION;

	/**
		draw right aligned text
		@param s string to be drawn
		@param bounds rectangle inside which the text will be displayed
	*/
	void DrawCharsRightAlign(const Char* s, const RectangleType* bounds) FORM_SECTION;

	/**
		Get a string containing a sum of money and convert it to pence
		@param sum the sting containing the formatted amount
		@return	the amount of pence in the sum
	*/
	Int32 String2Pence(Char* sum) FORM_SECTION;

	/**
		returns a pointer to a given object in the current form
	*/
	void* GetObjectPtr(UInt16 objID) FORM_SECTION;

	/**
		returns a pointer to a given object in a given form
	*/
	void*	GetObjectPtrByForm(FormPtr frm,UInt16 objID) FORM_SECTION;

	/**
		returns pointer to the text in a given field in the current form
	*/
	void*	GetTextPtr(UInt16 objID) FORM_SECTION;

	FieldPtr SetFieldTextFromHandle(Int16 fieldID, MemHandle txtH);

	/// Allocates new handle and copies incoming string
	FieldPtr SetFieldTextFromStr(Int16 fieldID, const Char* str) FORM_SECTION;

	/// Allocates new handle and copies incoming integer
	FieldPtr SetFieldTextFromInt(Int16 fieldID, const Int32 i) FORM_SECTION;

	///clears the text from a field:
	void  ClearFieldText(Int16 fieldID) FORM_SECTION;

	/**
		fills in a date field (pushbutton or selectortrigger) with a date
		@param date_field_id ID of the field
		@param new_date date to be displayed
		@param date_txt pointer to an allocated buffer where the date text can be stored
	*/
	void updateDateField(UInt16 date_field_id, DateType new_date, Char* date_txt) FORM_SECTION;

	/**
		call the system form to choose a new date
		@param dest the choosen date will be stored in here
		@param date_field_id the date choosen will be displayed in this field
		@param date_txt pointer to be passed to updateDateField
	*/
	void selectDate(DateType& dest, UInt16 date_field_id, Char* date_txt) FORM_SECTION;
	
	/**
		compares two dates
		@param date1 1st date
		@param date2 2nd date
		@return -1 if date1<date2, 0 if date1==date2 and 1 otherwise
	*/
	Int16 compareDates(DateType date1, DateType date2) ACDB_SECTION;

	/**
		check if a string contains a valid decimal number
		@param str the string
		@param minus_allowed boolean that indicates if the number may be negative
		@return true if the string is valid, false otherwise
	*/
	Boolean validNumberStr(const Char *str, Boolean allow_minus) ACDB_SECTION;

	/**
		set list selection and update associated popup trigger label
		@param listId the ID of the list to set
		@param trigId the ID of the associated popup trigger
		@param sel_index index of the item to select
	*/
	void PopupListSetSelection(UInt16 listId, UInt16 trigId, Int16 sel_index) FORM_SECTION;

};

///export by default all the functions/vars in this namespace
using namespace MiscFunc;

#endif
