/**	
	$Revision: 1.8 $
	Last updated: $Date: 2001/07/24 01:00:04 $
	
	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: PreferencesForm.cpp,v $
	Revision 1.8  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.7  2001/07/16 02:15:57  chaese
	ChooseAccountForm now shows the whole list of accounts (rather than having
	the user navigate through the hierarchy levels) and preselects the current
	setting as specified by the caller.
	
	Revision 1.6  2001/07/15 06:53:59  chaese
	Fix: #441393. To make things less convoluted, the preferences store the
	transaction category without offset, and the offset is used for displaying
	in the preferences form only.
	
	Revision 1.5  2001/07/15 05:17:07  chaese
	* display only one balance (expected or cleared) in account table, popup
	  trigger lets user choose.
	* New function PopupListSetSelection to eliminate code duplication
	
	Revision 1.4  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.3  2001/06/27 03:15:12  chaese
	Resurrected category selector in main form as type group selector
	
	Revision 1.2  2001/06/26 23:30:37  dizzy
	Forms are now classes rather then namespaces. This should enable future code optimisations
	
	Revision 1.1  2001/06/25 10:42:35  dizzy
	Change file extensions
*/

#include "PreferencesForm.h"

#include "MiscFunc.h"
#include "CoinsPref.h"
#include "FreeCoinsRsc.h"
#include "TransactionsDB.h"
#include "AccountsDB.h"

Boolean PreferencesForm::handleEvent(EventPtr event)
{
	Boolean 	handled = true;
	UInt16 temp_view;

	CoinsPrefType* prefs = preferences.get();
	// Get the form pointer
	frm = FrmGetActiveForm();

	// Key Events
	switch(event->eType) {
	
	case frmUpdateEvent:
		if(choose_account_form->chosen) {
            choose_account_form->chosen = false;
			acc_uid = choose_account_form->chosen_account;					
			accounts_db->getFullNameTruncLeftToFit(acc_uid, 100, acc_name);
			CtlSetLabel((ControlType*) GetObjectPtrByForm(FrmGetActiveForm(),PREF_DEF_ACC),acc_name);
		}
		if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
		FrmDrawForm(frm);
        break;

	// Form Open
	case frmOpenEvent:		
		//draw form
		FrmSetControlValue(frm,FrmGetObjectIndex(frm,PREF_EXPECTED_COL_CBX),preferences.get()->disp_expected_bal);
		FrmSetControlValue(frm,FrmGetObjectIndex(frm,PREF_CLEARED_CBX),preferences.get()->disp_cleared_col);
		FrmSetControlValue(frm,FrmGetObjectIndex(frm,PREF_DEF_CLEARED_CBX),preferences.get()->trans_cleared);
		PopupListSetSelection(PREF_TRANS_VIEW_LIST, PREF_TRANS_VIEW_TRIGGER,
			(preferences.get()->remember_trans_view) ?
			0 : (preferences.get()->trans_view+1) );
	
		acc_uid = preferences.get()->def_acc;
		if(acc_uid == 0) {
			StrCopy(acc_name,"not set");
		} else {
			accounts_db->getFullNameTruncLeftToFit( acc_uid, 100, acc_name);
		}
		CtlSetLabel((ControlType*) GetObjectPtrByForm(FrmGetActiveForm(),PREF_DEF_ACC),acc_name);
				
		FrmDrawForm(frm);
		break;
	
	// Controls
	case ctlSelectEvent:
		switch( event->data.ctlSelect.controlID ) {
			case PREF_OK:
				prefs->disp_expected_bal=FrmGetControlValue(frm,FrmGetObjectIndex(frm,PREF_EXPECTED_COL_CBX));
				prefs->disp_cleared_col=FrmGetControlValue(frm,FrmGetObjectIndex(frm,PREF_CLEARED_CBX));
				prefs->trans_cleared = FrmGetControlValue(frm,FrmGetObjectIndex(frm,PREF_DEF_CLEARED_CBX));
				temp_view=LstGetSelection((ListPtr) GetObjectPtr(
					PREF_TRANS_VIEW_LIST));
				prefs->def_acc = acc_uid;

				if(!temp_view) {
					prefs->remember_trans_view=true;
				}
				else {
					prefs->remember_trans_view=false;
					prefs->trans_view = temp_view - 1;
				}
				
				//Preferences are permanently saved in StopApplication

				//update the main form
				handled = true;				
				FrmUpdateForm(ACCOUNTS_MAIN_FORM,0);
				FrmReturnToForm(ACCOUNTS_MAIN_FORM);
				break;
			
			case PREF_CANCEL:
				//cancel button was pressed;
				//simply return to the main form
				handled = true;
				FrmReturnToForm(ACCOUNTS_MAIN_FORM);
				break;
				
			case PREF_DEF_ACC:
				handled = true;
				choose_account_form->allow_no_selection = false;	
				choose_account_form->setCallerFormId(PREF_FORM);	
				choose_account_form->chosen = false;	
				choose_account_form->chosen_account = acc_uid;	
				
				FrmPopupForm(CHOOSE_ACCOUNT_FORM);	
				break;
			default:
				handled = false;
		};
		break;	
	// Everything else...
	default:
		handled = false;
	}	
	return handled;
}
