/**	
	$Revision: 1.9 $
	Last updated: $Date: 2001/07/24 09:02:43 $
	
	Copyright (C) 2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: SplitsDB.cpp,v $
	Revision 1.9  2001/07/24 09:02:43  dizzy
	Renamed some #defines
	
	Revision 1.8  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.7  2001/07/02 14:03:53  dizzy
	Horde of bugfixes to the transaction editing routines...
	
	Revision 1.6  2001/06/29 21:32:44  dizzy
	Eliminated bug discovered with os4 (releasing record which was not busy)
	
	Revision 1.5  2001/06/29 16:17:26  dizzy
	Expanded Carsten's #define to handle strings and transformed it into a template fct. This should help tidying up packing operations a bit
	(provided it works as it should)
	
	Revision 1.4  2001/06/29 00:24:57  chaese
	Rewrote how total balances are tracked. This should eliminate all problems
	that were associated with the old way (i.e. balances not being recalculated
	when they should have been etc.)
	
	Revision 1.3  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.2  2001/06/27 01:25:18  dizzy
	added purge transaction function
	
	Revision 1.1  2001/06/25 10:42:35  dizzy
	Change file extensions
	
	Revision 1.1  2001/06/10 04:05:41  chaese
	basic infrastructure for split transactions (somewhat tested, but not
	guaranteed to be bug free)
	CAUTION! Database format has changed! Expect nastiness when you run the
	new version on an old Database.
*/

#include <PalmOS.h>

#include "SplitsDB.h"
#include "AccountsDB.h"
#include "TransactionsDB.h"
#include "MiscFunc.h" //macro defs
#include "FreeCoinsRsc.h"

SplitsDB* splits_db;

void SplitsDB::unpackRecord (splitStructure* spl, packedSplitStructure* packed_spl)
{
	spl->trans_uid = packed_spl->trans_uid;
	spl->acc_uid = packed_spl->acc_uid;
	spl->amount = packed_spl->amount;
	spl->line_no = packed_spl->line_no;
}

void SplitsDB::packRecord (splitStructure* spl, MemHandle db_entry)
{
	UInt32	length = 0;
	MemPtr	s;
	UInt32	offset = 0;

	length = sizeof(packedSplitStructure);

	// resize the record if necessary and write the data
	if (MemHandleResize(db_entry, length) == 0) {
		s = MemHandleLock(db_entry);
		writeAndAdvance(s, offset, &spl->trans_uid);
		writeAndAdvance(s, offset, &spl->acc_uid);
		writeAndAdvance(s, offset, &spl->amount);
		writeAndAdvance(s, offset, &spl->line_no);
		MemHandleUnlock(db_entry);
	}
}

SplitsDB::SplitsDB() : Database <splitStructure, packedSplitStructure> (
	SPLITSDB_TYPE, FREE_COINS_CREATOR, SPLITSDB_NAME, SPLITS_CATEGORIES)
{
	current_cat=dmAllCategories;
}

UInt16 SplitsDB::locateFirstSplit(UInt32 trans_uid)
{
	packedSplitStructure s;

	// Nifty trick: I just use DmFindSortPosition to see where an entry
	// with line number 0 would be inserted, which uses binary search
	s.trans_uid = trans_uid;
	s.acc_uid = 0;
	s.amount = 0;
	s.line_no = 0;

	return DmFindSortPosition(ref, &s, NULL, (DmComparF*)compareRecords, 0);
}

void SplitsDB::clearRecord(UInt16 id)
{
	MemHandle h;
	splitStructure s;

	UInt16 cleared_cat;
	cleared_cat = findCategory("Cleared");
	if (getRecordCategory(id)==cleared_cat)
		return;
	setRecordCategory(id,cleared_cat); 
	h = getUnpackedRecord(id, &s);
	accounts_db->incrementBalance(s.acc_uid, s.amount, 0);
}

void SplitsDB::unclearRecord(UInt16 id)
{
	MemHandle h;
	splitStructure s;

	UInt16 uncleared_cat;
	uncleared_cat = findCategory("Uncleared");
	if (getRecordCategory(id)==uncleared_cat)
		return;
	setRecordCategory(id,uncleared_cat); 
	h = getUnpackedRecord(id, &s);
	accounts_db->incrementBalance(s.acc_uid, -s.amount, 0);
}

UInt16	SplitsDB::newRecord(splitStructure* s, Boolean cleared)
{
	MemHandle					h,p;
	UInt16						index=0;

	h=DmNewHandle(ref,1);
	ErrFatalDisplayIf(!ref,"Ref is 0");

	packRecord(s,h);
	ErrFatalDisplayIf(!h,"Cannot create new split. No space left?");
	
	p= (MemHandle) MemHandleLock(h);
	index=DmFindSortPosition(ref,p, 0, (DmComparF *) compareRecords, 0);
	MemHandleUnlock(h);
	
	DmAttachRecord(ref,&index,h,0);	

	selectRecord(index);
   if(!transactions_db->isScheduled(transactions_db->getRecordId(s->trans_uid))) { //if the transaction is not scheduled
   	//update the balance of the involved account
   	accounts_db->incrementBalance(s->acc_uid, cleared?s->amount:0, s->amount);
   }
	
	//set the category
	if(cleared) setRecordCategory(index,CategoryFind(ref,"Cleared"));
	else setRecordCategory(index,CategoryFind(ref,"Uncleared"));

	return index;
}

void SplitsDB::deleteRecord(UInt16 id, Boolean keep_balance)
{
	splitStructure		s;
	Boolean 			cleared;

	getUnpackedRecord(id,&s);

	cleared = (getRecordCategory(id)==CategoryFind(ref,"Cleared"));
				
	//update the balance of the involved account
	if(!keep_balance) 
		accounts_db->incrementBalance(s.acc_uid, cleared?-s.amount:0, -s.amount);

	Database<splitStructure, packedSplitStructure>::deleteRecord(id);	
}

Int16	SplitsDB::compareRecords(
	packedSplitStructure* rec1, packedSplitStructure* rec2, Int16 other,
	SortRecordInfoPtr sr1, SortRecordInfoPtr sr2, MemHandle app_info)
{
	if (rec1->trans_uid==rec2->trans_uid) {
		return rec1->line_no - rec2->line_no;
	}
	else return rec1->trans_uid - rec2->trans_uid;
}
