/**	
	Code to access the database containing the Splits information

	$Revision: 1.4 $
	Last updated: $Date: 2001/07/24 01:00:04 $
	
	Copyright (C) 2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
	$Log: SplitsDB.h,v $
	Revision 1.4  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.3  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.2  2001/06/27 01:25:18  dizzy
	added purge transaction function
	
	Revision 1.1  2001/06/10 04:05:41  chaese
	basic infrastructure for split transactions (somewhat tested, but not
	guaranteed to be bug free)
	CAUTION! Database format has changed! Expect nastiness when you run the
	new version on an old Database.
*/

#ifndef __SPLITS_DB__
#define __SPLITS_DB__

#include "Database.h"

/// type of the Splits database
#define SPLITSDB_TYPE	'DATA'
/// db name
#define SPLITSDB_NAME	"FreeCoins-Splits"

///the packed structure of a split - see the unpacked struct for explanations
typedef struct {
	UInt32		trans_uid;
	UInt32		acc_uid;
	UInt32		amount;
	UInt8		line_no;
} packedSplitStructure;

///unpacked structure of a Split
typedef struct {
	/// UID of the transaction that the split belongs to
    UInt32      trans_uid;
	/// UID of the affected account
    UInt32      acc_uid;
	/// signed amount by which the account is affected
    Int32      amount;
	/// line number for sorting purposes
    UInt8       line_no;
} splitStructure;

///class to manipulate the splits database
class SplitsDB : public Database <splitStructure, packedSplitStructure>
{
 public:
	/// unpacks a Split record
	void	unpackRecord (splitStructure* spl, packedSplitStructure* packed_spl);

	/// packs a Split record
	void	packRecord (splitStructure* spl, MemHandle db_entry);

	/// initialize the database
	SplitsDB();

	/** locate the first split that belongs to the given transaction
		@param trans_uid UID of the transaction
		@return the index of the first split within the transaction
	*/
	UInt16 locateFirstSplit(UInt32 trans_uid);
 
    /**
        set a record to cleared, update the affected account's cleared balance
        @param id id of the record to be set
    */
    void    clearRecord(UInt16 id);

    /**
        set a record to uncleared and update the affected account's cleared
		balance
        @param id id of the record to be set
    */
    void    unclearRecord(UInt16 id);

	/**
		creates a new record and update the balance of the involved account.
		The transaction that the split belongs to is responsible for keeping
		the number of splits it has up to date.
		@param s pointer to the structure containing the split data
		@param cleared indicates if this split affects the cleared balance
		@return id of the newly created record
	*/
	UInt16	newRecord(splitStructure* s, Boolean cleared);
	
	/**
		deletes a record, given its id. The balance of the affected account
		is updated.
		The transaction that the split belonged to is responsible for keeping
		the number of splits it has up to date.
		@param id the id of the record to be deleted
		@param keep_balance if true then the balances of the account involved in the split will *not* be changed. Use only when necessary, as this
		results in unbalanced accounts	
	*/
	void deleteRecord(UInt16 id, Boolean keep_balance = false);

	/**
		Compares two packed split structures.
		For parameters see the PalmOS reference
	*/
	static Int16	compareRecords(packedSplitStructure* rec1,packedSplitStructure* rec2,Int16 other,SortRecordInfoPtr sr1,SortRecordInfoPtr sr2,MemHandle app_info);
};

extern SplitsDB* splits_db;

#endif
