/**	
	$Revision: 1.5 $
	Last updated: $Date: 2001/07/24 01:00:04 $

	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
	$Log: SplitsForm.cpp,v $
	Revision 1.5  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.4  2001/07/15 11:38:12  dizzy
	Added interface for purging transactions
	
	Revision 1.3  2001/07/14 04:18:02  chaese
	Fix: #435302 and similar memory leaks due to improper frmClose handling
	
	Revision 1.2  2001/07/09 02:29:39  chaese
	implement splits GUI & other small improvements along the way. The bulk of
	the splits GUI is complete, small data integrity checks are coming soon
	(need to prevent multiple splits for the same account within one transaction
	and stuff like that...)
	
	Revision 1.1  2001/07/06 03:03:35  chaese
	First baby steps toward a GUI for split transactions. Viewing works,
	editing doesn't (yet).
	
*/

#include "SplitsForm.h"
#include "FreeCoinsRsc.h"
#include "EditSplitForm.h"

SplitsForm * SplitsForm::theForm = NULL;

Boolean SplitsForm::checkIndex(UInt8 index)
{
	if (index>=splits_count) {
		ErrFatalDisplay("end of splits array exceeded");
		return false;
	}
	else
		return true;
}

UInt16 SplitsForm::getCount(void)
{
	return splits_count;
}

void SplitsForm::setCount(UInt8 count)
{
	UInt8 i;
	Int32 amount;

	if (count==0) {
		MemHandleFree(splits_data);
		splits_data = NULL;
	}
	else {
		if (!splits_data)
			splits_data = MemHandleNew(2*count*sizeof(UInt32));
		else
			MemHandleResize(splits_data, 2*count*sizeof(UInt32));
	}
	if (count>splits_count) {
		UInt32 *data_ptr;
		data_ptr = (UInt32*)MemHandleLock(splits_data);
		MemSet(data_ptr + (2*splits_count),
			2*(count-splits_count)*sizeof(UInt32), 0);
		MemHandleUnlock(splits_data);
	}
	splits_count = count;
	total_debits = total_credits = 0;
	for (i = 0; i<count; i++) {
		amount = getAmount(i);
		if (amount > 0) total_debits += amount;
		else total_credits -= amount;
	}	
}

UInt32 SplitsForm::getAccount(UInt8 index)
{
	UInt32 *data_ptr, result;
	if (checkIndex(index)) {
		data_ptr = (UInt32*)MemHandleLock(splits_data);
		result = data_ptr[2*index];
		MemHandleUnlock(splits_data);
		return result;
	}
	return 0;
}

Int32 SplitsForm::getAmount(UInt8 index)
{
	Int32 *data_ptr, result;
	if (checkIndex(index)) {
		data_ptr = (Int32*)MemHandleLock(splits_data);
		result = data_ptr[2*index+1];
		MemHandleUnlock(splits_data);
		return result;
	}
	return 0;
}
	
void SplitsForm::setAccount(UInt8 index, UInt32 acc_uid)
{
	UInt32 *data_ptr;
	if (checkIndex(index)) {
		data_ptr = (UInt32*)MemHandleLock(splits_data);
		data_ptr[2*index] = acc_uid;
		MemHandleUnlock(splits_data);
	}
}
		
void SplitsForm::setAmount(UInt8 index, Int32 amount)
{
	Int32 *data_ptr, prev_amount;
	if (checkIndex(index)) {
		data_ptr = (Int32*)MemHandleLock(splits_data);
		prev_amount = data_ptr[2*index+1];
		if (prev_amount > 0) total_debits -= prev_amount;
		else total_credits += prev_amount;
		data_ptr[2*index+1] = amount;
		if (amount > 0) total_debits += amount;
		else total_credits -= amount;
		MemHandleUnlock(splits_data);
	}
}

void SplitsForm::deleteSplit(UInt8 index)
{
	UInt32 *data_ptr;
	if (checkIndex(index)) {
		data_ptr = (UInt32*)MemHandleLock(splits_data);
		MemMove(data_ptr+2*index, data_ptr+2*(index+1),
			2*sizeof(UInt32)*(splits_count-1-index));
		MemHandleUnlock(splits_data);
		setCount(splits_count-1);
	}
}

Boolean SplitsForm::eventHandler(EventPtr event)
{
	if (theForm) return theForm->handleEvent(event);
	else return false;
}

Boolean SplitsForm::handleEvent( EventPtr event )
{
	Boolean           handled = true;
	ControlPtr ptr;

	// Get the form pointer
	frm = FrmGetActiveForm();

	// Key Events
	switch(event->eType) {

	// Form Open
	case frmOpenEvent:
		splits_tbl.initTable();
		FrmUpdateForm(SPLITS_FORM, 0);
		break;

	case keyDownEvent:
		switch( event->data.keyDown.chr ) {
			case pageUpChr:			// Scroll up key pressed
				//link the hard button functionality to the soft up/down controls
				ptr = (ControlPtr) GetObjectPtr(SPLIT_PGUP);
				CtlHitControl(ptr);
				break;								
			case pageDownChr:		// Scroll down key pressed						
				ptr = (ControlPtr) GetObjectPtr(SPLIT_PGDN);
				CtlHitControl(ptr);
				break;
			default:
				handled = false;
		};
		break;
	
	case ctlSelectEvent:
		switch( event->data.ctlSelect.controlID ) {
			case SPLIT_DONE:
				FrmUpdateForm(EDIT_TRANS_FORM, 1);
				FrmReturnToForm(0);
				break;
			case SPLIT_NEW:
				EditSplitForm::theForm->split_index = 255;
				FrmPopupForm(EDIT_SPLIT_FORM);
				break;
            case SPLIT_PGUP:
                if(splits_tbl.pageUp()) FrmUpdateForm(SPLITS_FORM,1);
                break;
            case SPLIT_PGDN:
                if(splits_tbl.pageDown()) FrmUpdateForm(SPLITS_FORM,1);
                break;
			default:
				handled = false;
		}

		break;

	case tblSelectEvent:
		{
			UInt8 index;
			index = TblGetRowID(event->data.tblSelect.pTable,
				event->data.tblSelect.row);
			TblUnhighlightSelection(event->data.tblSelect.pTable);
			EditSplitForm::theForm->split_index = index;
			FrmPopupForm(EDIT_SPLIT_FORM);
		}
		break;
			
	// Form Update
	case frmUpdateEvent:
		splits_tbl.loadData();

		if(event->data.frmUpdate.updateCode == 0 ||
			event->data.frmUpdate.updateCode == frmRedrawUpdateCode) {

		   	if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
			FrmDrawForm(frm);
		}

        if(event->data.frmUpdate.updateCode & 1) {
            TblMarkTableInvalid((TablePtr)GetObjectPtr(SPLITS_TABLE));
            TblRedrawTable((TablePtr)GetObjectPtr(SPLITS_TABLE));
        }

		// draw totals
		{
			RectangleType rect;
			Char *s;
	        RctSetRectangle(&rect, 65, 123, 45, 11);
			s = Pence2String(total_debits);
			DrawCharsRightAlign(s, &rect);
			MemPtrFree(s);
	        RctSetRectangle(&rect, 110, 123, 45, 11);
			s = Pence2String(total_credits);
			DrawCharsRightAlign(s, &rect);
			MemPtrFree(s);
			WinDrawLine(0, 123, 156, 123);
			WinDrawLine(0, 133, 156, 133);
		}

		splits_tbl.showScrollArrows();
		break;
	
	// Everything else...
	default:
		handled = false;
	}	
	return handled;
}
