/**	
	$Revision: 1.4 $
	Last updated: $Date: 2001/07/24 01:00:04 $
	
	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: SplitsTable.cpp,v $
	Revision 1.4  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.3  2001/07/23 10:51:49  chaese
	code cleanup in loadData method
	
	Revision 1.2  2001/07/09 02:29:39  chaese
	implement splits GUI & other small improvements along the way. The bulk of
	the splits GUI is complete, small data integrity checks are coming soon
	(need to prevent multiple splits for the same account within one transaction
	and stuff like that...)
	
	Revision 1.1  2001/07/06 03:03:35  chaese
	First baby steps toward a GUI for split transactions. Viewing works,
	editing doesn't (yet).
	
*/

#define __SPLITS_TABLE_C__

#include "AccountsDB.h"
#include "SplitsTable.h"
#include "MiscFunc.h"
#include "SplitsForm.h"

#include "FreeCoinsRsc.h"

void SplitsTable::drawCell(void* tbl, Int16 row, Int16 column, RectangleType* bounds)
{
	RectangleType rect;
	UInt32 acc_uid;
	Char tmp_str[100];
	accountStructure ac;
	Int32 amount;
	Char *s;
	UInt8 index;
	SplitsForm *frm = SplitsForm::theForm;
	Boolean is_debit;

	index = TblGetRowID((TablePtr)tbl, row & 0xfe);
	acc_uid = frm->getAccount(index);
	amount = frm->getAmount(index);
	is_debit = (amount>0);
	if (!is_debit) amount *= -1;

	if ((row & 1)==0) {
		// draw account name
		accounts_db->getFullNameTruncLeftToFit(acc_uid,
			bounds->extent.x, tmp_str);
		WinDrawChars(tmp_str, StrLen(tmp_str),
			bounds->topLeft.x, bounds->topLeft.y);
	}
	else {
		// draw debit/credit description
		accounts_db->getUnpackedRecord(acc_uid, &ac);
		SysStringByIndex(is_debit?ACC_DEBIT_DESC:ACC_CREDIT_DESC,
			(UInt8)ac.acc_type, tmp_str, 100);
		WinDrawChars(tmp_str, StrLen(tmp_str),
			bounds->topLeft.x+11, bounds->topLeft.y);

		// draw amount	
		if (is_debit)
			RctSetRectangle(&rect,
				bounds->topLeft.x+64, bounds->topLeft.y, 45, 11);
		else
			RctSetRectangle(&rect,
				bounds->topLeft.x+109, bounds->topLeft.y, 45, 11);
		s = Pence2String(amount);
		DrawCharsRightAlign(s, &rect);
		MemPtrFree(s);

		WinDrawGrayLine(bounds->topLeft.x, bounds->topLeft.y+10,
			bounds->topLeft.x + bounds->extent.x, bounds->topLeft.y+10);
	}
}

Boolean SplitsTable::recordIsValid(UInt16 id)
{
	return true;
}

void SplitsTable::loadData()
{
	UInt16 row, i;
	SplitsForm *frm = SplitsForm::theForm;

	initColumns();
	row = 0;

	top_visible_item -= (top_visible_item % 2);

	for (i = 0; i<2*frm->getCount(); i++) {
		if (i<top_visible_item) continue;
		if (row>=rows) continue;
		TblSetRowID(table, row, i/2);
		TblSetRowUsable(table, row, true);
		row++;
	}
	setRowsUnusable(row);
	total_items = i;
}

SplitsTable::SplitsTable() : Table(SPLITS_TABLE, SPLIT_PGUP, SPLIT_PGDN)
{ }

void SplitsTable::initColumns()
{
	TblSetCustomDrawProcedure(table, 0, drawCell);

	//set style for all cells
	for (UInt16 row = 0; row < rows; row++) {
		TblSetItemStyle(table, row, 0, customTableItem);
	}
	
 	//set columns as usable
	TblSetColumnUsable(table, 0, true);
}
