/**	
	$Revision: 1.5 $
	Last updated: $Date: 2001/06/27 22:58:11 $
	
	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: Table.h,v $
	Revision 1.5  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.4  2000/12/12 00:17:28  dizzy
	Fixed a bug relate to displaying scroll arrows
	
	Revision 1.3  2000/12/11 16:27:25  dizzy
	C++ version of the code.		
*/


#ifndef __TABLE__
#define __TABLE__

#include <PalmOS.h>

/**
	A wrapper for the Table functions; its main function is to set a few standard functions
	which are to be overloaded in derived classes
*/
class Table
{
 protected:
	TablePtr	table;
	UInt32 		table_resource;
	UInt32		up_arrow;
	UInt32		down_arrow;

 public:
		 
	/// the id of the top visible item
	UInt16		top_visible_item;
	/// the max no of rows that can be displayed at once in this table
	UInt16		rows;
	/// no of elements that could be loaded in the table
	UInt16		total_items;

	/**
		initialises the table pointer. this code is not in the constructor because delayed initialisation
		might be desired (after the form was initialised)
		When finished this function calls the loadData method in orded to populate the table.
	*/
	void initTable();
	/**
		initialises the table
		@param table_resource id of the resource associated with this table
		@param up id of the resource associeted with the 'up' scroll arrow 
		@param down id of the resource associeted with the 'down' scroll arrow 
	*/
	Table(UInt32 table_resource, UInt32 up, UInt32 down);
	
	/// destructor: by default it hides the scroll arrows
	virtual ~Table();


	/**
		set part of the table rows to unusable
		@param start row from which all other rows will be unusable
	*/
	void setRowsUnusable(UInt16 start);

	///unhilights the curent selection
	void unhighlight();
	
	/**
		scroll a page up (if possible)
		@return true if the operation was executed, false otherwise
	*/
	Boolean pageUp();
	
	/**
		scroll a page down (if possible)
		@return true if the operation was executed, false otherwise
	*/
	Boolean pageDown();

	/**
		hide the scroll arrows
	*/
	void hideScrollArrows();
			
	/**
		displays scroll arrows for the table (if needed)
	*/
	void showScrollArrows();
	
	///initialises the columns
	virtual void initColumns()=0;

	///loads date into the table
	virtual void loadData()=0;
	
	/**
		this function should be used to check whether a record belongs in this table or not
		@param id the id of the record to be checked
		@return true if the record is valid and false otherwise
	*/
	virtual Boolean recordIsValid(UInt16 id)=0;	
};

#endif
