/**	
	$Revision: 1.18 $
	Last updated: $Date: 2001/07/24 09:02:43 $
	
	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: TransactionsDB.cpp,v $
	Revision 1.18  2001/07/24 09:02:43  dizzy
	Renamed some #defines
	
	Revision 1.17  2001/07/24 01:11:08  chaese
	FIX: deleting a scheduled transaction left orphaned splits behind
	
	Revision 1.16  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.15  2001/07/23 23:36:04  chaese
	Write scheduling info to database for scheduled transactions only
	
	Revision 1.14  2001/07/23 06:22:32  dizzy
	Scheduling bugfixes, Some modifications to the TransactionStructure
	
	Revision 1.13  2001/07/18 23:06:29  dizzy
	A few scheduling bugfixes. Plenty of issues left to resolve :-(
	
	Revision 1.12  2001/07/15 11:38:12  dizzy
	Added interface for purging transactions
	
	Revision 1.11  2001/07/15 06:53:59  chaese
	Fix: #441393. To make things less convoluted, the preferences store the
	transaction category without offset, and the offset is used for displaying
	in the preferences form only.
	
	Revision 1.10  2001/07/09 02:29:39  chaese
	implement splits GUI & other small improvements along the way. The bulk of
	the splits GUI is complete, small data integrity checks are coming soon
	(need to prevent multiple splits for the same account within one transaction
	and stuff like that...)
	
	Revision 1.9  2001/07/02 14:52:12  dizzy
	Splits are copied along with the transaction information when scheduled transactions are scheduled
	
	Revision 1.8  2001/07/02 14:03:53  dizzy
	Horde of bugfixes to the transaction editing routines...
	
	Revision 1.7  2001/07/02 11:03:04  dizzy
	Cleaned up TransactionsForm.
	Removed details button (tap will take user to details)
	Unbalanced transactions are marked
	The total in the Scheduled view is the total of all scheduled transactions (as opposed to the account balance)
	
	Revision 1.6  2001/07/01 12:21:55  dizzy
	Merged ScheduledDB into TransactionsDB
	
	Revision 1.5  2001/06/29 21:32:44  dizzy
	Eliminated bug discovered with os4 (releasing record which was not busy)
	
	Revision 1.4  2001/06/29 16:17:26  dizzy
	Expanded Carsten's #define to handle strings and transformed it into a template fct. This should help tidying up packing operations a bit
	(provided it works as it should)
	
	Revision 1.3  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.2  2001/06/27 01:25:18  dizzy
	added purge transaction function
	
	Revision 1.1  2001/06/25 10:42:35  dizzy
	Change file extensions
	
	Revision 1.30  2001/06/23 04:51:48  chaese
	fixed bug #435294 and minor cleanups
	
	Revision 1.29  2001/06/11 11:03:21  chaese
	Increased display speed in account and transaction tables:
	* Associate table rows with the record index rather than UID. (Minor gain)
	* Perform preference lookup at program startup rather than every time a
	date or number is drawn. (Major gain! That was really stupid of me to put
	the lookups into the drawing function in the first place!)
	
	Revision 1.28  2001/06/10 04:05:41  chaese
	basic infrastructure for split transactions (somewhat tested, but not
	guaranteed to be bug free)
	CAUTION! Database format has changed! Expect nastiness when you run the
	new version on an old Database.
	
	Revision 1.27  2001/06/06 00:09:52  chaese
	Bugfix and code cleanup regarding the adjustment of parent balances. This
	fixes the bugs that deleting a transaction and adjusting a balance didn't
	properly update the parent balances, and removes a bit of code duplication.
	
	Revision 1.26  2001/05/18 18:33:56  dizzy
	Fixed clearing problems
	
	Revision 1.25  2001/05/14 17:15:55  dizzy
	Parent accounts display total in children accounts when collapsed
	The program displays a message while creating the sample accounts on startup
	Quick transaction button (together with default account selection)
	Can choose default cleared state for transactions
	Fixed bug 407185 (adding scheduled transactions)
	
	Revision 1.24  2001/04/21 12:54:59  dizzy
	Added number field for transactions
	Added credit checkbox (the payments/withdrawals are display with inverted sign)
	
	Revision 1.23  2001/04/20 20:58:44  dizzy
	fixed bug 408435
	Fixed preferences initialisation problem (causing a memory write to an invalid location) when the program is run for the first time
	
	Revision 1.22  2001/03/06 19:43:33  dizzy
	Working scheduling code ;-)
	
	Revision 1.21  2001/03/05 20:06:28  dizzy
	Updates to the scheduling code
	
	Revision 1.20  2001/02/13 20:03:49  dizzy
	Fixed account editing problems.
	Income/Expense categories can be no longer edited (addresses bug #130736)
	
	Revision 1.19  2001/01/31 18:07:24  dizzy
	More updates to the scheduling code (still not working though :( )
	Moved all Form code in a separate code section
	
	Revision 1.18  2001/01/29 18:49:30  dizzy
	Put code for scheduled transactions in a separate code section to enable the linking process to succeed.
	Scheduled trans info can be add/deleted/updated
	Added cleared field to trans structure
	
	Revision 1.17  2001/01/23 20:40:11  dizzy
	Initial support for scheduled transactions.
	
	Revision 1.16  2000/12/13 16:37:34  dizzy
	Ability to display a column which indicates/modifies the cleared state for each transaction
		(it is on by default; it can be turned off from the preferences dialog)
	The default view in the transaction list can be selected from the preferences dialog
	
	Revision 1.15  2000/12/11 16:27:25  dizzy
	C++ version of the code.
*/

#include "TransactionsDB.h"

#include <DateTime.h>

#include "SplitsDB.h"
#include "MiscFunc.h" //macro defs
#include "CoinsPref.h"
#include "FreeCoinsRsc.h"

TransactionsDB* transactions_db;

Boolean	TransactionsDB::isCleared(UInt16 id)
{
	if(getRecordCategory(id)==findCategory("Cleared")) return true;
	else if(isScheduled(id)) return willBeCleared(id);
	return false;	
}

Boolean	TransactionsDB::isScheduled(UInt16 id)
{
	if(getRecordCategory(id)==findCategory("Scheduled")) return true;
	return false;	
}

Boolean	TransactionsDB::willBeCleared(UInt16 id)
{
	transactionStructure ts;	
	getUnpackedRecord(id,&ts);

	return ts.to_be_cleared;
}

void	TransactionsDB::clearRecord(UInt16 id)
{
	MemHandle				tr_h;
	transactionStructure 	ts;	
	UInt32 trans_uid;
	UInt16 index;
	UInt16 count;
	
	tr_h=getUnpackedRecord(id,&ts);	

	if(isScheduled(id)) {
		ts.to_be_cleared=true;
		packRecord(&ts,tr_h);
		return;
	}		

	trans_uid = getRecordUid(id);		
	setRecordCategory(id,findCategory("Cleared"));
	count = ts.num_splits;
	for (index=splits_db->locateFirstSplit(trans_uid);count>0;index++,count--){
		splits_db->clearRecord(index);
	}
}

void	TransactionsDB::unclearRecord(UInt16 id)
{
	MemHandle 				tr_h;
	transactionStructure	ts;	
    UInt32 trans_uid;
    UInt16 index;
    UInt16 count;

	tr_h=getUnpackedRecord(id,&ts);	
	
	if(getRecordCategory(id)==findCategory("Scheduled")) {
		ts.to_be_cleared=false;
		packRecord(&ts,tr_h);
		return;
	}		
	
	setRecordCategory(id,findCategory("Uncleared"));
    trans_uid = getRecordUid(id);
    count = ts.num_splits;
    for (index=splits_db->locateFirstSplit(trans_uid);count>0;index++,count--){
        splits_db->unclearRecord(index);
    }
}


Boolean TransactionsDB::isBalanced(UInt32 trans_uid)
{
    MemHandle               tr_h;
    transactionStructure    ts;
    splitStructure			   s;
    UInt16 index;
    UInt16 count;
	Int16 total_effect=0;

    tr_h=getUnpackedRecord(trans_uid,&ts);

    count = ts.num_splits;
    for (index=splits_db->locateFirstSplit(trans_uid);count>0;index++,count--){
		splits_db->getUnpackedRecord(index, &s);
		total_effect += s.amount;
    }                                                   
	return (total_effect==0);
}

Int32 TransactionsDB::getAmount(UInt32 trans_uid, UInt32 acc_uid)
{
    MemHandle               tr_h;
    transactionStructure    ts;
    splitStructure			   s;
    UInt16 index;
    UInt16 count;

    tr_h=getUnpackedRecord(trans_uid,&ts);

    count = ts.num_splits;
    for (index=splits_db->locateFirstSplit(trans_uid);count>0;index++,count--){
		splits_db->getUnpackedRecord(index, &s);
		if (s.acc_uid == acc_uid)
			return s.amount;
    }                                                   
	return 0;
}


UInt32 TransactionsDB::getPeerAcc(UInt32 trans_uid, UInt32 acc_uid)
{
    MemHandle               tr_h;
    transactionStructure    ts;
    splitStructure          s;
    UInt16 index;
    UInt16 count;

    tr_h=getUnpackedRecord(trans_uid,&ts);

    count = ts.num_splits;
	ErrFatalDisplayIf(count!=2, "getPeerAcc called for non-transfer");

    for (index=splits_db->locateFirstSplit(trans_uid);count>0;index++,count--){
        splits_db->getUnpackedRecord(index, &s);
        if (s.acc_uid != acc_uid)
            return s.acc_uid;
    }
	return 0;
}


void	TransactionsDB::getDate(UInt16 record, DateType * pDate)
{
	MemHandle					rech;
	packedTransactionStructure* pts;

	rech=DmQueryRecord(ref,record);
	ErrFatalDisplayIf(!rech, "rech is 0 - aborting");

	pts=(packedTransactionStructure*) MemHandleLock(rech);
	ErrFatalDisplayIf(!pts, "pts is 0 - aborting");

	pDate->month = pts->date.month;	
	pDate->day = pts->date.day;	
	pDate->year = pts->date.year;	

	MemHandleUnlock(rech);
}


void TransactionsDB::unpackRecord (transactionStructure* trans, packedTransactionStructure* packed_trans)
{
	Char* s = (Char*)packed_trans + sizeof(packedTransactionStructure);
	packedScheduleStructure *sched;
	
	trans->date = packed_trans->date;
	trans->num_splits = packed_trans->num_splits;
	trans->method = (payment_method) packed_trans->method;
	trans->is_scheduled = packed_trans->is_scheduled;

	MemSet(&(trans->sched), sizeof(scheduledStructure), 0);
	if (trans->is_scheduled) {
		sched = (packedScheduleStructure *)s;
		s += sizeof(packedScheduleStructure);
    	trans->sched.repeat_interval = sched->repeat_interval;
		trans->sched.repeat_on = sched->repeat_on;
		trans->sched.frequency = sched->frequency;
		trans->sched.end_by_date = sched->end_by_date;
		trans->sched.end_date = sched->end_date;	
		trans->sched.repeat_times = sched->repeat_times;
		trans->to_be_cleared = sched->to_be_cleared;
	}
	else {
		// this flag should be moved into the scheduledStructure
		trans->to_be_cleared = false;
	}

	trans->payee = s;
	s += StrLen(s) + 1;
	trans->num = s;
	s += StrLen(s) + 1;
	trans->note = s;
}

void TransactionsDB::packRecord (transactionStructure* trans, MemHandle db_entry)
{
	UInt32	length = 0;
	MemPtr	s;
	UInt32	offset = 0;
  	packedTransactionStructure pt;
	packedScheduleStructure ps;

	// figure out necessary size
	length = sizeof(packedTransactionStructure)
		+ StrLen(trans->payee) + StrLen(trans->num) 
		+ StrLen(trans->note) + 3 ;  // 3 string terminators

	pt.date = trans->date;
	pt.num_splits = trans->num_splits;
	pt.method = trans->method;
	pt.is_scheduled = trans->is_scheduled;
	pt.reserved = 0;

	if (trans->is_scheduled) {
		length += sizeof(packedScheduleStructure);
		ps.repeat_on = trans->sched.repeat_on;
		ps.frequency = trans->sched.frequency;
		ps.to_be_cleared = trans->to_be_cleared;
		ps.end_by_date = trans->sched.end_by_date;
		ps.end_date = trans->sched.end_date;
		ps.repeat_times = trans->sched.repeat_times;
		ps.repeat_interval = trans->sched.repeat_interval;
	}

	// resize the VoidHand
	if (MemHandleResize(db_entry, length) == 0) {
		// copy the fields
		s = MemHandleLock(db_entry);
		
		writeAndAdvance(s, offset, &pt);
		if (trans->is_scheduled) {
			writeAndAdvance(s, offset, &ps);
		}
		writeAndAdvance(s, offset, trans->payee);		
		writeAndAdvance(s, offset, trans->num);		
		writeAndAdvance(s, offset, trans->note);		
				
		MemHandleUnlock(db_entry);
	}
}

TransactionsDB::TransactionsDB() : Database <transactionStructure, packedTransactionStructure> (TRANSACTIONSDB_TYPE, FREE_COINS_CREATOR,
		TRANSACTIONSDB_NAME, TRANSACTIONS_CATEGORIES)
{
	current_cat=preferences.get()->trans_view;
}

UInt16	TransactionsDB::newRecord(transactionStructure* s, Boolean cleared, Boolean scheduled)
{
	MemHandle					h,p;
	UInt16						index=0;

	s->to_be_cleared = cleared;
	s->is_scheduled = scheduled;

	h=DmNewHandle(ref,1);
	ErrFatalDisplayIf(!ref,"Ref is 0");

	packRecord(s,h);
	ErrFatalDisplayIf(!h,"Cannot create new transaction. No space left?");
	
	p= (MemHandle) MemHandleLock(h);
	index=DmFindSortPosition(ref,p, 0, (DmComparF *) compareRecords, 0);
	//have to use Ptr and not handle, probably because FindSortPosition locks the handle
	MemPtrUnlock(p);
	
	DmAttachRecord(ref,&index,h,0);	

	selectRecord(index);

   if(scheduled) {
		setRecordCategory(index,CategoryFind(ref,"Scheduled"));		
      schedule();      
		return index;
	}
	
	//set the category
	if(cleared) setRecordCategory(index,CategoryFind(ref,"Cleared"));
	else setRecordCategory(index,CategoryFind(ref,"Uncleared"));

	return index;
}

void TransactionsDB::deleteRecord(UInt16 id, Boolean keep_balance)
{
	transactionStructure		t;
	Boolean 					cleared;
	UInt16 index, count;
	UInt32 trans_uid;

	if (isScheduled(id)) {
		//deleting a scheduled transaction shouldn't back out account balances
		keep_balance = true;
	}

	getUnpackedRecord(id,&t);

	cleared = (isCleared(id));
    trans_uid = getRecordUid(id);
    count = t.num_splits;
//  for (index=splits_db->locateFirstSplit(trans_uid);count>0;index++,count--){
    for (index=splits_db->locateFirstSplit(trans_uid);count>0;count--) {
			//delete splits while maintaing the requested balance policy
        splits_db->deleteRecord(index,keep_balance);
    }

	Database<transactionStructure, packedTransactionStructure>::deleteRecord(id);	
}

Int16	TransactionsDB::compareRecords(packedTransactionStructure* rec1,packedTransactionStructure* rec2,Int16 other,SortRecordInfoPtr sr1,SortRecordInfoPtr sr2,MemHandle app_info)
{
	return compareDates(rec2->date,rec1->date);
}	

void TransactionsDB::purgeTransactions(DateType last_date)
{	
	//transaction are sorted either ascending or descending by date; we will just go sequentially through all the database and remove the transactions
	//which have a date which satisfies the given conditions. Since this operation is only done once every (few) months it should not be a big problem	
	
	UInt16						 record_id;
	transactionStructure    ts;

  	for(record_id=0;record_id<DmNumRecords(ref);record_id++) {
		getUnpackedRecord(record_id,&ts);

		if(compareDates(ts.date,last_date) < 0 && ! isScheduled(record_id)) {
			//if the date is ok and record not scheduled, delete it
			deleteRecord(record_id, true);
            record_id--;
		}
	}
}

Boolean TransactionsDB::calculateNextDate(UInt16 id)
{
	transactionStructure	t;
	MemHandle				rec_h;
	DateType				   new_date;
	
   if(!isScheduled(id)) ErrFatalDisplay("Transaction is not scheduled. This should not happen.");
   
   rec_h=transactions_db->getUnpackedRecord(id,&t); // get the transaction record; current (next) date is stored in t.date
	
	switch (t.sched.repeat_interval) {
		case rep_day: 
			DateDaysToDate(DateToDays(t.date)+t.sched.frequency,&new_date);
			break;
		case rep_week:
			DateDaysToDate(DateToDays(t.date)+t.sched.frequency*7,&new_date);
			break;
		case rep_month:
			new_date.year=t.date.year+(t.date.month+t.sched.frequency) / 12;
			new_date.month=(t.date.month+t.sched.frequency) % 12;
			//if the day was initially the last in its month, then make next occurance to have the day as last in its month
			if(t.date.day == (UInt16) DaysInMonth(t.date.month,t.date.year)) 
				new_date.day=DaysInMonth(new_date.month, new_date.year);				
			else //if the new month has less days then the initial month, adjust 
				new_date.day=(t.date.day>(UInt16) DaysInMonth(new_date.month,new_date.year)) ? DaysInMonth(new_date.month,new_date.year) : t.date.day;
			break;
		case rep_year:
			new_date.month=t.date.month;
			new_date.year=t.date.year+t.sched.frequency;
			if(t.date.day == (UInt16) DaysInMonth(t.date.month,t.date.year)) 
				new_date.day=DaysInMonth(new_date.month, new_date.year);				
			else //if the new month has less days then the initial month, adjust 
				new_date.day=(t.date.day>(UInt16) DaysInMonth(new_date.month,new_date.year)) ? DaysInMonth(new_date.month,new_date.year) : t.date.day;
			break;
	}
   //record the date change
	t.date.day=new_date.day;
	t.date.month=new_date.month;
	t.date.year=new_date.year;
   
	//now save the date changes
	packRecord(&t,rec_h);
	    
	if( t.sched.end_by_date) {
        if(compareDates(t.sched.end_date,t.date) <=0 ) {
		    deleteRecord(id,true); //if the new date is past the specified end date, delete scheduled transaction; we need to maintain the balances
		    return true;
        }
        return false;
    }
	
    if(t.sched.repeat_times==-1) return false; //repeat forever; record is never deleted

	if( !t.sched.end_by_date ) { //if we stop after a given number of transactions
		if(t.sched.repeat_times>0) { // decrease the no of times we have left to repeat
			t.sched.repeat_times--;
			packRecord(&t,rec_h);
		} 
		if(t.sched.repeat_times==0) { //only makes sense to check this after a decrement
			deleteRecord(id,true); //if we have repeated the asked no of times, delete record
			return true;
		}
	}
	return false;
}

Int32      TransactionsDB::getScheduledTotal(UInt32 account_uid)
{
   UInt32                     total = 0;
   UInt16					      record_no;

  	for(record_no=0;DmQueryNextInCategory(ref,&record_no,CategoryFind(ref,"Scheduled"));record_no++) {
         total+=getAmount(getRecordUid(record_no), account_uid); //getAmount will simply return 0 if account not affected
   }
   return total;
}

Boolean TransactionsDB::schedule(Boolean force)
{   //this is very inneficient, but at least it works :-)
	DateType today;
	DateSecondsToDate(TimGetSeconds(),&today);
	
	if(!force) //if we are not forced to continue
		if(compareDates(preferences.get()->last_update,today)>=0) //and we don't need to update anything yet
			return false;
    
    Boolean rez=false;
    while(scheduleNextRecord()) { rez = true; }
	preferences.get()->last_update=today;
    return rez;
    
}

Boolean TransactionsDB::scheduleNextRecord()
{
	DateType today;
	DateSecondsToDate(TimGetSeconds(),&today);
	
	/*
	now we go through all the records and if we find one which has an associated transaction which is due
	today or on a previous date, we record that transaction and get its next date
	*/
	
	Boolean 			  updated=false;

  	for(UInt16 record_no=0;DmQueryNextInCategory(ref,&record_no,CategoryFind(ref,"Scheduled"));record_no++) {        
       	transactionStructure	temp;

		getUnpackedRecord(record_no,&temp); //get the next scheduled record
        
        //for each record repeat updating until there are no updates left to do
		if (compareDates(temp.date,today) <=0 ) {	//if record date is in the past or today
                UInt32 template_uid = getRecordUid(record_no);
                UInt16 count = temp.num_splits;
            
            //first create a new transaction record
            UInt32 new_uid = getRecordUid(newRecord(&temp,temp.to_be_cleared));
            for (UInt16 index=splits_db->locateFirstSplit(template_uid);count>0;count--,index++) {
              splitStructure              split;

                //cycle through the splits and duplicate each one of them
              splits_db->getUnpackedRecord(index,&split);
              split.trans_uid = new_uid; //make split point to our new transaction
              splits_db->newRecord(&split,temp.to_be_cleared);
            }
            updated=true;

            //now calculate the next date when the transaction is due
            calculateNextDate(record_no);
		}
	}
	
	return updated;
}
