/**	
	$Revision: 1.12 $
	Last updated: $Date: 2001/07/24 01:00:04 $

	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
	$Log: TransactionsForm.cpp,v $
	Revision 1.12  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.11  2001/07/23 10:54:09  chaese
	* make separate calls to showScrollArrows (which is not called by loadData
	  anymore)
	* enable partial update (table only) for actions that require only updating
	  the table and not the whole form.
	
	Revision 1.10  2001/07/15 06:53:59  chaese
	Fix: #441393. To make things less convoluted, the preferences store the
	transaction category without offset, and the offset is used for displaying
	in the preferences form only.
	
	Revision 1.9  2001/07/14 04:18:02  chaese
	Fix: #435302 and similar memory leaks due to improper frmClose handling
	
	Revision 1.8  2001/07/12 01:10:48  chaese
	reapply fix for bug #435607 that was lost by previous code cleanup
	
	Revision 1.7  2001/07/02 11:03:04  dizzy
	Cleaned up TransactionsForm.
	Removed details button (tap will take user to details)
	Unbalanced transactions are marked
	The total in the Scheduled view is the total of all scheduled transactions (as opposed to the account balance)
	
	Revision 1.6  2001/07/01 12:21:55  dizzy
	Merged ScheduledDB into TransactionsDB
	
	Revision 1.5  2001/06/30 01:17:05  chaese
	credit accounts are handled correctly now
	
	Revision 1.4  2001/06/29 00:24:57  chaese
	Rewrote how total balances are tracked. This should eliminate all problems
	that were associated with the old way (i.e. balances not being recalculated
	when they should have been etc.)
	
	Revision 1.3  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.2  2001/06/26 23:30:37  dizzy
	Forms are now classes rather then namespaces. This should enable future code optimisations
	
	Revision 1.1  2001/06/25 10:42:35  dizzy
	Change file extensions
	
	Revision 1.33  2001/06/23 04:51:48  chaese
	fixed bug #435294 and minor cleanups
	
	Revision 1.32  2001/06/22 23:23:29  chaese
	fix bug #435607
	
	Revision 1.31  2001/06/21 11:04:23  chaese
	* initial support for account types
	* fixed some obscure bugs that materialized on a PalmOS 3.5 debug rom
	(see related message on palmcoins-misc for details)
	
	Revision 1.30  2001/06/14 04:19:49  chaese
	corrected/optimized preferences handling
	
	Revision 1.29  2001/06/11 11:03:21  chaese
	Increased display speed in account and transaction tables:
	* Associate table rows with the record index rather than UID. (Minor gain)
	* Perform preference lookup at program startup rather than every time a
	date or number is drawn. (Major gain! That was really stupid of me to put
	the lookups into the drawing function in the first place!)
	
	Revision 1.28  2001/05/18 18:33:56  dizzy
	Fixed clearing problems
	
	Revision 1.27  2001/05/16 17:37:20  dizzy
	Fixed bug 424845 (deleting the default account)
	Fixed bug which prevented displaying last account under certain conditions
	Groovy new icons :) (large, small, b&w and colour)
	
	Revision 1.26  2001/05/14 17:15:55  dizzy
	Parent accounts display total in children accounts when collapsed
	The program displays a message while creating the sample accounts on startup
	Quick transaction button (together with default account selection)
	Can choose default cleared state for transactions
	Fixed bug 407185 (adding scheduled transactions)
*/

#include "TransactionsForm.h"

#include "CoinsPref.h"
#include "FreeCoinsRsc.h"

void	TransactionsForm::initCat()
{
	ControlPtr	ctl;	
	// Set the label of the category trigger.
	ctl = (ControlPtr) GetObjectPtr (TRANS_CAT_TRIGGER);
	CategoryGetName (transactions_db->getRef(), transactions_db->current_cat, cat_name);
	CategorySetTriggerLabel(ctl, cat_name);
}

void	TransactionsForm::selectCat()
{
 	UInt16		category=transactions_db->current_cat;
	Boolean   categoryEdited;

	//have to do this due to some odd behaviour 
	frm = FrmGetActiveForm();
	
	categoryEdited=CategorySelect(transactions_db->getRef(),frm, TRANS_CAT_TRIGGER,TRANS_CAT_LIST,false,&category,
		cat_name,4,(osVerGe(palmos_3_5)) ? categoryHideEditCategory:0);
	if(categoryEdited || category!=transactions_db->current_cat) {
		transactions_db->current_cat=category;
		if(preferences.get()->remember_trans_view) {
			preferences.get()->trans_view=category;
		}
		//start from the beginning each time the category is changed
		trans_tbl.top_visible_item=0;
		trans_tbl.hideScrollArrows();
	}
}

void	TransactionsForm::adjustAccBalance()
{
	accountStructure ac;
	Char	r[9];
	UInt16	rez;
	Int32 new_bal;

	rez=FrmCustomResponseAlert(ADJUST_BALANCE_ALERT,NULL,NULL,NULL,r,9,NULL);
	if (!rez) {
		new_bal = String2Pence(r);
		accounts_db->getUnpackedRecord(&ac);
		new_bal *= (ac.credit_account ? -1 : 1);	
		accounts_db->updateBalance(accounts_db->getSelectedUid(),new_bal);			
	}
}

void	TransactionsForm::displayTotals()
{
	Char* 				s;
	RectangleType		rp;
	Int32 factor;

	accountStructure	ac;
      	
   accounts_db->getUnpackedRecord(&ac);
	factor = ac.credit_account?-1:1;

   if(transactions_db->current_cat==transactions_db->findCategory("Scheduled")) {
      // have to display total of scheduled trans	
      factor *= transactions_db->getScheduledTotal(accounts_db->getSelectedUid());
   } else {
      factor *= ac.balance;
   }
   
	s=Pence2String(factor);
	
	RctSetRectangle(&rp,115,129,45,11);
	WinEraseRectangle(&rp,0);
	DrawCharsRightAlign(s, &rp);
	MemPtrFree(s);

   if(transactions_db->current_cat==transactions_db->findCategory("Scheduled")) {
      // have to display expected total of scheduled trans	... do nothing
   } else {
     	factor = ac.credit_account?-1:1;	
      factor *= ac.expected_balance;
   }
	s=Pence2String(factor);

	RctSetRectangle(&rp,70,129,45,11);
	WinEraseRectangle(&rp,0);
	DrawCharsRightAlign(s, &rp);
	MemPtrFree(s);	

	//add some lines at the bottom
	WinDrawLine(0,129,160,129);
	WinDrawLine(0,140,160,140);	
	
}

Boolean TransactionsForm::handleEvent( EventPtr event )
{
	Boolean           handled = true;
	ControlPtr         ptr;
	accountStructure    as;
	FormPtr                 f;

	// Get the form pointer
	frm = FrmGetActiveForm();

	// Key Events
	switch(event->eType) {

	// Form Open
	case frmOpenEvent:

		//set the form title to the name of the account
		accounts_db->getUnpackedRecord(&as);
		FrmSetTitle(frm,as.acname);
      
		if(preferences.get()->disp_payee) 
			FrmSetControlGroupSelection(frm,1,TRANS_OPT_PAYEE);
		else FrmSetControlGroupSelection(frm,1,TRANS_OPT_ACCOUNT);

		transactions_db->current_cat = preferences.get()->trans_view;				
		trans_tbl.initTable();
      transactions_db->unselectRecord();

		//display the category name
		initCat();
      
		//draw form
		if(!osVerGe(palmos_3_5)) FrmEraseForm(frm);
		FrmDrawForm(frm);
		trans_tbl.showScrollArrows();
		
		//display the running totals
		displayTotals();
		break;

	case keyDownEvent:
		switch( event->data.keyDown.chr ) {
			case pageUpChr:			// Scroll up key presed
				//link the hard button functionality to the soft up/down controls
				ptr = (ControlPtr) GetObjectPtr(TRANS_PGUP);
				CtlHitControl(ptr);
				break;								
			case pageDownChr:		// Scroll down key presed						
				ptr = (ControlPtr) GetObjectPtr(TRANS_PGDN);
				CtlHitControl(ptr);
				break;
			default:
				handled = false;
		};
		break;
	
	// Controls
	case ctlSelectEvent:
		switch( event->data.ctlSelect.controlID ) {
			case TRANS_NEW:
               handleNewTransaction();
               break;
			case TRANS_ACCTS:
				//cancel the current selection
		      transactions_db->unselectRecord();
				trans_tbl.hideScrollArrows();
				accounts_db->unselectRecord();
				FrmGotoForm(ACCOUNTS_MAIN_FORM);
				break;				
			case TRANS_CAT_TRIGGER:		//Select a category
				selectCat();
				FrmUpdateForm(TRANSACTIONS_FORM,0);
				break;
			case TRANS_OPT_PAYEE: //switch to payee vieew
				if(preferences.get()->disp_payee) break;
				else preferences.get()->disp_payee=true;
				FrmUpdateForm(TRANSACTIONS_FORM,1);
				break;
			case TRANS_OPT_ACCOUNT: //switch to account view
				if(!preferences.get()->disp_payee) break;
				else preferences.get()->disp_payee=false;
				FrmUpdateForm(TRANSACTIONS_FORM,1);
				break;
				
			case TRANS_PGUP:
				if(trans_tbl.pageUp()) FrmUpdateForm(TRANSACTIONS_FORM,1);
				break;								
			case TRANS_PGDN:		// Scroll down control presed?	
				if(trans_tbl.pageDown()) FrmUpdateForm(TRANSACTIONS_FORM,1);
				break;								
				
			default:
				handled = false;
		}
		break;
	case tblSelectEvent:		
		transactions_db->selectRecord(TblGetRowID((TablePtr) GetObjectPtr(TRANS_TABLE),event->data.tblSelect.row));
		if(event->data.tblSelect.column == TransactionsTable::cleared_col) {
			Boolean clr = TblGetItemInt((TablePtr) GetObjectPtr(TRANS_TABLE),event->data.tblSelect.row,event->data.tblSelect.column);
			if(clr && !transactions_db->isCleared(transactions_db->getSelectedRecord())) {
				transactions_db->clearRecord(transactions_db->getSelectedRecord());
		   } else 
				if(!clr && transactions_db->isCleared(transactions_db->getSelectedRecord())) {
					transactions_db->unclearRecord(transactions_db->getSelectedRecord());
				} else  //exit witouth updating the database
						break;
			//we don't need to update the form in the cl/uncl category
			if(transactions_db->current_cat!=transactions_db->findCategory("Cl/Uncl") && transactions_db->current_cat!=transactions_db->findCategory("Scheduled"))		
				FrmUpdateForm(TRANSACTIONS_FORM,0);		
			else displayTotals();
			transactions_db->unselectRecord();
		} else {
         handleTransactionDetails();
      }
		break;
			
	// Menus
	case menuEvent:
		switch (event->data.menu.itemID) {
			case TRANS_MENU_NEW:
               handleNewTransaction();
					break;					
			case TRANS_MENU_ADJST_BAL:
				adjustAccBalance();
				FrmUpdateForm(TRANSACTIONS_FORM,0);
				//cancel the current selection
		      transactions_db->unselectRecord();
				break;		
			case TRANS_MENU_ACC_DETAILS:
				edit_account_form->setRecordIsNew(false);
				if(!osVerGe(palmos_3_5)) FrmEraseForm(frm);
				FrmPopupForm(EDIT_ACC_FORM);
				FrmUpdateForm(TRANSACTIONS_FORM,0);
				//cancel the current selection
            transactions_db->unselectRecord();
			if(! accounts_db->isRecordSelected()) {
					FrmUpdateForm(ACCOUNTS_MAIN_FORM,0);
					FrmGotoForm(ACCOUNTS_MAIN_FORM);
				}
				break;		
						
			case TRANS_MENU_ABOUT:
			    f = FrmInitForm(ABOUT_FORM);
    
				FrmSetActiveForm(f);
				FrmDoDialog(f);
			    if (frm) {
               FrmSetActiveForm(frm);
               FrmDeleteForm(f);
				}
				break;
			case TRANS_MENU_HELP:
				FrmHelp(TRANS_HELP);
				break;
				
			default:
				handled = false;
		}
		break;
	
	// Form Close
	case frmCloseEvent:
		trans_tbl.hideScrollArrows();
		FrmEraseForm(frm);
		FrmDeleteForm(frm);
		break;
	
	// Form Update
	case frmUpdateEvent:
		trans_tbl.loadData();

		if(event->data.frmUpdate.updateCode == 0 ||
			event->data.frmUpdate.updateCode == frmRedrawUpdateCode) {
			accounts_db->getUnpackedRecord(&as);
			FrmSetTitle(frm,as.acname);
		
			//display the category name
			initCat();
		
			//draw form
			if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
			FrmDrawForm(frm);
			//display the running totals
			displayTotals();
		}
		if(event->data.frmUpdate.updateCode & 1) {
			TblMarkTableInvalid((TablePtr)GetObjectPtr(TRANS_TABLE));
			TblRedrawTable((TablePtr)GetObjectPtr(TRANS_TABLE));
		}			
		trans_tbl.showScrollArrows();
		break;
	
	// Everything else...
	default:
		handled = false;
	}	
	return handled;
}


void TransactionsForm::handleTransactionDetails()
{
   edit_trans_form->setRecordIsNew(false);
   edit_trans_form->setCallerFormId(TRANSACTIONS_FORM);					
	// Please do not touch this line, it fixes bug #435607
	trans_tbl.unhighlight();
   FrmPopupForm(EDIT_TRANS_FORM);		
}                  

void TransactionsForm::handleNewTransaction()
{
    transactions_db->unselectRecord();
   edit_trans_form->setRecordIsNew(true);
   edit_trans_form->setCallerFormId(TRANSACTIONS_FORM);					
   FrmPopupForm(EDIT_TRANS_FORM);				
}            
