/**	
	$Revision: 1.9 $
	Last updated: $Date: 2001/07/02 11:03:04 $

	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
	$Log: TransactionsForm.h,v $
	Revision 1.9  2001/07/02 11:03:04  dizzy
	Cleaned up TransactionsForm.
	Removed details button (tap will take user to details)
	Unbalanced transactions are marked
	The total in the Scheduled view is the total of all scheduled transactions (as opposed to the account balance)
	
	Revision 1.8  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.7  2001/06/26 23:30:37  dizzy
	Forms are now classes rather then namespaces. This should enable future code optimisations
	
	Revision 1.6  2001/01/31 18:07:24  dizzy
	More updates to the scheduling code (still not working though :( )
	Moved all Form code in a separate code section
	
	Revision 1.5  2001/01/26 18:44:38  dizzy
	The date is now changed/choosen by a selector trigger (as opposed to a button)
	Moved Cleared column on the rhs of the screen
	TODO renamed to todo.html
	Account details can be accessed from the transactions screen menu
	Tap on account name will now take the user to the transactions screen (there are some problems with account deletion :( )
	Tap on expected balance column will take the user to scheduled transactions list
	
	Revision 1.4  2000/12/11 16:27:25  dizzy
	C++ version of the code.
	
	Revision 1.3  2000/11/16 02:38:32  chaese
	Merged New Transaction and Transaction Details forms
	
	Revision 1.2  2000/10/12 17:17:24  dizzy
	Email address update; lots of bugfixes
*/

#ifndef __TRANSACTIONS_FORM__
#define __TRANSACTIONS_FORM__

#include <PalmOS.h>

#include "Form.h"

#include "TransactionsTable.h"
#include "EditAccountForm.h"
#include "EditTransactionForm.h"

/**
   functions and vars that manipulate the form which deals with transactions
   \note the duplicate menu and button events are handled by handle* functions do eliminate code duplication
*/
class TransactionsForm : public Form
{	
 private:
   //pointer to the active edit account form
	EditAccountForm* edit_account_form;
   //pointer to the active edit  transaction form
	EditTransactionForm* edit_trans_form;  
   ///used to store the current category name 
	Char cat_name [dmCategoryLength];
	
	///the transactions table
	TransactionsTable trans_tbl;

 public:
	TransactionsForm(EditAccountForm* a, EditTransactionForm* t) 
		 : edit_account_form(a), edit_trans_form(t) {}
 	virtual ~TransactionsForm () {}
	
	void	adjustAccBalance() FORM_SECTION;	
	
	/**
		updates the totals for the current account (if necessary) and draws them on the screen
	*/
	void	displayTotals() FORM_SECTION;

	/**
		Generic handler for the application's static form(s)
		@return true if the event was handled, false otherwise
	*/
	Boolean handleEvent(EventPtr event) FORM_SECTION;

	/// init the category trigger
	void	initCat() FORM_SECTION;

	///handle category selection
	void	selectCat() FORM_SECTION;
   
   /**
      executes the necessary operations to start creating a new transaction
   */
   void handleNewTransaction();
         
   /**
      handles transaction details
   */
   void handleTransactionDetails();
         
};

#endif
