/**	
	$Revision: 1.7 $
	Last updated: $Date: 2001/07/24 01:00:04 $
	
	Copyright (C) 2000-2001 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: TransactionsTable.cpp,v $
	Revision 1.7  2001/07/24 01:00:04  chaese
	eliminated references to PalmCoins except in situations that require
	renaming CVS controlled files (or other instances that I've overlooked :)
	
	Revision 1.6  2001/07/23 10:51:49  chaese
	code cleanup in loadData method
	
	Revision 1.5  2001/07/12 03:58:44  chaese
	visual improvement: Moved mark for unbalanced trans. down a bit
	
	Revision 1.4  2001/07/09 02:29:39  chaese
	implement splits GUI & other small improvements along the way. The bulk of
	the splits GUI is complete, small data integrity checks are coming soon
	(need to prevent multiple splits for the same account within one transaction
	and stuff like that...)
	
	Revision 1.3  2001/07/02 11:03:04  dizzy
	Cleaned up TransactionsForm.
	Removed details button (tap will take user to details)
	Unbalanced transactions are marked
	The total in the Scheduled view is the total of all scheduled transactions (as opposed to the account balance)
	
	Revision 1.2  2001/06/27 22:58:11  dizzy
	Restructured #include directives
	
	Revision 1.1  2001/06/25 10:42:35  dizzy
	Change file extensions
	
	Revision 1.28  2001/06/17 05:23:32  chaese
	*** CAUTION: DATABASE STRUCTURE HAS CHANGED YET AGAIN ***
	I generally don't like 'cross-logging', but too many changes have accumulated
	to keep them separated.
	1. completely eliminated account categories
	2. introduced currently unused type field in the account record structure
	(This will become what the user sees as 'category selector'. Coming soon!)
	3. completely eliminated protected accounts (mostly because #1 made it so much
	harder to locate the "Misc" account in the Expense category:) Those should be
	replaced with Pref. settings for default Income and Expense accounts anyway.
	4. changes to user interface; some as mere consequences of 1. (e.g.
	Account chooser), some for usability reasons (Transaction form).
	5. bugfixes and minor improvements.
	
	Revision 1.27  2001/06/14 00:52:05  chaese
	show account names with as much of their full path name as possible
	
	Revision 1.26  2001/06/12 03:28:35  chaese
	Shaved off code in date and currency formatting (amazing what you can do
	with StrPrintF! :)
	
	Revision 1.25  2001/06/11 11:03:21  chaese
	Increased display speed in account and transaction tables:
	* Associate table rows with the record index rather than UID. (Minor gain)
	* Perform preference lookup at program startup rather than every time a
	date or number is drawn. (Major gain! That was really stupid of me to put
	the lookups into the drawing function in the first place!)
	
	Revision 1.24  2001/06/10 04:05:41  chaese
	basic infrastructure for split transactions (somewhat tested, but not
	guaranteed to be bug free)
	CAUTION! Database format has changed! Expect nastiness when you run the
	new version on an old Database.
	
	Revision 1.23  2001/05/16 17:37:20  dizzy
	Fixed bug 424845 (deleting the default account)
	Fixed bug which prevented displaying last account under certain conditions
	Groovy new icons :) (large, small, b&w and colour)
	
	Revision 1.22  2001/04/25 21:10:30  dizzy
	Account names are clipped if they don't fit in the column
	Collapsable account tree
	
	Revision 1.21  2001/04/21 12:54:59  dizzy
	Added number field for transactions
	Added credit checkbox (the payments/withdrawals are display with inverted sign)
	
	Revision 1.20  2001/04/20 20:58:44  dizzy
	fixed bug 408435
	Fixed preferences initialisation problem (causing a memory write to an invalid location) when the program is run for the first time
	
	Revision 1.19  2001/03/06 19:43:33  dizzy
	Working scheduling code ;-)
	
	Revision 1.18  2001/01/26 18:44:38  dizzy
	The date is now changed/choosen by a selector trigger (as opposed to a button)
	Moved Cleared column on the rhs of the screen
	TODO renamed to todo.html
	Account details can be accessed from the transactions screen menu
	Tap on account name will now take the user to the transactions screen (there are some problems with account deletion :( )
	Tap on expected balance column will take the user to scheduled transactions list
	
	Revision 1.17  2000/12/13 16:37:34  dizzy
	Ability to display a column which indicates/modifies the cleared state for each transaction
		(it is on by default; it can be turned off from the preferences dialog)
	The default view in the transaction list can be selected from the preferences dialog
	
	Revision 1.16  2000/12/13 13:46:04  dizzy
	Fixed bug which caused a crash each time a transaction was updated
	PalmOS < 3.5 compatibility code (now the program seems to run on PalmOS 3.1)
	
	Revision 1.14  2000/12/12 00:17:28  dizzy
	Fixed a bug relate to displaying scroll arrows
	
	Revision 1.13  2000/12/11 16:27:25  dizzy
	C++ version of the code.
	
	Revision 1.12  2000/12/03 13:05:12  dizzy	
*/

#define __TRANSACTIONS_TABLE_C__

#include "TransactionsTable.h"

#include "TransactionsDB.h"
#include "AccountsDB.h"
#include "MiscFunc.h"
#include "CoinsPref.h"

#include "FreeCoinsRsc.h"

void TransactionsTable::drawCurrency(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	Char					*s;
	accountStructure 		ac;
	UInt32 trans_uid, acc_uid;
	Int32 amount;

	accounts_db->getUnpackedRecord(&ac);
	trans_uid = transactions_db->getRecordUid(TblGetRowID((TablePtr)table,row));
	acc_uid = accounts_db->getSelectedUid();

	amount = transactions_db->getAmount(trans_uid, acc_uid);
	if (ac.credit_account) amount *= -1;
	
	s=Pence2String(amount);

	DrawCharsRightAlign(s, bounds);
	//free memory allocated by Pence2String
	MemPtrFree(s);	

   // mark the transaction if it is unbalanced
   if(!transactions_db->isBalanced(trans_uid)) {
		MemHandle res_h = DmGetResource('Tbmp', BMP_UNBALANCED);
		BitmapType *bitmap = (BitmapType *) MemHandleLock(res_h);
		WinDrawBitmap(bitmap, bounds->topLeft.x, bounds->topLeft.y+2);
		MemHandleUnlock(res_h);
		DmReleaseResource(res_h);
   }
}

void TransactionsTable::drawDate(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	Char		s[6];
	DateType d;
	UInt8 num1, num2;

	transactions_db->getDate(TblGetRowID((TablePtr) table,row), &d);
	if (MiscFunc::bDayFirst) {
		num1 = d.day; num2 = d.month;
	} else {
		num2 = d.day; num1 = d.month;
	}
	StrPrintF(s, "%02d%c%02d", num1, MiscFunc::cDateSep, num2);

	WinDrawChars(s,5,bounds->topLeft.x,bounds->topLeft.y);
}


void TransactionsTable::drawAcc(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	transactionStructure 	ts;
	UInt32 trans_uid, peer_uid;
	Char buf[50];
	char *str;

	transactions_db->getUnpackedRecord(TblGetRowID((TablePtr) table,row),&ts);
	
	if(preferences.get()->disp_payee) // display the payee
		str = ts.payee;
	else { //display the peer account
		switch (ts.num_splits) {
			case 1: str = "Not set"; break;
			case 2:
				trans_uid = transactions_db->getRecordUid(TblGetRowID((TablePtr) table,row));
				peer_uid = transactions_db->getPeerAcc(
					trans_uid, accounts_db->getSelectedUid());
				accounts_db->getFullNameTruncLeftToFit(peer_uid,
					TblGetColumnWidth((TablePtr) table, dest_col), buf);
				str = buf;
				break;
			default: str = "--Split--"; break;
		}
	}	
	WinDrawChars(str,StrLen(str),bounds->topLeft.x,bounds->topLeft.y);
}

Boolean TransactionsTable::recordIsValid(UInt16 id)
{
	UInt32 trans_uid;
	
	//check the category
	if(transactions_db->current_cat==transactions_db->findCategory("Cl/Uncl") && ( \
		transactions_db->getRecordCategory(id)!=transactions_db->findCategory("Cleared") && \
		transactions_db->getRecordCategory(id)!=transactions_db->findCategory("Uncleared"))) return false; 
	//check accounts
	trans_uid = transactions_db->getRecordUid(id);
	if(transactions_db->getAmount(trans_uid, accounts_db->getSelectedUid())==0)
		return false;
	return true;
}

void TransactionsTable::loadData()
{
	UInt16 	recordNo,row,i;
	UInt16	categ;

	row = 0;
	i = 0;

	initColumns();
	
	categ=transactions_db->current_cat;
	if (categ==transactions_db->findCategory("Cl/Uncl"))
		categ = dmAllCategories;
	for (recordNo=0;
		DmQueryNextInCategory(transactions_db->getRef(),
		&recordNo,categ); recordNo++)
	{
		if (!recordIsValid(recordNo)) continue;
		if (i++ < top_visible_item) continue;
		if (row>=rows) continue;
		TblSetRowID(table, row, recordNo);
		TblSetItemInt(table, row, cleared_col,
			transactions_db->isCleared(recordNo));
		TblSetRowUsable(table, row, true);
		row++;
	}
	setRowsUnusable(row);
	total_items = i;
}

TransactionsTable::TransactionsTable() : Table(TRANS_TABLE, TRANS_PGUP, TRANS_PGDN)
{ }

void TransactionsTable::initColumns()
{
	//the numbers are drawn with our custom procedure
	TblSetCustomDrawProcedure(table, date_col, drawDate);
	TblSetCustomDrawProcedure(table, dest_col, drawAcc);
	TblSetCustomDrawProcedure(table, amount_col, drawCurrency);

	//set style for all cells
	for (UInt16 row = 0; row < rows; row++) {
		TblSetItemStyle(table, row, date_col, customTableItem);
		TblSetItemStyle(table, row, dest_col, customTableItem);
		TblSetItemStyle(table, row, amount_col, customTableItem);
	}
	
 	//set columns as usable
	TblSetColumnUsable(table, date_col, true);
	
	//display the cleared column if needed
	if(preferences.get()->disp_cleared_col) {
		TblSetColumnUsable(table, cleared_col,true);
	} else {
		//dont display the column
		TblSetColumnUsable(table, cleared_col,false);
		TblSetColumnWidth(table,dest_col,87);
	}
	//display the amount and dest column
 	TblSetColumnUsable(table, dest_col, true);	
	TblSetColumnUsable(table, amount_col, true);	
}
